/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.KeyStroke;
import objets.CListeObjets;
import objets.CMathGrafDoc;
import types.DimensionFenetre;
import entreesSorties.ImageSelection;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class ActionEditionCopier extends MtgAction {
		public ActionEditionCopier(MtgFrame frameInit) {
		super(frameInit, null, getString("chaineMenuEditionCopier"), getString("chaineMenuEditionCopier"),
				iconeAssociee(), KeyStroke.getKeyStroke("control C"));
	}
		
	@Override // Ajout version 4.3
  public boolean estDansBarreHorizontale() {
  	return true;
  }	

	// Modifi version 4.7.0.3 pour bug avec CEditeur et image de fond
	@Override
	public void actionPerformed(ActionEvent e) {
		MtgInternalFrame cadre = frame.cadreActif();
		if (cadre != null) {
			CMathGrafDoc doc = cadre.document;
			// Pour que l'image copie soit de meilleure qualit on cre une nouvelle image
			// font les dimensions sont celles du gbuffer multiplies par 2 dans les deux dimensions
			// int width = cadre.dimf.longueurAbscisses*2; // Modifi version 4.6
			// int height = cadre.dimf.longueurOrdonnees*2; // Modifi version 4.6
			int width;
			int height;
			BufferedImage image;
			try {
				width = (int) (cadre.dimf.longueurAbscisses*frame.pref_ClipboardImagesRatio);
				height = (int) (cadre.dimf.longueurOrdonnees*frame.pref_ClipboardImagesRatio);
				image = new BufferedImage(width, height, BufferedImage.TYPE_4BYTE_ABGR);
			}
			catch (Exception er) {
    		frame.dialogueAvertissement(getString("ch71"));
    		// Il est probable que c'est le coeffcient choisi dans Options - Prfrences qui est trop grand.
    		// On le rinitialise
    		frame.pref_ClipboardImagesRatio = 2;
    		return;
			}
			// Image image = cadre.createImage(width, height);
			Graphics2D g2D = (Graphics2D) image.getGraphics();
			g2D.setColor(cadre.couleurFond); // Ncessaire car peut tre appel en dehors
			// de paint
			g2D.fillRect(0, 0, width, height);
	    // AffineTransform oldat = g2D.getTransform();
	    // Le 0.2 car l'image est deux fois plus grande que l'original dans chacune des dimensions
	    // AffineTransform at = new AffineTransform(0.2, 0, 0, 0.2, 0, 0); // Modifi version 4.6
	    // double coef = frame.pref_ClipboardImagesRatio/10;
	    // AffineTransform at = new AffineTransform(coef, 0, 0, coef, 0, 0);
	    // g2D.setTransform(at);
			Image imageFond = cadre.document.imageFond;
			if (cadre.document.modeTraceActive) g2D.drawImage(cadre.imageTraces, 0, 0,
					(int)(frame.pref_ClipboardImagesRatio*cadre.dimf.longueurAbscisses), (int)(frame.pref_ClipboardImagesRatio*cadre.dimf.longueurOrdonnees),
					cadre.couleurFond, null);
			else {
				if ((imageFond != null) && cadre.document.imageFondVisible) {
					g2D.drawImage(imageFond, 0, 0, (int)(frame.pref_ClipboardImagesRatio*imageFond.getWidth(null)), 
							(int)(frame.pref_ClipboardImagesRatio*imageFond.getHeight(null)), cadre.couleurFond,  null);
				}
			}
			
			// On change la rsolution. Pour cela on fait avant une sauvegarde de la figure dans un
			// flux afin de la recharger ensuite dans le mme tat
	    try {
				ByteArrayOutputStream sauvegarde = new ByteArrayOutputStream();
				MathGraphDataOutputStream oups = new MathGraphDataOutputStream(sauvegarde);
		    cadre.listePr.write(oups);
		    oups.close();
		    CListeObjets listeCopie = new CListeObjets(cadre.listePr);
		    listeCopie.bCreationComposants = false;
		    ByteArrayInputStream ba = new ByteArrayInputStream(sauvegarde.toByteArray());
		    MathGraphDataInputStream inps = new MathGraphDataInputStream(ba);
		    listeCopie.read(inps);
		    inps.close();
		    
		    DimensionFenetre dim = new DimensionFenetre((int) (frame.pref_ClipboardImagesRatio*cadre.dimf.longueurAbscisses),
		    		(int) (frame.pref_ClipboardImagesRatio*cadre.dimf.longueurOrdonnees));
		    listeCopie.basseDefinition = false;
		    listeCopie.adaptePourHauteResolution(frame.pref_ClipboardImagesRatio);
		    listeCopie.bAttenuationEpaisseurTrait = frame.pref_attenuation;
		    listeCopie.positionne(cadre.infRand, dim);
				
		    listeCopie.afficheTout(g2D, dim, cadre.masquageObjetsCaches, doc.couleurFond, doc.transparencePourSurfaces, frame.pref_antialiasing);
		    listeCopie.retireTout();
		    listeCopie = null; // Pour le garbageCollector
		    // Ajout version 4.7.0.3
	    }
	    catch (IOException err) {
	    }			

      ImageSelection imgSel = new ImageSelection(image);
      Toolkit toolKit = Toolkit.getDefaultToolkit();
      Clipboard cb = toolKit.getSystemClipboard();
      // Il y a un gros lag lors du premier appel. On affiche un sablier
      Cursor cursor = frame.getCursor();
      frame.setCursor(MtgFrame.waitCursor);
      cb.setContents(imgSel, null);
      frame.setCursor(cursor);
		}
	}
	
  public static ImageIcon iconeAssociee() {
    URL adr = MtgFrame.class.getResource("images/Copy24.png");
    return new ImageIcon(adr);    
  }
	

  @Override
  public int idMenu() {
    return 57634;
  }
  
}
