/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;

import objets.CListeObjets;
import objets.CMathGrafDoc;
import types.DimensionFenetre;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import entreesSorties.MtgFileManager;

// Modifi version 4.6.4
@SuppressWarnings("serial")
public class ActionFichierExporterJPEG extends MtgAction {
	class MyFilter extends javax.swing.filechooser.FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory())
				return true;
			String filename = file.getName().toLowerCase();
			return filename.endsWith(".jpg") || filename.endsWith(".jpeg");
		}

		public String getDescription() {
			return "*.jpg; *.jpeg";
		}
	}
	
	public ActionFichierExporterJPEG(MtgFrame frameInit) {
		super(frameInit, null, getString("chaineMenuFichierExporterJPEG"), null, null, null);
	}

  @Override
  public int idMenu() {
    return 33016;
  }	
	
	// Modifi version 4.7.0.3 pour bug avec CEditeur et image de fond
	@Override
	public void actionPerformed(ActionEvent e) {
		MtgInternalFrame cadre = frame.cadreActif();
		if (cadre != null) {
			CMathGrafDoc doc = cadre.document;
			// Pour que l'image copie soit de meilleure qualit on cre une nouvelle image
			// font les dimensions sont celles du gbuffer multiplies par 2 dans les deux dimensions
			int width = cadre.dimf.longueurAbscisses*2;
			int height = cadre.dimf.longueurOrdonnees*2;
			BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			Graphics2D g2D = (Graphics2D) image.getGraphics();
			g2D.setColor(cadre.couleurFond); // Ncessaire car peut tre appel en dehors
			// de paint
			g2D.fillRect(0, 0, width, height);
	    // AffineTransform oldat = g2D.getTransform();
	    // Le 0.2 car l'image est deux fois plus grande que l'original dans chacune des dimensions
	    // AffineTransform at = new AffineTransform(0.2, 0, 0, 0.2, 0, 0);			
	    // g2D.setTransform(at);
			Image imageFond = cadre.document.imageFond;
			// Correction bug pour taille image de fond version 4.7.0.3
			if (cadre.document.modeTraceActive) g2D.drawImage(cadre.imageTraces, 0, 0, (int)(2*cadre.dimf.longueurAbscisses),
					(int)(2*cadre.dimf.longueurOrdonnees), cadre.couleurFond, null);
			else {
				if ((imageFond != null) && cadre.document.imageFondVisible) {
					g2D.drawImage(imageFond, 0, 0, (int)(2*imageFond.getWidth(null)), 
							(int)(2*imageFond.getHeight(null)), cadre.couleurFond,  null);
				}
			}
			// On change la rsolution. Pour cela on fait avant une sauvegarde de la figure dans un
			// flux afin de la recharer ensuite dans le mme tat
	    try {
				ByteArrayOutputStream sauvegarde = new ByteArrayOutputStream();
				MathGraphDataOutputStream oups = new MathGraphDataOutputStream(sauvegarde);
		    cadre.listePr.write(oups);
		    oups.close();
		    CListeObjets listeCopie = new CListeObjets(cadre.listePr);
		    listeCopie.bCreationComposants = false;
		    ByteArrayInputStream ba = new ByteArrayInputStream(sauvegarde.toByteArray());
		    MathGraphDataInputStream inps = new MathGraphDataInputStream(ba);
		    listeCopie.read(inps);
		    inps.close();
		    
		    DimensionFenetre dim = new DimensionFenetre(width, height);
		    listeCopie.adaptePourHauteResolution(2);
		    listeCopie.basseDefinition = false;
		    listeCopie.positionne(cadre.infRand, dim);
		    listeCopie.afficheTout(g2D, dim, cadre.masquageObjetsCaches, doc.couleurFond, doc.transparencePourSurfaces, frame.pref_antialiasing);
		    listeCopie.retireTout();
	    }
	    catch (IOException err) {}			
			JFileChooser fc = new JFileChooser(frame.repertoireFichiers);
			fc.setFileFilter(new MyFilter());
	    int rVal = fc.showSaveDialog(frame);
	    if(rVal == JFileChooser.APPROVE_OPTION) {
	  		File file = fc.getSelectedFile();
				String ch = file.getAbsolutePath();
				String chlwc = ch.toLowerCase();
				if (!chlwc.endsWith(Global.GLOBAL_COMMA_JPG) && !chlwc.endsWith(".jpeg")) {
					ch = ch + Global.GLOBAL_COMMA_JPG;;
					file = new File(ch);
				}
				// On regarde si un fichier du mme nom existe dj.
				// Si oui on demande confirmation pour l'crasement
				if (!MtgFileManager.accepteEnregistrement(frame, ch)) return;
        try {
					// FileImageOutputStream output = new FileImageOutputStream(file);
					ImageIO.write(image, "jpg", file);					
				} catch (FileNotFoundException e1) {
					frame.dialogueAvertissement(getString("ch45"));
				} catch (IOException e1) {
					frame.dialogueAvertissement(getString("ch45"));
				}
	    }
		}		
	}	
}

