/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.event.ActionEvent;

import javax.swing.ImageIcon;

import mathgraph32.MtgFrame;
import outils.Outil;

/**
 * Classe anctre de toutes les actions utilises dans MathGraph32 qui peuvent tre utilises
 * sans qu'une fentre soit ouverte
 * @author Yves
 */

@SuppressWarnings("serial")
public abstract class MtgActionToujoursActivable extends MtgAction {
  /**
   * Constructeur sans outil associ
   * @param frameInit le frame de l'application
   * @param chaineMenu la chane pour le menu
   * @param tip le Tooltip asoci
   * @param icon L'icne asscoi (nul si pas de bouton associ)
   * @param mnemonic Le raccouric clavier associ (nul si aucun)
   */
  public MtgActionToujoursActivable(MtgFrame frameInit, String chaineMenu,
          String tip, ImageIcon icon, Object mnemonic) {
    super(frameInit, chaineMenu, tip, icon, mnemonic);
    outilAssocie = null;
  }
  /**
   * 
   * @param frameInit le frame de l'application
   * @param outilAssocieInit l'outim associ
   * @param chaineMenu la chane pour le menu
   * @param tip le Tooltip asoci
   * @param icon L'icne asscoi (nul si pas de bouton associ)
   * @param mnemonic Le raccouric clavier associ (nul si aucun)
   */
  public MtgActionToujoursActivable(MtgFrame frameInit, Outil outilAssocieInit, String chaineMenu,
          String tip, ImageIcon icon, Object acceleratorKey) {
    super(frameInit, outilAssocieInit, chaineMenu, tip, icon, acceleratorKey);
  }
  
  public void actionPerformed(ActionEvent e) {
    if (outilAssocie != null) {
      if (outilAssocie.activationValide()) {
        frame.outilActif.deselect();
        frame.outilActif = outilAssocie;
        outilAssocie.select();
        // requestFocus();      
      }
      else {
      	String ch = getString("messageOutilPasDisponible");
      	if (outilAssocie.messageActivationNonValide() != null) ch = ch + "\n" + outilAssocie.messageActivationNonValide();
        frame.dialogueAvertissement(ch);
        // jb.setEnabled(false);
        // buttonCapt.grabFocus();
        outilAssocie.deselect();
        // jb.setSelected(false);
        // On r?active l'outil de capture si l'outil ne peut pas ?tre activ?
        frame.activeOutilCapture();
      }         
    }  
  }  
}
