/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;


import glob.Kernel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


import objets.CListeObjets;
import objets.CVariableBornee;

import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class ChoixVariableDlg extends MtgDlg{
	private int indiceMaxi; // Indice de la dernire valeur qu'on peut choisir
	private ListeVariables listeVariables;
	public CVariableBornee variableChoisie;
  private javax.swing.JButton btnOK;
  private javax.swing.JButton btnCancel;
  private JLabel label1;
  private JLabel label2;
  private JLabel label3;
  private JLabel label4;
  private JLabel label5;
  
  private JTextField tfValMini;
  private JTextField tfValMaxi;
  private JTextField tfValPas;
  private JTextField tfValAct;
  
  public ChoixVariableDlg(MtgFrame frameInit, int indiceMaxiInit) {
    super(getString("choixVariableDlg1"), true, frameInit);
    indiceMaxi = indiceMaxiInit;
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());  
    label1 = new JLabel(getString("choixVariableDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new JLabel(getString("variableDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false); 
    label3 = new JLabel(getString("variableDlg4"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false); 
    label4 = new JLabel(getString("variableDlg5"), JLabel.RIGHT);
    label4.setRequestFocusEnabled(false); 
    label5 = new JLabel(getString("variableDlg6"), JLabel.RIGHT);
    label5.setRequestFocusEnabled(false);
    tfValMini = new JTextField();
    tfValMini.setEditable(false);
    tfValMini.setColumns(15);
    tfValMaxi = new JTextField();
    tfValMaxi.setEditable(false);
    tfValMaxi.setColumns(15);
    tfValPas = new JTextField();
    tfValPas.setEditable(false);
    tfValPas.setColumns(15);
    tfValAct = new JTextField();
    tfValAct.setEditable(false);
    tfValAct.setColumns(15);
    CListeObjets liste = frame.cadreActif().listePr;
    listeVariables = new ListeVariables(liste, indiceMaxi, false);
    listeVariables.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listValeursSelectionChanged();
      }
    });
    listeVariables.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent evt) {
				listeVariableMouseClicked(evt);
			}

			public void mouseEntered(MouseEvent evt) {
			}

			public void mouseExited(MouseEvent evt) {
			}

			public void mousePressed(MouseEvent evt) {
			}

			public void mouseReleased(MouseEvent evt) {
			}
    });
 
    JPanel panePrincipal = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new BorderLayout());
    paneLeft.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));

    JScrollPane scrollPane= new JScrollPane(listeVariables,  JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
    		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 50, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneLeft.add(label1, constraints);	
    setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(scrollPane, constraints);	
    setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    paneLeft.add(label2, constraints);	
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(tfValMini, constraints);	
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST);
    paneLeft.add(label3, constraints);	
    setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(tfValMaxi, constraints);	
    setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST);
    paneLeft.add(label4, constraints);	
    setConstraints(constraints, 1, 3, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(tfValPas, constraints);	
    setConstraints(constraints, 0, 4, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST);
    paneLeft.add(label5, constraints);	
    setConstraints(constraints, 1, 4, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(tfValAct, constraints);	
    
    
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    panePrincipal.add(paneLeft,"West");
    panePrincipal.add(paneRight, "East");
    setContentPane(panePrincipal);
    pack();
    listValeursSelectionChanged();
  }
    
  private void btnCancelActionPerformed() {                                          
    dispose();
  }       

  private void btnOKActionPerformed() {
  	int ind = listeVariables.getSelectedIndex();
  	if (ind == -1) variableChoisie = null;
  	else variableChoisie = listeVariables.pointeurs.get(ind);
  	dispose();
  	setRetourOK(true);
  }
  
  private void listValeursSelectionChanged() {
  	int ind = listeVariables.getSelectedIndex();
  	if (ind != -1) {
  		CVariableBornee var = listeVariables.pointeurs.get(ind);
  		tfValMini.setText(var.chaineValeurMini);
  		tfValMaxi.setText(var.chaineValeurMaxi);
  		tfValPas.setText(var.chaineValeurPas);
  		tfValAct.setText(Kernel.chaineNombre(var.valeurActuelle, 12));
  		pack();
  	}
  }
  
	private void listeVariableMouseClicked(MouseEvent evt) {
		if ((evt.getClickCount() == 2) && (listeVariables.getSelectedIndex() != -1)) btnOKActionPerformed();
	}
}
