/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;
import javax.swing.*;
import javax.swing.text.JTextComponent;
import objets.*;
import types.ErreurCalculException;
import types.Int;
import types.UniteAngle;
import mathgraph32.MtgFrame;

/**
 *
 * @author Yves
 * ComboBox utilise pour proposer les valeurs d'angle standard.
 */
@SuppressWarnings("serial")
public final class EditeurAngle extends JComboBox<String> {
	private MtgFrame frame;
  private CValeurAngle val; // L'objet CValeurAngle que ce combo doit modifier.
  private CElementBase objet; // L'objet qui contient val et qui doit tre modifi
  private int uniteAngle;
  
  /** Creates a new instance of EditeurAngle */
  public EditeurAngle(MtgFrame frameInit, 
      CValeurAngle valInit, CElementBase objetInit, int uniteAngleInit) {
    super();
  	JTextField editor = (JTextField) getEditor().getEditorComponent();
  	editor.setColumns(20);
    
    frame = frameInit;
    val = valInit;
    objet = objetInit;
    uniteAngle = uniteAngleInit;
    setEditable(true);
    if (uniteAngle == UniteAngle.Radian) {
	    addItem("pi/2");
	    addItem("-pi/2");
	    addItem("pi/3");
	    addItem("-pi/3");
	    addItem("pi/6");
	    addItem("-pi/6");
	    addItem("+pi/4");
	    addItem("-pi/4");    	
    }
    else {
	    addItem("+90");
	    addItem("-90");
	    addItem("+60");
	    addItem("-60");
	    addItem("+30");
	    addItem("-30");
	    addItem("+45");
	    addItem("-45");
    }
  }
  /* Vrification de la validit de la rponse contenue dans le champ d'dition
   * du combo
  */
  public final boolean verifieValidite() {
    @SuppressWarnings("unused")
		double v;
    int indice;
    CListeObjets liste = objet.listeProprietaire;
    String rep = (String) getSelectedItem();
    if (rep == null) {
    	frame.dialogueAvertissement(MtgFrame.getString("ch97"));
    	return false;
    }
    // Dans le cas ou on cre un nouvel objet, celui-ci n'a pas encore t ajout
    //  la liste des objets de l'applet.
    int indobj = liste.indexOf(objet);
    if (indobj == -1) indice = liste.size() - 1;
    else indice = indobj - 1;

    Int inderr = new Int();
    // On vrifie d'abord la syntaxe de la rponse
    // La seule valeur qu'on a le droit d'utiliser est pi qui est toujours
    // le premier objet de la liste
    if (CCalculBase.verifieSyntaxe(liste, rep, inderr, indice, null)){
    
      CCalculBase calc = CCalculBase.creeCalculBase(rep ,liste, 0, rep.length()-1, null);
      try {
        calc.listeProprietaire.initialiseNombreIterations();
        v = calc.resultat(false);
      }
      catch (ErreurCalculException e) {
      	if (e.depassementCapacite)
      		frame.dialogueAvertissement(MtgFrame.getString("ch179"));
      	else frame.dialogueAvertissement(MtgFrame.getString("ch44"));
        requestFocus();
        return false;
      }
      val.donneCalcul(calc);
      return true;
    }
    else {
      // On place le curseur  l'endroit de l'erreur de syntaxe
    	JTextComponent editor = (JTextComponent) getEditor().getEditorComponent();
      editor.setSelectionStart(inderr.getValue());
      editor.setSelectionEnd(inderr.getValue());
      frame.dialogueAvertissement(MtgFrame.getString("ch5"));
      requestFocus();
      return false;
    }
  }
}

