/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCalculBase;
import objets.CFoncNVar;
import objets.CListeObjets;
import types.Int;

@SuppressWarnings("serial")
public class FonctionReelle2VarDlg extends MtgDlg {
	private CFoncNVar fonctionAModifier;
	private boolean modification;
  private javax.swing.JButton btnCancel;
  private javax.swing.JButton btnOK;
  private JLabel label1, label2, label3, label4;
  private javax.swing.JTextField editeurFormule;
  private javax.swing.JTextField editeurNom;
  private javax.swing.JTextField editeurVariableFormelle1;
  private javax.swing.JTextField editeurVariableFormelle2;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;

  public FonctionReelle2VarDlg(MtgFrame frameInit, CFoncNVar fonctionAModifierInit, boolean modificationInit) {
    super (getString("fonction2VarDialog1"), true, frameInit);
    fonctionAModifier = fonctionAModifierInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }
  
  private void initComponents() {
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    label1 = new JLabel(getString("fonctionDialog2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new JLabel(getString("fonction2VarDialog3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new JLabel(getString("fonction2VarDialog4"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    label4 = new JLabel(getString("fonctionDialog4"), JLabel.RIGHT);
    label4.setRequestFocusEnabled(false);
    editeurFormule = new javax.swing.JTextField(null, 30);
    editeurFormule.setAutoscrolls(true);
    editeurNom = new javax.swing.JTextField(null,8);
    editeurVariableFormelle1 = new javax.swing.JTextField(null,8);
    editeurVariableFormelle2 = new javax.swing.JTextField(null,8);
  	int indmax;
  	if (modification) indmax = frame.cadreActif().listePr.indexOf(fonctionAModifier) - 1;
  	else indmax = frame.cadreActif().listePr.size() - 1;
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurFormule, true, indmax, false);
    if (modification) {
    	String[] ch = new String[2];
    	ch[0] = fonctionAModifier.chaineVariable[0];    	
    	ch[1] = fonctionAModifier.chaineVariable[1];    	
    	editeurVariableFormelle1.setText(ch[0]);
    	editeurVariableFormelle2.setText(ch[1]);
      editeurFormule.setText(fonctionAModifier.calcul.chaineCalculSansPar(ch));
      editeurNom.setText(fonctionAModifier.nom());
      editeurNom.setEditable(false);
    }
    JPanel panePrincipal = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new FlowLayout());
    paneLeft.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 20, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 2, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    constraints.insets.bottom = 0;
    paneLeft.add(editeurNom, constraints);
    
    setConstraints(constraints, 0, 1, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneLeft.add(label2, constraints);
   
    setConstraints(constraints, 1, 1, 2, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(editeurVariableFormelle1, constraints);
    
    setConstraints(constraints, 0, 2, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneLeft.add(label3, constraints);
   
    setConstraints(constraints, 1, 2, 2, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(editeurVariableFormelle2, constraints);

    setConstraints(constraints, 0, 3, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneLeft.add(label4, constraints);
   
    setConstraints(constraints, 1, 3, 2, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(editeurFormule, constraints);
    
    constraints.insets.bottom = 5;
    setConstraints(constraints, 1, 4, 1, 1, 30, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(paneListeValeurEtFonctions, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();
  }
  
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {                                      
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    String chVarFor1 = editeurVariableFormelle1.getText();
    String chVarFor2 = editeurVariableFormelle2.getText();
    // On regarde d'abord si le nom propos est correct et non vide en cas de cration d'un calcul
  	// On vrifie d'abord la validit du nom de la fonction
    if (! modification && !listePr.validationNomCalcul(frame, chNom)) {
      editeurNom.requestFocus();
      return;
    }
    // puis celui des variables formelles
    if (!listePr.validationNomVariableFormelle(frame, chVarFor1)) {
      editeurVariableFormelle1.requestFocus();
      return;
    }
    if (!listePr.validationNomVariableFormelle(frame, chVarFor2)) {
      editeurVariableFormelle2.requestFocus();
      return;
    }
    if (chVarFor1.equalsIgnoreCase(chVarFor2)) {
    	dialogueAvertissement(getString("ch1"));
      editeurVariableFormelle2.requestFocus();
      return;
    }
    // On vrifie ensuite que le nom de la variable formelle n'est pas le mme que le nom de la fonction
    if (chNom.equalsIgnoreCase(chVarFor1)) {
      dialogueAvertissement(getString("ch92"));
      editeurVariableFormelle1.requestFocus();
      return;	    		    	
    }
    if (chNom.equalsIgnoreCase(chVarFor2) ) {
      dialogueAvertissement(getString("ch92"));
      editeurVariableFormelle2.requestFocus();
      return;	    		    	
    }
    //Le nom est correct. On vrifie maintenant la formule
    String chCalcul = editeurFormule.getText();
    int indDernier; // Dernier objet autoris dans la formule
    if (modification) indDernier = listePr.indexOf(fonctionAModifier) - 1;
    else indDernier = listePr.size()-1;
    Int indErr = new Int();
    // On n'autorise d'utiliser que les objets crs par l'utilisateur
    String[] varFor = new String[2];
    varFor[0] = chVarFor1;
    varFor[1] = chVarFor2;
    if (CCalculBase.verifieSyntaxe(listePr, chCalcul, indErr, indDernier, varFor)) {
      CCalculBase calc = CCalculBase.creeCalculBase(chCalcul ,listePr, 0, chCalcul.length()-1, varFor);
      fonctionAModifier.calcul = calc;
      fonctionAModifier.chaineVariable[0] = chVarFor1;
      fonctionAModifier.chaineVariable[1] = chVarFor2;
      // Modification version 4.5.2 : Quand la chane de calcul sera reconstruite elle pourra tre diffrente de celle ici entre
      // On la change tout de suite
      fonctionAModifier.chaineCalcul = calc.chaineCalculSansPar(varFor);
      
      if (modification) {
      	cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil48"));
      	// A revoir version 3.0
      	// Il faut mettre  jour les drives dpendant de cette fonction
      	// Supprim version 4.2 car metAJour a t rajout pour CDerivee et recalcule la formule des drives
      	// listePr.recalculeDeriveesDependantDe(fonctionAModifier);
      }
      else {
      	fonctionAModifier.nomCalcul = chNom;   	
      }
      setRetourOK(true);
      dispose();
    }
    else {
    	dialogueAvertissement(getString("ch5"));
      editeurFormule.setCaretPosition(indErr.getValue());
      editeurFormule.requestFocus();
      return;     
    }
  }                                     

  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }
}
