/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import outils.OutilHistorique;

import objets.CBarycentre;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CTransformation;
import objets.CValeurDynamique;
import types.NatureCalcul;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 *
 * @author Yves
 */
@SuppressWarnings("serial")
public class HistoriqueDlg extends MtgDlg {
  public int indiceObjetEnCours; // L'indice de l'objet qui est affich
  public int indiceReelObjetEnCours; // L'indice rel sans tenir compte des
  // transformations ou des objets intersection ni des objets intermdiaires des constructions
  public int indiceReelObjetEnCoursAvecInterm; // L'indice rel sans tenir compte des
  // transformations ou des objets intersection mais en tenant compte des objets intermdiaires des constructions
  public static Insets insetsZero = new Insets(0,0,0,0);

  private JLabel label1;
  private JLabel label2;
  private JButton btnOK;
  private JButton buttonDebut;
  private JButton buttonFin;
  private JButton buttonPrec;
  private JButton buttonSuiv;
  private JButton buttonSupprimer;
  private JButton buttonSupprimerJusqueFin;
  private JButton buttonModifierObjet;
  private JScrollPane scrollPane;
  private JPanel panePrincipal;
  private JPanel paneLeft;
  private JPanel paneLeftLeft;
  private JPanel paneFleches;
  private JCheckBox checkTraceInterm;  // True si trace dans les objets intermdiaires des macros
  
  private javax.swing.JTextArea textArea;
  private javax.swing.JTextField textField;
  
  public HistoriqueDlg(MtgFrame frameInit) {
    super (getString("historiqueDlg0"), true, frameInit);
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
    addWindowListener(new java.awt.event.WindowAdapter() {
        public void windowClosing(java.awt.event.WindowEvent evt) {
          fermeture();
        } 
    } );
  }
  private void initComponents() {
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });
    label1 = new JLabel(getString("historiqueDlg1") + " ", JLabel.RIGHT);
    label2 = new JLabel(" " + getString("historiqueDlg2"), JLabel.RIGHT);
    checkTraceInterm = new JCheckBox(getString("historiqueDlg3"));
    checkTraceInterm.setSelected(false);
    checkTraceInterm.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        checkTraceIntermPerformed();
      }
 		});    
    scrollPane = new javax.swing.JScrollPane();
    textField = new javax.swing.JTextField();
    textArea = new javax.swing.JTextArea();
    URL adr = MtgFrame.class.getResource("images/Precedent.gif");    
    buttonPrec = new javax.swing.JButton(new ImageIcon(adr));
    adr = MtgFrame.class.getResource("images/Suivant.gif");    
    buttonSuiv = new javax.swing.JButton(new ImageIcon(adr));
    adr = MtgFrame.class.getResource("images/ModificationObjetGraphique.png");    
    buttonModifierObjet = new javax.swing.JButton(new ImageIcon(adr));    
    adr = MtgFrame.class.getResource("images/Debut.gif");    
    buttonDebut = new javax.swing.JButton(new ImageIcon(adr));
    adr = MtgFrame.class.getResource("images/Fin.gif");    
    buttonFin = new javax.swing.JButton(new ImageIcon(adr));
    adr = MtgFrame.class.getResource("images/Delete24.png");    
    buttonSupprimer = new javax.swing.JButton(new ImageIcon(adr));
    adr = MtgFrame.class.getResource("images/SuppressionJusqueFin.png");    
    buttonSupprimerJusqueFin = new javax.swing.JButton(new ImageIcon(adr));
    
    textField.setEditable(false);
    textField.setColumns(4);
    textArea.setColumns(20);
    textArea.setEditable(false);
    textArea.setLineWrap(true);
    textArea.setRows(5);
    scrollPane.setViewportView(textArea);
    scrollPane.setPreferredSize(new Dimension(350, 90));
    
    setModal(true);
    setResizable(false);
    
    buttonPrec.setMargin(insetsZero);
    buttonPrec.setToolTipText(getString("tipPrecedent"));
    buttonPrec.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        buttonPrecActionPerformed();
      }
    });
    buttonSuiv.setMargin(insetsZero);
    buttonSuiv.setToolTipText(getString("tipSuivant"));
    buttonSuiv.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        buttonSuivActionPerformed();
      }
    });
    buttonDebut.setMargin(insetsZero);
    buttonDebut.setToolTipText(getString("tipDebut"));
    buttonDebut.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        buttonDebutActionPerformed();
      }
    });

    buttonFin.setMargin(insetsZero);
    buttonFin.setToolTipText(getString("tipFin"));
    buttonFin.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        buttonFinActionPerformed();
      }
    });
    buttonSupprimer.setMargin(insetsZero);
    buttonSupprimer.setToolTipText(getString("tipSuppressionObjet"));
    buttonSupprimer.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        buttonSupprimerActionPerformed();
      }
    });
    buttonSupprimerJusqueFin.setMargin(insetsZero);
    buttonSupprimerJusqueFin.setToolTipText(getString("tipSuppressionJusqueFin"));
    buttonSupprimerJusqueFin.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        buttonSupprimerJusqueFinActionPerformed();
      }
    });
    buttonModifierObjet.setMargin(insetsZero);
    buttonModifierObjet.setToolTipText(getString("tipModifierObjet"));
    buttonModifierObjet.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        buttonModifierObjetActionPerformed();
      }
    });
    
    panePrincipal = new JPanel();
    panePrincipal.setLayout(new BoxLayout(panePrincipal, BoxLayout.X_AXIS));
    paneLeftLeft = new JPanel();
    paneLeftLeft.setLayout(new BorderLayout());
    paneLeft = new JPanel();
    paneFleches = new JPanel();
    paneFleches.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    
    setConstraints(constraints, 0, 0, 1, 1, 30, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneFleches.add(buttonPrec, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    // constraints.insets.bottom = 0;
    paneFleches.add(buttonSuiv, constraints);
    
    constraints.insets.top = 5;
    constraints.insets.right = 5;
    setConstraints(constraints, 0, 1, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneFleches.add(buttonDebut, constraints);
   
    constraints.insets.right = 0;
    constraints.insets.left = 5;
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneFleches.add(buttonFin, constraints);
    
    constraints.insets.right = 0;
    constraints.insets.left = 0;
    setConstraints(constraints, 0, 2, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneFleches.add(buttonSupprimer, constraints);
    
    constraints.insets.left = 5;
    setConstraints(constraints, 1, 2, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneFleches.add(buttonSupprimerJusqueFin, constraints);
    
    setConstraints(constraints, 2, 2, 1, 1, 30, 30, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneFleches.add(buttonModifierObjet, constraints);
    
    setConstraints(constraints, 2, 0, 1, 1, 30, 30, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneFleches.add(btnOK, constraints);
    
    paneLeftLeft.add("West",label1);
    paneLeftLeft.add("Center",textField);
    paneLeftLeft.add("East",label2);
    paneLeftLeft.add("South",checkTraceInterm);
    paneLeft.add(paneLeftLeft);
    paneLeft.add(scrollPane);
    panePrincipal.add(paneLeft);
    panePrincipal.add(Box.createRigidArea(new Dimension(5,5)));
    panePrincipal.add(paneFleches);  
    panePrincipal.add(Box.createRigidArea(new Dimension(5,5)));
    setContentPane(panePrincipal);
    pack();
    
  }
  
  private void fermeture() {
    dispose();
    frame.outilHistorique.deselect();
    MtgInternalFrame cadre = frame.cadreActif();
    cadre.effacegbuffer();
    frame.cadreActif().repaintFigureSansCalcul();
    frame.activeOutilCapture();
  }
  
  public void reset() {
    indiceObjetEnCours = 0;
    indiceReelObjetEnCours = 0;
    indiceReelObjetEnCoursAvecInterm = 0;
    remplitChamps();
  }
 /**
 * Fonction remplissant le champ devant contenir le n de l'objet en cours
 * et le champ contenant le texte dcrivant l'objet en cours
 */  
private void remplitChamps() {
	System.gc();
  CElementBase elb = elementEnCours();
  if (!elb.estElementIntermediaire() && elb.modifiableParMenu()) buttonModifierObjet.setVisible(true);
  else buttonModifierObjet.setVisible(false);
  if (checkTraceInterm.isSelected()) textField.setText(Integer.toString(indiceReelObjetEnCoursAvecInterm + 1));
  else textField.setText(Integer.toString(indiceReelObjetEnCours + 1));
  String ch = elb.infoPourHistorique();
  if (elb.estDeNature(NatureObjet.NToutObjet)) {
  	CElementGraphique elg = (CElementGraphique) elb;
  	if (elg.masque) ch += "\n" + getString("ch164");
  }
  if ((!elb.existe) && (elb.getNatureCalcul() == NatureCalcul.NAucunCalcul)) ch = ch + "\n" + getString("ch18");
  if (elb.estElementIntermediaire()) ch = ch + "\n" + getString("ch159");
  else if (elb.estElementFinal) ch = ch + "\n" + getString("ch160");
  textArea.setText(ch);
  textArea.setCaretPosition(0);
  OutilHistorique oh = frame.outilHistorique;
  oh.annuleClignotement();
  if ((elb.getNature() & NatureObjet.NToutObjet) != 0) { 
    oh.ajouteClignotementDe((CElementGraphique)elb);
    elb.ajouteAntecedents(frame, frame.listeClignotante);
    // Il faut positionner la liste clignotante car pour les translations
    // on lui a rajout des lments qui ne sont pas des objets suppmentaires
    // ne faisant pas partie de la figure
    frame.listeClignotante.positionne(false, frame.cadreActif().dimf);
    oh.resetClignotement();
  }
  else {
  	if (elb.estDeNatureCalcul(NatureCalcul.NTouteMesure)) {
      elb.ajouteAntecedents(frame, frame.listeClignotante);
      oh.resetClignotement();
  	}
  }
}

private CElementBase elementEnCours() {
  return (CElementBase) (frame.cadreActif().listePr.get(indiceObjetEnCours));
}
   


  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
  	frame.cadreActif().listePr.retablitComposants(); // Ajout version 4.5.2
    fermeture();
  }                                     

  private void buttonSuivActionPerformed() {                                           
    CListeObjets liste = frame.cadreActif().listePr;
    if (indiceObjetEnCours < liste.size() - 1) {
      indiceObjetEnCours ++;
      indiceReelObjetEnCours ++;
      indiceReelObjetEnCoursAvecInterm++;
      CElementBase elb = elementEnCours();
      if (!checkTraceInterm.isSelected()) {
	      if (elb.estElementIntermediaire()) {
	      	for (int i = indiceObjetEnCours; i < liste.size() - 1; i++ ) {
	      		elb = elementEnCours();
	      		if (elb.estElementIntermediaire()) {
	      			indiceObjetEnCours++;
	      			indiceReelObjetEnCoursAvecInterm++;
	      			elb = elementEnCours();
	      		}
	      		else break;
	      	}
	      }
      }
      while (elb.estDeNature(NatureObjet.NTransformation | NatureObjet.NBipoint)) {
        indiceObjetEnCours ++;
        elb = elementEnCours();
      }
      frame.cadreActif().paneFigure.repaint();
      remplitChamps();
    }
    else Toolkit.getDefaultToolkit().beep();
}                        

  private void buttonPrecActionPerformed() {
    MtgInternalFrame cadre = frame.cadreActif();
    if (indiceObjetEnCours > 0) {
      indiceObjetEnCours --;
      indiceReelObjetEnCours --;
      indiceReelObjetEnCoursAvecInterm--; // Bug corrig version 3.6.1
      CElementBase elb = elementEnCours();
      if (!checkTraceInterm.isSelected()) {
	      if (elb.estElementIntermediaire()) {
	      	for (int i = indiceObjetEnCours; i >= 0; i-- ) {
	      		elb = elementEnCours();
	      		if (elb.estElementIntermediaire()) {
	      			indiceObjetEnCours--;
	      			indiceReelObjetEnCoursAvecInterm--;
	      			elb = elementEnCours();
	      		}
	      		else break;
	      	}
	      }
      }
      while (elb.estDeNature(NatureObjet.NTransformation | NatureObjet.NBipoint)) {
        indiceObjetEnCours --;
        elb = elementEnCours();
      }
      cadre.paneFigure.repaint();
      remplitChamps();
    }
    else Toolkit.getDefaultToolkit().beep();
}                                          

  private void buttonDebutActionPerformed() {                                            
    reset();
    frame.cadreActif().paneFigure.repaint();    
}                                           

  private void buttonFinActionPerformed() {
    MtgInternalFrame cadre = frame.cadreActif();    
    CListeObjets liste = cadre.listePr;
    indiceObjetEnCours = liste.size() - 1;
    indiceReelObjetEnCours = liste.nombreElementsPourHistorique(checkTraceInterm.isSelected()) - 1;
    indiceReelObjetEnCoursAvecInterm = liste.nombreElementsPourHistorique(true) - 1;
    cadre.paneFigure.repaint();
    remplitChamps();
}                                         
    
  private void buttonSupprimerActionPerformed() {                                                
    if (indiceObjetEnCours == 0) {
      dialogueAvertissement(getString("ch24"));
      return;
    }
    MtgInternalFrame cadre = frame.cadreActif();    
    CListeObjets liste = cadre.listePr;
    CElementBase elb = elementEnCours();
    // Destruction de l'objet en cours
    CElementBase antecedentDirect = elb.antecedentDirect();
    if (antecedentDirect.getNature() == NatureObjet.NTransformation) {
    	// Si l'objet antcdent direct est une transformation, on regarde s'il y a d'autres images par
    	// cette transformation  dtruire. Si oui, on ne dtruit pas la transformation.
    	// Sinon on dtruit la transformation et ses dpendants.
    	// On peut avoir par exemple cr un point image d'un point par une transformation puis l'iamge de cette image
    	// Dans ce cas il ne faut pas dtruite la transformation
    	if (liste.nombreImagesParTransformationNonDepDe((CTransformation) antecedentDirect, elb) > 0) antecedentDirect = elb;
    }
    if (antecedentDirect.getNature() == NatureObjet.NBipoint)  {
      if (!frame.dialogueConfirmation_YES_NO(getString("ch38"))) {
      	return;
      }
    }
    
    if (cadre.listePr.nombreDependants(elb) >= 2) {
      if (frame.dialogueConfirmation_YES_NO(getString("ch37"))) {
      	if (antecedentDirect != elb) indiceObjetEnCours --;
        frame.retireDependants(antecedentDirect);
        cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
        if (indiceObjetEnCours >= liste.size()) {
          indiceObjetEnCours = liste.size() - 1;
          indiceReelObjetEnCours = liste.nombreElementsPourHistorique(cadre.traceObjetsInterm) - 1;
        }
        cadre.effacegbuffer();
        cadre.repaintFigureSansCalcul();
        remplitChamps();
      }      
    }
    else {
      frame.retireDependants(antecedentDirect);
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
      if (indiceObjetEnCours >= liste.size()) {
        indiceObjetEnCours = liste.size() - 1;
        indiceReelObjetEnCours = liste.nombreElementsPourHistorique(cadre.traceObjetsInterm) - 1;
      }
      cadre.effacegbuffer();
      cadre.repaintFigureSansCalcul();
      remplitChamps();
    }   
  }
  
  private void buttonSupprimerJusqueFinActionPerformed() {                                              
   	if (indiceObjetEnCours == 0) {
      dialogueAvertissement(getString("ch24"));
      return;
    }
    MtgInternalFrame cadre = frame.cadreActif();    
    CListeObjets liste = cadre.listePr;
    CElementBase elb = elementEnCours();
    // Destruction de l'objet en cours
    CElementBase antecedentDirect = elb.antecedentDirect();
    if (antecedentDirect.getNature() == NatureObjet.NBipoint)  {
      if (frame.dialogueConfirmation_YES_NO(getString("ch38"))) {
		    frame.retireNDerniersElements(liste.size() - indiceObjetEnCours);
		    // Si l'antcdent direct n'tait pas le mme que l'lment, on regarde :
		    // Si c'est une transformation, on regarde s'il reset d'autres objets dpendant de cette transformation.
		    // S'il n'y en a pas, on dtruit la transformation, sinon on la garde
		    // Dans les autres cas (destruction d'une intersection d'objets) on dtruit l'antcdent direct et ses dpendants
		    if (antecedentDirect != elb) {
		    	if (antecedentDirect.getNature() == NatureObjet.NTransformation) {
		    		indiceObjetEnCours --;
		    		if (liste.nombreDependants(antecedentDirect) == 1) {
		    			frame.retireDependants(antecedentDirect);
		    		}
		    	}
		  		else frame.retireDependants(antecedentDirect);
		    }
		    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
	      if (indiceObjetEnCours >= liste.size()) {
	        indiceObjetEnCours = liste.size() - 1;
	        indiceReelObjetEnCours = liste.nombreElementsPourHistorique(cadre.traceObjetsInterm) - 1;
	      }
	      cadre.effacegbuffer();
        cadre.repaintFigureSansCalcul();
	      remplitChamps();		    
		  }
    }
    else {
	    frame.retireNDerniersElements(liste.size() - indiceObjetEnCours);
	    // Si l'antcdent direct n'tait pas le mme que l'lemnt, on regarde :
	    // Si c'est une transformation, on regarde s'il reset d'autres objets dpendant de cette transformation.
	    // S'il n'y en a pas, on dtruit la transformation, sinon on la garde
	    // Dans les autres cas (destruction d'une intersection d'objets) on dtruit l'antcdent direct et ses dpendants
	    if (antecedentDirect != elb) {
	    	indiceObjetEnCours --;
	    	if (antecedentDirect.getNature() == NatureObjet.NTransformation) {
	    		if (liste.nombreDependants(antecedentDirect) == 1) {
	    			frame.retireDependants(antecedentDirect);
	    		}
	    	}
	  		else frame.retireDependants(antecedentDirect);
	    }
	    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
      if (indiceObjetEnCours >= liste.size()) {
        indiceObjetEnCours = liste.size() - 1;
        indiceReelObjetEnCours = liste.nombreElementsPourHistorique(cadre.traceObjetsInterm) - 1;
      }
      cadre.effacegbuffer();
      cadre.repaintFigureSansCalcul();
      remplitChamps();
	  }
  }
  
  public void buttonModifierObjetActionPerformed() {
    CElementBase elb = elementEnCours();
    MtgInternalFrame cadre = frame.cadreActif();
    if (elb.modifiableParMenu()) {
    	if (elb.getNature() == NatureObjet.NBarycentre) {
    		// Pour une modification de barycentre, on doit fermer l'historique au contraire des autres objets.
        dispose();
        frame.outilHistorique.deselect();
        cadre.effacegbuffer();
        cadre.repaintFigureSansCalcul();
    		frame.outilActif = frame.outilModifierBarycentre;
    		frame.outilModifierBarycentre.select((CBarycentre) elb);
    	}
    	else {
	    	if (elb.estDeNature(NatureObjet.NToutObjet)) {
	    		frame.outilModifierObjetGraphique.objetDesigne = (CElementGraphique) elb;
	    		frame.outilModifierObjetGraphique.creeObjetPourHistorique();
	    		remplitChamps();
	    	}
	    	else {
	    		CValeurDynamique val = (CValeurDynamique) elb;
	      	MtgDlg dlg = val.dialogueModification(frame);
	      	dlg.setLocationRelativeTo(this);
	      	dlg.setVisible(true);
	      	if (dlg.retourOK()) {
	    		  cadre.listePr.metAJourObjetsDependantDe(val);      		
	      		frame.positionne();
	      		remplitChamps();
	      	}
	    	}
    	}
    }
  }
  
  // Le cadre actif a besoin d'tre inform si on trace ou non dans les objets intermdiaires
  // cviz la fonction paint() de son paneFigure
  public void checkTraceIntermPerformed() {
  	boolean selected = checkTraceInterm.isSelected();
  	frame.cadreActif().traceObjetsInterm = selected;
  	// Modification version 3.6.1 : on remet tout au dbut
  	buttonDebutActionPerformed();
  	// Si on slectionne le bouton de traage dans les objest intermdiaires des macros
  	// et si l'objet en cours est un objet intermdiaire
  	/*
  	CElementBase elb = elementEnCours();
  	if (elb.estElementIntermediaire()) buttonSuivActionPerformed();
  	*/
  }
}
