/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.net.URL;
import java.text.NumberFormat;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.border.TitledBorder;

import entreesSorties.MtgFileManager;
import glob.Kernel;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class HtmlExportDlg extends MtgDlg {
	private static final String strue = "true";
	private static final String sfalse = "false";
	private JCheckBox cbIncludeJarFile;
	private JCheckBox cbAllowLeftToolBar;
	private JCheckBox cbAllowTopToolBar;
	private JCheckBox cbAllowRightToolBar;
	private JCheckBox cbAllowMenuBar;
	private JCheckBox cbAllowToolsChoice;
	private JCheckBox cbAllowFileMenu;
	private JCheckBox cbAllowOptionsMenu;
	private JCheckBox cbLaunch; // Sera true si on veut lancer le navigateur avec la figure gnre aprs l'exportation
	// Version 4.4.1 : Deux applets proposes
	private JRadioButton rbCreationMode, rbDisplayMode;
	private JRadioButton rbNiveau1, rbNiveau2, rbNiveau3, rbNiveau4;
	
	private JLabel label1, label2, label3, label4, label5, label6;
	private JButton btnExporter, btnCancel;
	private JTextField titleEditor, authorNameEditor;
	private JFormattedTextField widthEditor, heightEditor;
	private JTextPane textEditor1, textEditor2;
	private JTabbedPane tabbedPane;
	private JPanel paneHaut, paneBas, paneGeneral, paneOptions, paneNiveau;
	
	class MyFilter extends javax.swing.filechooser.FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory())
				return true;
			String filename = file.getName().toLowerCase();
			return filename.endsWith(Global.GLOBAL_COMMA_HTM)
				|| filename.endsWith(Global.GLOBAL_COMMA_HTML);
		}

		public String getDescription() {
			return Global.GLOBAL_COMMA_STAR_HTM + Global.GLOBAL_COMMA_STAR_HTM;
		}
	}
	
	public HtmlExportDlg(MtgFrame frameInit)	{
		super(getString("dlgExport1"), true, frameInit);
		tabbedPane=new JTabbedPane();
		paneHaut = new JPanel();
		paneBas = new JPanel();
		paneGeneral = new JPanel();
		paneOptions = new JPanel();
		label1 = new JLabel(getString("titrePage"));
		label2 = new JLabel(getString("auteur"));
		label3 = new JLabel(getString("texteHtm1"));
		label4 = new JLabel(getString("texteHtm2"));
		label5 = new JLabel(getString("largeur"));
		label6 = new JLabel(getString("hauteur"));
		
    btnExporter = new javax.swing.JButton(getString("btnExporter"));
    btnExporter.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnExporterActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    titleEditor = new JTextField(20);
    authorNameEditor = new JTextField(20);
    authorNameEditor.setText(System.getProperty("user.name"));
    textEditor1 = new JTextPane();
    textEditor1.setContentType("text/html");
		JScrollPane scrollPaneText1 = new JScrollPane(textEditor1, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
  	scrollPaneText1.setPreferredSize(new Dimension(500, 120));
    
    textEditor2 = new JTextPane();
    textEditor2.setContentType("text/html");
		JScrollPane scrollPaneText2 = new JScrollPane(textEditor2, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
  	scrollPaneText2.setPreferredSize(new Dimension(500, 120));
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    widthEditor = new JFormattedTextField(format);
    widthEditor.setColumns(5);
    // Modification version 4.4.5 : On propose par dfaut la taille de l'applet
    // widthEditor.setText("800"); // Largeur de 800 par dfaut
    MtgInternalFrame cadre = frame.cadreActif();
    widthEditor.setText(Integer.toString(cadre.dimf.longueurAbscisses));
    heightEditor = new JFormattedTextField(format);
    heightEditor.setColumns(5);
    // Modification version 4.4.5 : On propose par dfaut la taille de l'applet
    // heightEditor.setText("600"); // Hauteur de 600 par dfaut
    heightEditor.setText(Integer.toString(cadre.dimf.longueurOrdonnees));

		cbAllowLeftToolBar = new JCheckBox(getString("dlgExport2"), true);
		cbAllowTopToolBar = new JCheckBox(getString("dlgExport3"), true);	
		cbAllowMenuBar = new JCheckBox(getString("dlgExport5"), true);
		cbAllowToolsChoice = new JCheckBox(getString("dlgExport4"), true);
		cbAllowFileMenu = new JCheckBox(getString("dlgExport6"), true);
		cbAllowOptionsMenu = new JCheckBox(getString("dlgExport7"), true);
  	cbIncludeJarFile = new JCheckBox(getString("dlgExport8"), false);
		cbAllowRightToolBar = new JCheckBox(getString("dlgExport9"), true);
		cbLaunch = new JCheckBox(getString("dlgExport10"),true);
		
		rbNiveau1 = new JRadioButton(getString("niveauElementaire"));
		rbNiveau2 = new JRadioButton(getString("niveauMoyen"));
		rbNiveau3 = new JRadioButton(getString("niveauAvanceSansComplexes"));
		rbNiveau4 = new JRadioButton(getString("niveauAvanceAvecComplexes"));
		ButtonGroup btng = new ButtonGroup();
		btng.add(rbNiveau1);
		btng.add(rbNiveau2);
		btng.add(rbNiveau3);
		btng.add(rbNiveau4);
		rbNiveau3.setSelected(true);
		
		rbCreationMode = new JRadioButton(getString("modeCreation"));
		rbCreationMode.addActionListener(new ActionListener() {		
			public void actionPerformed(ActionEvent e) {
				rbCreationModeActionPerformed();
			}
		});
		rbDisplayMode = new JRadioButton(getString("modeVisualisation"));
		rbDisplayMode.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				rbDisplayModeActionPerformed();
			}
		});
		ButtonGroup btngroup = new ButtonGroup();
		btngroup.add(rbCreationMode);
		btngroup.add(rbDisplayMode);
		
		
    JPanel panePrincipal = new JPanel();
    paneBas = new JPanel();
    panePrincipal.setLayout(new FlowLayout());
    paneGeneral = new JPanel();
    paneGeneral.setLayout(new GridBagLayout());
    paneOptions = new JPanel();
    paneOptions.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    paneHaut.setLayout(new GridBagLayout());
    paneBas.setLayout(new GridBagLayout());
    JPanel paneBas2 = new JPanel();
    paneBas2.setLayout(new BoxLayout(paneBas2, BoxLayout.X_AXIS));
    paneBas2.add(btnExporter);
    paneBas2.add(Box.createRigidArea(new Dimension(10,0)));
    paneBas2.add(btnCancel);
    setConstraints(constraints, 0, 0, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneBas.add(paneBas2, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(cbLaunch, constraints);
    JPanel paneBas3 = new JPanel();
    paneBas3.add(rbCreationMode);
    paneBas3.add(rbDisplayMode);
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(paneBas3, constraints);
    

    setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHaut.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 80, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneHaut.add(titleEditor, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneHaut.add(label2, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 80, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneHaut.add(authorNameEditor, constraints);
    JPanel paneDimensions = new JPanel();
    paneDimensions.add(label5);
    paneDimensions.add(widthEditor);
    paneDimensions.add(label6);
    paneDimensions.add(heightEditor);
    setConstraints(constraints, 0, 2, 2, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneHaut.add(paneDimensions, constraints);
    
    
    
    setConstraints(constraints, 0, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneGeneral.add(label3, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    paneGeneral.add(scrollPaneText1, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneGeneral.add(label4, constraints);
    setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    paneGeneral.add(scrollPaneText2, constraints);

    paneNiveau = new JPanel();
    paneNiveau.setBorder(new TitledBorder(getString("ongletNiveau")));
    paneNiveau.setLayout(new BoxLayout(paneNiveau, BoxLayout.Y_AXIS));
    paneNiveau.add(rbNiveau1);
    paneNiveau.add(rbNiveau2);
    paneNiveau.add(rbNiveau3);
    paneNiveau.add(rbNiveau4);
    
    setConstraints(constraints, 0, 0, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(cbIncludeJarFile, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(cbAllowLeftToolBar, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(cbAllowTopToolBar, constraints);
    setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(cbAllowRightToolBar, constraints);
    setConstraints(constraints, 0, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(cbAllowMenuBar, constraints);
    setConstraints(constraints, 0, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(cbAllowToolsChoice, constraints);
    setConstraints(constraints, 0, 6, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(cbAllowFileMenu, constraints);
    setConstraints(constraints, 0, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(cbAllowOptionsMenu, constraints);
    setConstraints(constraints, 1, 0, 1, 6, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneOptions.add(paneNiveau, constraints);
    
    
    tabbedPane.addTab(getString("texteHtml"), paneGeneral);
    tabbedPane.addTab(getString("menuOptions"), paneOptions);
    panePrincipal.setLayout(new BoxLayout(panePrincipal, BoxLayout.Y_AXIS));
    panePrincipal.add(paneHaut);
    panePrincipal.add(tabbedPane);
    panePrincipal.add(paneBas);
    setContentPane(panePrincipal);
    getRootPane().setDefaultButton(btnExporter);    
    pack();
		rbDisplayMode.setSelected(true);
		showOptions(false);
	}
	
	private void btnCancelActionPerformed() {
		dispose();
	}
	
  private void btnExporterActionPerformed() {
  	int width, height;
  	try {
  		width = Integer.parseInt(widthEditor.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			widthEditor.requestFocus();
			return;
		}
  	try {
  		height = Integer.parseInt(heightEditor.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			heightEditor.requestFocus();
			return;
		}
  	boolean bCreationMode = rbCreationMode.isSelected();
    URL adr = MtgFrame.class.getResource(bCreationMode ? "export/exportCre.htm" : "export/exportVis.htm");
    try {
			InputStream is = adr.openStream();
			InputStreamReader ipsr=new InputStreamReader(is);
			BufferedReader br = new BufferedReader(ipsr);
			StringBuffer sb = new StringBuffer(10000); // Gros car peut contenir des images
			String s;
			while ((s = br.readLine()) !=null){
				sb.append(s + "\r\n");
			}
			br.close();
			String texte = sb.toString();
			int ind;
			// Version 4.4.1 : On le fait une premire fois pour le width de <Table
			ind = texte.indexOf("*width=");
			texte = texte.substring(0, ind) + "width=" + "\"" + width + "\"" + texte.substring(ind + 7 );
			ind = texte.indexOf("*width=");
			texte = texte.substring(0, ind) + "width=" + width + texte.substring(ind + 7 );
			ind = texte.indexOf("*height=");
			texte = texte.substring(0, ind) + "height=" + height + texte.substring(ind + 8 );

			ind = texte.indexOf("*codebase=");
			texte = texte.substring(0, ind) + "codebase=" + (cbIncludeJarFile.isSelected() ? "\"./\"" :
				"\"http://www.mathgraph32.org/webstart/" + MtgFrame.numeroVersion + "/\"") + texte.substring(ind + 10);
			String nomAuteur = Kernel.convertStringToHTML(authorNameEditor.getText());
			ind = texte.indexOf("*auteur");
			texte = texte.substring(0, ind) + getString("auteur") + " " + nomAuteur + texte.substring(ind + 7);
			String title = Kernel.convertStringToHTML(titleEditor.getText());
			// On met en place le titre de la page
			ind = texte.indexOf("<title>");
			texte = texte.substring(0, ind + 7) + title + " - " + getString("feuilleTravail") + texte.substring(ind + 7);
			ind = texte.indexOf("*title");
			texte = texte.substring(0, ind) + title + texte.substring(ind + 6);
			String texte1 = extraitCodeHTML(textEditor1.getText());
			ind = texte.indexOf("*texte1");
			texte = texte.substring(0, ind) + texte1 + texte.substring(ind + 7);
			String texte2 = extraitCodeHTML(textEditor2.getText());
			ind = texte.indexOf("*texte2");
			texte = texte.substring(0, ind) + texte2 + texte.substring(ind + 7);
			ind = texte.indexOf("*creeAvec");
			texte = texte.substring(0, ind) + getString("creeAvec") + texte.substring(ind + 9);
			ind = texte.indexOf("*commentaireJavaAbsent");
			texte = texte.substring(0, ind) + getString("commentaireJavaAbsent") + texte.substring(ind + 22);
			if (bCreationMode) {
				ind = texte.indexOf("*allowLeftToolbar");
				texte = texte.substring(0, ind) + (cbAllowLeftToolBar.isSelected() ? strue : sfalse) + texte.substring(ind + 17);
				ind = texte.indexOf("*allowTopToolbar");
				texte = texte.substring(0, ind) + (cbAllowTopToolBar.isSelected() ? strue : sfalse) + texte.substring(ind + 16);
				ind = texte.indexOf("*allowRightToolbar");
				texte = texte.substring(0, ind) + (cbAllowRightToolBar.isSelected() ? strue : sfalse) + texte.substring(ind + 18);
				ind = texte.indexOf("*allowToolsChoice");
				texte = texte.substring(0, ind) + (cbAllowToolsChoice.isSelected() ? strue : sfalse) + texte.substring(ind + 17);
				ind = texte.indexOf("*allowMenuBar");
				texte = texte.substring(0, ind) + (cbAllowMenuBar.isSelected() ? strue : sfalse) + texte.substring(ind + 13);
				ind = texte.indexOf("*allowFileMenu");
				texte = texte.substring(0, ind) + (cbAllowFileMenu.isSelected() ? strue : sfalse) + texte.substring(ind + 14);
				ind = texte.indexOf("*allowOptionsMenu");
				texte = texte.substring(0, ind) + (cbAllowOptionsMenu.isSelected() ? strue : sfalse) + texte.substring(ind + 17);
				ind = texte.indexOf("*code=");
				texte = texte.substring(0, ind) + "code=\"mathgraph32.MtgFrame.class\"" + texte.substring(ind + 6);
				ind = texte.indexOf("*archive=");
				texte = texte.substring(0, ind) + "archive=\"MathGraph32Applet.jar\"" + texte.substring(ind + 9);
				int level = 3;
				if (rbNiveau1.isSelected()) level = 1;
				else if (rbNiveau2.isSelected()) level = 2;
				else if (rbNiveau3.isSelected()) level = 3;
				else if (rbNiveau4.isSelected()) level = 4;
				ind = texte.indexOf("*level");
				texte = texte.substring(0, ind) + Integer.toString(level) + texte.substring(ind + 6);
				
			}
			else {
				ind = texte.indexOf("*code=");
				texte = texte.substring(0, ind) + "code=\"Applet.AppletMathGraph32.class\"" + texte.substring(ind + 6);
				ind = texte.indexOf("*archive=");
				texte = texte.substring(0, ind) + "archive=\"AppletMathGraph32.jar\"" + texte.substring(ind + 9);
			}
			// Le code Base64 dcrivant la figure
			ind = texte.indexOf("*figureData");
			texte = texte.substring(0, ind) + frame.cadreActif().getXML() + texte.substring(ind + 11);
			// On enregistre le fichier html modifi
			JFileChooser fc = new JFileChooser(frame.pref_constDirectory);
			fc.setFileFilter(new MyFilter());
			String name = frame.cadreActif().nomDocument();
			int indpoint = name.lastIndexOf(".");
			if (indpoint != -1) name = name.substring(0, name.lastIndexOf("."));
  		File fileInit = new File(frame.repertoireFichiers + File.separator + name + Global.GLOBAL_COMMA_HTML);
  		fc.setSelectedFile(fileInit);
	    if(fc.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
	  		File file = fc.getSelectedFile();
				if (file != null) {
					String ch = file.getAbsolutePath();
					if (!ch.toLowerCase().endsWith(Global.GLOBAL_COMMA_HTML)) {
						ch = ch + Global.GLOBAL_COMMA_HTML;
						file = new File(ch);
					}
					if (!MtgFileManager.accepteEnregistrement(frame, ch)) return;
					FileOutputStream fos;
					try {
						fos = new FileOutputStream(file);
						OutputStreamWriter opsr=new OutputStreamWriter(fos);
						BufferedWriter bfw =new BufferedWriter(opsr);
						bfw.write(texte);
						bfw.close();
						if (cbIncludeJarFile.isSelected()) {
					    URL adrApplet = MtgFrame.class.getResource(bCreationMode ? "export/MathGraph32Applet.jar"
					    		: "export/AppletMathGraph32.jar");
							BufferedInputStream buff = new BufferedInputStream(adrApplet.openStream());
							String path = file.getAbsolutePath();
							path = path.substring(0, path.lastIndexOf(File.separator));
							File fileApplet = new File( path + File.separator + (bCreationMode ? "MathGraph32Applet.jar" : "AppletMathGraph32.jar"));
							FileOutputStream fosJar = new FileOutputStream(fileApplet);
						  byte bytebuf[]=new byte[4096];
						  int len;
						  while((len=buff.read(bytebuf))>0)	fosJar.write(bytebuf,0,len);
						  fosJar.close();
						  buff.close();						  
						}
						// Si la case est reste coche, on lance le navigateur par dfaut avec le fichier html cr
						if (cbLaunch.isSelected()) {
							try {
								String path = file.getAbsolutePath();
								path = path.replaceAll(" ", "%20");
								// Pas d'appel  replaceAll  cause des caractres antislash
								StringBuffer buf = new StringBuffer(path.length());
								for (int i = 0; i < path.length();i++) {
									char car = path.charAt(i);
									if (car == '\\') car = '/';
									buf.append(car);
								}
								path= buf.toString();
								Desktop.getDesktop().browse(new URI("file:///" + path));
							} catch (Exception e2) {
							}
						}
					} catch (IOException e1) {
						frame.dialogueAvertissement(getString("ch45"));
						return;
					}
					
				}
	    }    
	    dispose();			
			
		} catch (IOException e) {
		}
    
  }
	
  private final static String extraitCodeHTML(String ch) {
  	String ch1 = ch.toLowerCase();
  	int ind1 = ch1.indexOf("<body");
  	int ind2 = ch1.indexOf("</body>");
  	String ch2 = ch.substring(ch1.indexOf(">", ind1)+1, ind2);
  	ind1 = ch2.indexOf("<p>");
  	ind2 = ch2.lastIndexOf("</p>");
  	return ch2.substring(ind1 + 1, ind2);  	
  }

  // Modifi version 4.4.5 pour que par dfaut on propose la taille relle de l'applet
  private void rbDisplayModeActionPerformed() {
  	MtgInternalFrame cadre = frame.cadreActif();
    widthEditor.setText(Integer.toString(cadre.dimf.longueurAbscisses));  	 	
    heightEditor.setText(Integer.toString(cadre.dimf.longueurOrdonnees));  	
  	showOptions(false);
  }
  
  // Modifi pour la version 4.4.5 pour que par dfaut elle propose 800 sur 600 pour une exportation en mode crtion
  private void rbCreationModeActionPerformed() {
    widthEditor.setText("800");  	 	
    heightEditor.setText("600");
  	showOptions(true);
  }
  
  private final void showOptions(boolean b) {
  	cbAllowLeftToolBar.setVisible(b);
  	cbAllowTopToolBar.setVisible(b);
  	cbAllowRightToolBar.setVisible(b);
  	cbAllowMenuBar.setVisible(b);
  	cbAllowToolsChoice.setVisible(b);
  	cbAllowFileMenu.setVisible(b);
  	cbAllowOptionsMenu.setVisible(b);
  	paneNiveau.setVisible(b);
  }
   
}
