/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.util.ListIterator;
import java.util.Vector;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCalcul;
import objets.CElementBase;
import objets.CListeObjets;
import objets.CValeurDynamique;
import types.NatureCalcul;

@SuppressWarnings("serial")
public class ListeObjetsNumeriquesFinauxPourConst extends JList<String> {
  private MtgFrame frame;
  public Vector<CElementBase> pointeurs;

  // Constructeur remplissant la liste avec les objets de listeInit qui peuvent tre des objets sources
  public ListeObjetsNumeriquesFinauxPourConst(MtgFrame frameInit) {
    super();
    frame = frameInit;
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    CListeObjets listeSources = new CListeObjets();
    listeSources.ajouteElementsDe(cadre.listeObjetsSourcesG);
    listeSources.ajouteElementsDe(cadre.listeObjetsSourcesNG);
    // Modifi version 3.4.0
    // Les calculs ou fonctions constants et dont dpend au moins un objet dpendant de au moins un objet source
    // sont ajouts  la liste des objest sources.
  	// objets finaux
  	// On fait de mme avec les variables pour que les variables servant 
  	// dfinir les sommes indices soient rajoutes
    pointeurs = new Vector<CElementBase>();
    Vector<String> chaines = new Vector<String>();
    setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    ListIterator<CElementBase> li = liste.listIterator();
    int i = -1;
    while (li.hasNext()) {
    	i++;
    	CElementBase elb = li.next();
    	if (elb.estDeNatureCalcul(NatureCalcul.NVariable)) {
    		CValeurDynamique val = (CValeurDynamique) elb;
    		if (!listeSources.contains(val)) {
    			for (int j = i+1; j < liste.size(); j++) {
    				CElementBase el2 = liste.get(j);
    				if (el2.dependDe(val) && el2.dependDeAuMoinsUn(listeSources))
    					listeSources.add(elb);
    			}
    		}
    	}
    	else {
      	if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculOuFonctionReelleOuComplexe)) {
      		CCalcul val = (CCalcul) elb;
      		if (val.estConstant()) {
	      		if (!listeSources.contains(val)) {
	      			for (int j = i+1; j < liste.size(); j++) {
	      				CElementBase el2 = liste.get(j);
	      				if (el2.dependDe(val) && el2.dependDeAuMoinsUn(listeSources))
	      					listeSources.add(elb);
	      			}
	      		}
      		}
      	}
    	}
    }
    // Modification version 3.1.5 : On rajoute aussi la possibilit de chosir comme objet final une variable qui
    // a t choisie dans les objets sources. Si cette variable est choisie, elle sera retire des objets sources
    li = liste.listIterator();
    while (li.hasNext()) {
    	CElementBase elb = li.next();
    	// On accepte aussi les repres comme objets numriques
    	if (elb.estDeNatureCalcul(NatureCalcul.NObjetFinalNumerique) && !elb.estElementIntermediaire()) {
    		if (elb.getNatureCalcul() == NatureCalcul.NVariable) {
    			if (listeSources.contains(elb)) {
    				pointeurs.add(elb);
    				chaines.add(elb.nom());
    			}
    		}
    		else {
	     		if ((elb.estDefiniParObjetDans(listeSources)) && !listeSources.contains(elb)) {
	    			pointeurs.add(elb);
	    			chaines.add(elb.nom());
	    		}
    		}
    	}
    }
    setListData(chaines);
		// On slectionne les lements dj choisis
    ListIterator<CElementBase> li2 = pointeurs.listIterator();
    i = 0;
    while (li2.hasNext()) {
    	CElementBase el = li2.next();
    	if (cadre.listeObjetsFinauxNG.contains(el)) addSelectionInterval(i, i);;
    	i++;
    }    
  }
}
