/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import mathgraph32.MtgFrame;
import objets.CCalculBase;
import objets.CListeObjets;
import objets.CMesureLongueurLigne;

@SuppressWarnings("serial")
public class MesureLongueurLigneDlg extends MtgDlg {
	private CMesureLongueurLigne mesure;
	private EditeurNomCalcul editeurNom;
	private JLabel label1; 
	private JButton btnOK, btnCancel;
	
	
  public MesureLongueurLigneDlg(MtgFrame frameInit, CMesureLongueurLigne mesureInit) {
    super (getString("mesureLongueurLigneDlg1"), true, frameInit);
    mesure = mesureInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
    editeurNom.requestFocus();
  }
	
	public void initComponents() {
		label1 = new JLabel(getString("mesureLongueurLigneDlg2"));
		editeurNom = new EditeurNomCalcul();
	  btnOK = new javax.swing.JButton(getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new javax.swing.JButton(getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());	
    JPanel paneRight = new JPanel();
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    getContentPane().setLayout(new FlowLayout());
    getContentPane().add(label1);
    getContentPane().add(editeurNom);
    getContentPane().add(paneRight);
    pack();
	}
	
  private void btnOKActionPerformed() {
  	CListeObjets listePr = frame.cadreActif().listePr;
    String chNom = editeurNom.getText();
    // On vrifie d'abord la validit du nom de la fonction
    if ((chNom.length() == 0) || !EditeurNomCalcul.validationNomVariableOuCalcul(chNom)) {
      dialogueAvertissement(getString("ch1"));
      editeurNom.requestFocus();
      return;
    }
    if (chNom.equals("i")) {
      dialogueAvertissement(getString("ch2"));
      editeurNom.requestFocus();
      return;
    }
    if (listePr.egalNomValeurOuFonctionOuMesure(chNom, listePr.size()-1)) {
      dialogueAvertissement(getString("ch3"));
      editeurNom.requestFocus();
      return;
    }
    if (CCalculBase.commenceParNomFonctionPredefinie(chNom)) {
      dialogueAvertissement(getString("ch91"));
      editeurNom.requestFocus();
      return;	    	
    }
    mesure.nomCalcul = chNom;
  	setRetourOK(true);
  	dispose();
    
  }

  private void btnCancelActionPerformed() {
  	dispose();
  }
  
}
