/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import objets.CListeObjets;
import objets.CValeurAssocieeAVariable;
import types.DimensionFenetre;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class NouvelleFigureAvecRepereDlg extends MtgDlg {
	private CListeObjets liste; // La liste  laquelle rajouter le repre
	private boolean avecVecteurs;
	private JButton btnOK, btnCancel;
	private JLabel label1, label2, label3, label4, label5, label6, label7, label8, label9;
	private JRadioButton radioType1,radioType2, radioType3, radioType4;
	private JRadioButton radioQuad1, radioQuad2, radioQuad3;
	private JPanel paneType, paneQuad;
	private JTextField editeurNomO, editeurNomI, editeurNomJ, editeurVecti, editeurVectj;
	// Modification version 4.6.3 : On ajoute une case  cocher pour un repre avec vecteurs
	private JCheckBox cbRepereAvecVect;
  private EditeurValeurDynamique editeurOrigineAbscisses;
  private EditeurValeurDynamique editeurOrigineOrdonnees;
  private EditeurValeurPositiveStrict editeurUnitex;
  private EditeurValeurPositiveStrict editeurUnitey;
  private CValeurAssocieeAVariable valOrigineAbscisses;
  private CValeurAssocieeAVariable valOrigineOrdonnees;
  private CValeurAssocieeAVariable valUnitex;
  private CValeurAssocieeAVariable valUnitey;
  // Ajout version 4.5
  public JRadioButton rbSansGraduation, rbGraduationSimple, rbGraduationTrigo;
  private JPanel paneBas; // Ajout version 4.6.3
  
	
  public NouvelleFigureAvecRepereDlg(MtgFrame frameInit, CListeObjets listeInit) {
    super(getString("nouveauRepereDlg0"), true, frameInit);
    liste = listeInit;
    initComponents();
    editeurNomO.setText("O");
    editeurNomI.setText("I");
    editeurNomJ.setText("J");
    radioType1.setSelected(true);
    // Spcial dyspraxie : quadrillage par dfaut
    radioQuad2.setSelected(true);
    
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents() {
    valOrigineAbscisses = new CValeurAssocieeAVariable(liste,0);
    valOrigineOrdonnees = new CValeurAssocieeAVariable(liste,0);
    valUnitex = new CValeurAssocieeAVariable(liste,1);
    valUnitey = new CValeurAssocieeAVariable(liste,1);
  	
    label1 = new JLabel(getString("nouveauRepereDlg1"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new JLabel(getString("nouveauRepereDlg2"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new JLabel(getString("nouveauRepereDlg3"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    label6 = new JLabel(getString("repereDlg2"));
    label7 = new JLabel(getString("repereDlg3"));
    label8 = new JLabel(getString("repereDlg10"));
    label9 = new JLabel(getString("repereDlg11"));
    editeurNomO = new JTextField();
    editeurNomO.setColumns(8);
    editeurNomI = new JTextField();
    editeurNomI.setColumns(8);
    editeurNomJ = new JTextField();
    editeurNomJ.setColumns(8);
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    cbRepereAvecVect = new JCheckBox(getString("repAvecVect"),false);
    cbRepereAvecVect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				avecVecteurs = cbRepereAvecVect.isSelected();
				paneBas.setVisible(avecVecteurs);
				pack();
			}
		});
    editeurOrigineAbscisses = new EditeurValeurDynamique(frame, valOrigineAbscisses, null);
    editeurOrigineAbscisses.setText("0");
    editeurOrigineOrdonnees = new EditeurValeurDynamique(frame, valOrigineOrdonnees, null);
    editeurOrigineOrdonnees.setText("0");
    editeurUnitex = new EditeurValeurPositiveStrict(frame, valUnitex, null);
    editeurUnitex.setText("1");
    editeurUnitey = new EditeurValeurPositiveStrict(frame, valUnitey, null);
    editeurUnitey.setText("1");
    
    ButtonGroup group1 = new ButtonGroup();
    radioType1=new JRadioButton(getString("nouveauRepereDlg5"));
    radioType2=new JRadioButton(getString("nouveauRepereDlg6"));
    radioType3=new JRadioButton(getString("nouveauRepereDlg7"));
    radioType4=new JRadioButton(getString("nouveauRepereDlg8"));
    group1.add(radioType1);
    group1.add(radioType2);
    group1.add(radioType3);
    group1.add(radioType4);
    
    rbSansGraduation = new JRadioButton(getString("repereDlg12")); // Slectionn par dafut : sans geaduation
    rbGraduationSimple = new JRadioButton(getString("repereDlg13"), true);
    rbGraduationTrigo = new JRadioButton(getString("repereDlg14"));
    ButtonGroup group3 = new ButtonGroup();
    group3.add(rbSansGraduation);
    group3.add(rbGraduationSimple);
    group3.add(rbGraduationTrigo);
    
    paneType = new JPanel();
		paneType.setBorder(BorderFactory.createTitledBorder(getString("nouveauRepereDlg4")));
		paneType.setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 0;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    setConstraints(constraints, 0, 0, 1, 1, 50, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneType.add(radioType1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneType.add(radioType2, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneType.add(radioType3, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneType.add(radioType4, constraints);
		
    ButtonGroup group2 = new ButtonGroup();
    radioQuad1=new JRadioButton(getString("nouveauRepereDlg10"));
    radioQuad2=new JRadioButton(getString("nouveauRepereDlg11"));
    radioQuad3=new JRadioButton(getString("nouveauRepereDlg12"));
    group2.add(radioQuad1);
    group2.add(radioQuad2);
    group2.add(radioQuad3);
    paneQuad = new JPanel();
    paneQuad.setBorder(BorderFactory.createTitledBorder(getString("nouveauRepereDlg9")));
    paneQuad.setLayout(new GridBagLayout());
    setConstraints(constraints, 0, 0, 1, 1, 33, 100, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneQuad.add(radioQuad1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 33, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneQuad.add(radioQuad2, constraints);
    setConstraints(constraints, 2, 0, 1, 1, 33, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneQuad.add(radioQuad3, constraints);
    
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    setConstraints(constraints, 0, 0, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 30, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(editeurNomO, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(editeurNomI, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(editeurNomJ, constraints);
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    panePrincipal.add(paneRight, constraints);
    
    setConstraints(constraints, 0, 3, 3, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(paneType, constraints);
    
    setConstraints(constraints, 0, 4, 3, 1, 0, 40, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(paneQuad, constraints);

    setConstraints(constraints, 0, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label6, constraints);
    setConstraints(constraints, 1, 5, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(editeurOrigineAbscisses, constraints);
    setConstraints(constraints, 0, 6, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label7, constraints);
    setConstraints(constraints, 1, 6, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(editeurOrigineOrdonnees, constraints);
    setConstraints(constraints, 0, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label8, constraints);
    setConstraints(constraints, 1, 7, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(editeurUnitex, constraints);
    setConstraints(constraints, 0, 8, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label9, constraints);
    setConstraints(constraints, 1, 8, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(editeurUnitey, constraints);
    paneBas = new JPanel();
    paneBas.setLayout(new GridBagLayout());
    paneBas.setVisible(avecVecteurs);
    int lig = 8;
  	lig++;
    label4 = new JLabel(getString("nouveauRepereDlg13"), JLabel.RIGHT);
    label4.setRequestFocusEnabled(false);
    label5 = new JLabel(getString("nouveauRepereDlg14"), JLabel.RIGHT);
    label5.setRequestFocusEnabled(false);
    editeurVecti = new JTextField();
    editeurVecti.setColumns(4);
    editeurVecti.setText("i");
    editeurVectj = new JTextField();
    editeurVectj.setColumns(4);
    editeurVectj.setText("j");
    setConstraints(constraints, 0, 0, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    paneBas.add(label4, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    paneBas.add(editeurVecti, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    paneBas.add(label5, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    paneBas.add(editeurVectj, constraints);
    
    JPanel paneGraduation = new JPanel();
    paneGraduation.setLayout(new GridBagLayout());
    setConstraints(constraints, 0, 0, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneGraduation.add(rbSansGraduation, constraints);    
    setConstraints(constraints, 1, 0, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneGraduation.add(rbGraduationSimple, constraints);    
    setConstraints(constraints, 2, 0, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneGraduation.add(rbGraduationTrigo, constraints);
    
    setConstraints(constraints, 0, lig + 2, 4, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    panePrincipal.add(paneGraduation, constraints);
    setConstraints(constraints, 0, lig + 3, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    panePrincipal.add(cbRepereAvecVect, constraints);
    setConstraints(constraints, 0, lig + 4, 4, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    panePrincipal.add(paneBas, constraints);
    setContentPane(panePrincipal);
    pack(); 		    
  }
  // Modifi version 4.7.7 pour les appels  verifieValidite qui doivent utiliser le paramtre liste
  // car sinon plantage quand on voulait crer uen figure quand aucune n'est ouverte
  private void btnOKActionPerformed() {
    String chNomO = editeurNomO.getText();
    if (chNomO.length() == 0) {
      dialogueAvertissementCentreEcran(getString("ch36"));
      editeurNomO.setSelectionStart(0);
      editeurNomO.setSelectionEnd(chNomO.length());
      editeurNomO.requestFocus();
      return;
    }
    if (!liste.validationNomPoint(chNomO)) {  
      dialogueAvertissementCentreEcran(getString("ch19"));
      editeurNomO.setSelectionStart(0);
      editeurNomO.setSelectionEnd(chNomO.length());
      editeurNomO.requestFocus();
      return;    	
    }
    String chNomI = editeurNomI.getText();
    if (chNomI.length() == 0) {
      dialogueAvertissementCentreEcran(getString("ch36"));
      editeurNomI.setSelectionStart(0);
      editeurNomI.setSelectionEnd(chNomI.length());
      editeurNomI.requestFocus();
      return;
    }
    if (!liste.validationNomPoint(chNomI)) {  
      dialogueAvertissementCentreEcran(getString("ch19"));
      editeurNomI.setSelectionStart(0);
      editeurNomI.setSelectionEnd(chNomI.length());
      editeurNomI.requestFocus();
      return;    	
    }
    String chNomJ = editeurNomJ.getText();
    if (chNomJ.length() == 0) {
      dialogueAvertissementCentreEcran(getString("ch36"));
      editeurNomJ.setSelectionStart(0);
      editeurNomJ.setSelectionEnd(chNomJ.length());
      editeurNomJ.requestFocus();
      return;
    }
    if (!liste.validationNomPoint(chNomJ)) {  
      dialogueAvertissementCentreEcran(getString("ch19"));
      editeurNomJ.setSelectionStart(0);
      editeurNomJ.setSelectionEnd(chNomJ.length());
      editeurNomJ.requestFocus();
      return;    	
    }
    String chVecti;
    if (avecVecteurs) {
    	chVecti = editeurVecti.getText();
    	if (chVecti.length() == 0) chVecti = "i";
    }
    else chVecti = "";
    String chVectj;
    if (avecVecteurs) {
    	chVectj = editeurVectj.getText();
    	if (chVectj.length() == 0) chVectj = "j";
    }
    else chVectj = "";
    
    if (editeurOrigineAbscisses.verifieValidite(liste)) {
    	if (editeurOrigineOrdonnees.verifieValidite(liste)) {
    		if (editeurUnitex.verifieValidite(liste)) {
    			if (editeurUnitey.verifieValidite(liste)) {
				    
				    // Tout est correct. On rajoute le repre la la liste principale
				    setRetourOK(true);
				    DimensionFenetre dim;
						if (frame.cadreActif() != null) {
							if (frame.cadreActif().isMaximum()) {
								dim = new DimensionFenetre(frame.cadreActif().paneFigure.getWidth(), frame.cadreActif().paneFigure.getHeight());
							}
							else dim = MtgInternalFrame.dimNouvelleFigure;
						}
						else dim = MtgInternalFrame.dimNouvelleFigure;
				    
				    if (radioType1.isSelected()) {
				    	liste.ajouteRepereOrthonormal(dim, radioQuad2.isSelected(), radioQuad2.isSelected(),
				    			radioQuad3.isSelected(), chNomO, chNomI, chNomJ, chVecti, chVectj, valOrigineAbscisses,
				    			valOrigineOrdonnees, valUnitex, valUnitey, !rbSansGraduation.isSelected());
				    	dispose();
				    	return;
				    }
				    if (radioType2.isSelected()) {
				    	liste.ajouteRepereOrthogonal(dim, radioQuad2.isSelected(), radioQuad2.isSelected(),
				    			radioQuad3.isSelected(), chNomO, chNomI, chNomJ, chVecti, chVectj, valOrigineAbscisses,
				    			valOrigineOrdonnees, valUnitex, valUnitey, !rbSansGraduation.isSelected());
				    	dispose();
				    	return;
				    }
				    if (radioType3.isSelected()) {
				    	liste.ajouteRepereObliqueQuelconque(dim, radioQuad2.isSelected(), radioQuad2.isSelected(),
				    			radioQuad3.isSelected(), chNomO, chNomI, chNomJ, chVecti, chVectj, valOrigineAbscisses,
				    			valOrigineOrdonnees, valUnitex, valUnitey, !rbSansGraduation.isSelected());
				    	dispose();
				    	return;
				    }
				    if (radioType4.isSelected()) {
				    	liste.ajouteRepereObliqueNorme(dim, radioQuad2.isSelected(), radioQuad2.isSelected(),
				    			radioQuad3.isSelected(), chNomO, chNomI, chNomJ, chVecti, chVectj, valOrigineAbscisses,
				    			valOrigineOrdonnees, valUnitex, valUnitey, !rbSansGraduation.isSelected());
				    	dispose();
				    }
    			}
    		}
    	}
    }
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }

}
