/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import objets.CMathGrafDoc;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import mathgraph32.MtgMenu;

@SuppressWarnings("serial")
public class PaneOptionsFigureMenu extends PaneOptionsFigure {

	private MtgMenu menuFichier, menuEdition, menuCreer, menuCalculs, menuModifier, menuMouvoir;
	private MtgMenu menuMacros, menuConstructions;
	private JLabel label1;
	private JRadioButton radioItemCochesInterdits;
	private JRadioButton radioItemCochesPermis;
	private MtgMenuTree tree;
	private DefaultTreeModel model;
	private DefaultMutableTreeNode principalNode;
	private JScrollPane scrollPaneTree;
	private ButtonGroup btnGroup;
	private JButton btnCocherTout;
	private JButton btnDecocherTout;
	private boolean bInitialItemCochesInterdits;
	private DefaultMutableTreeNode nodeFichier;
	private DefaultMutableTreeNode nodeEdition;
	private DefaultMutableTreeNode nodeCreer;
	private DefaultMutableTreeNode nodeCalculs;
	private DefaultMutableTreeNode nodeModifier;
	private DefaultMutableTreeNode nodeMouvoir;
	private DefaultMutableTreeNode nodeMacros;
	private DefaultMutableTreeNode nodeConstructions;
	
	
	public PaneOptionsFigureMenu(MtgInternalFrame cadreInit) {
		super(cadreInit);
		initComponents();
		bInitialItemCochesInterdits = radioItemCochesInterdits.isSelected();
	}
	
	private void initComponents() {
		label1 = new JLabel(MtgDlg.getString("preferencesMenu1"));
		btnGroup = new ButtonGroup();
		radioItemCochesInterdits = new JRadioButton(MtgDlg.getString("preferencesMenu2"));
		// Ajout version 4.3. Si on clique sur le bouton on dcochera tout.
		radioItemCochesInterdits.addActionListener(new ActionListener() {	
			public void actionPerformed(ActionEvent arg0) {
				btnDecocherToutPerformed(true);			
			}
		});
		btnGroup.add(radioItemCochesInterdits);
		radioItemCochesPermis = new JRadioButton(MtgDlg.getString("preferencesMenu3"));
		// Ajout version 4.3. Si on clique sur le bouton on cochera tout.
		radioItemCochesPermis.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btnCocherToutPerformed(true);
			}
		});
		btnGroup.add(radioItemCochesPermis);
		btnCocherTout = new JButton(MtgDlg.getString("btnCocherTout"));
		btnCocherTout.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCocherToutPerformed(false);
	    }
	  });    
		btnDecocherTout = new JButton(MtgDlg.getString("btnDecocherTout"));
		btnDecocherTout.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnDecocherToutPerformed(false);
	    }
	  });    
		prepareItems();
		tree.setCellRenderer(new RenduComposant());
		tree.setCellEditor(new EditComposant(tree));
		tree.setEditable(true);
		tree.setBackground(Color.white);

		scrollPaneTree = new JScrollPane(tree, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPaneTree.setPreferredSize(new Dimension(400, 200));
		radioItemCochesInterdits.setSelected(cadre.document.itemsCochesInterdits);
		radioItemCochesPermis.setSelected(!radioItemCochesInterdits.isSelected());
    setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 100, 60, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
    add(scrollPaneTree, constraints);
    MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(label1, constraints);
    MtgDlg.setConstraints(constraints, 0, 2, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(radioItemCochesInterdits, constraints);
    MtgDlg.setConstraints(constraints, 0, 3, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(radioItemCochesPermis, constraints);		
    MtgDlg.setConstraints(constraints, 0, 4, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    add(btnCocherTout, constraints);		
    MtgDlg.setConstraints(constraints, 0, 5, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    add(btnDecocherTout, constraints);		
	}
	
	
	private void prepareItems() {
		MtgFrame frame = cadre.frame;
		CMathGrafDoc docRef = frame.cadreActif().document;
		menuFichier = new MtgMenu(MtgDlg.getString("menuFichier"));
		menuEdition = new MtgMenu(MtgDlg.getString("menuEdition"));
		menuCreer = new MtgMenu(MtgDlg.getString("menuCreer"));
		menuModifier = new MtgMenu(MtgDlg.getString("menuModifier"));
		menuMouvoir = new MtgMenu(MtgDlg.getString("menuMouvoir"));
		menuCalculs = new MtgMenu(MtgDlg.getString("menuCalculs"));
		menuMacros = new MtgMenu(MtgDlg.getString("menuMacros"));
		menuConstructions = new MtgMenu(MtgDlg.getString("menuConstructions"));
		
		// Nouveau version 4.3 : On n'autorise que les items du niveau de fonctionnement choisi
		// Modif version 4.6 : Au niveau lmaximum tous les outils doivet tre permis
		// CMathGrafDoc doc = frame.docPreferencesNiveau[frame.pref_niveau];
		CMathGrafDoc doc = frame.pref_niveau > 1 ? frame.docPreferencesNiveau[frame.pref_niveau] : null;
		frame.menuFichierInitial.addCloneAvecCheck(menuFichier, doc, docRef);
		frame.menuEditionInitial.addCloneAvecCheck(menuEdition, doc, docRef);
		frame.menuCreerInitial.addCloneAvecCheck(menuCreer, doc, docRef);
		frame.menuCalculsInitial.addCloneAvecCheck(menuCalculs, doc, docRef);
		frame.menuModifierInitial.addCloneAvecCheck(menuModifier, doc, docRef);
		frame.menuMouvoirInitial.addCloneAvecCheck(menuMouvoir, doc, docRef);
		frame.menuMacrosInitial.addCloneAvecCheck(menuMacros, doc, docRef);
		frame.menuConstructionsInitial.addCloneAvecCheck(menuConstructions, doc, docRef);
		principalNode = new DefaultMutableTreeNode();
		creeNoeuds();
		model = new DefaultTreeModel(principalNode);
		tree = new MtgMenuTree(model);	
	}

	@Override
	public boolean sauveDonnees() {
		MtgFrame frame = cadre.frame;
		CMathGrafDoc doc = cadre.document;
		doc.initialiseItemsMenu();
		CMathGrafDoc docRef = new CMathGrafDoc(frame);	
		doc.itemsCochesInterdits = radioItemCochesInterdits.isSelected();
		docRef.itemsCochesInterdits = doc.itemsCochesInterdits;
		if (doc.itemsCochesInterdits) {
		// Nouveau version 4.3 : On interdit les items interdits pour le niveau en cours
			MtgMenu menuFichier0, menuEdition0, menuCreer0, menuCalculs0, menuModifier0, menuMouvoir0;
			MtgMenu menuMacros0, menuConstructions0;
			menuFichier0 = new MtgMenu(MtgDlg.getString("menuFichier"));
			menuEdition0 = new MtgMenu(MtgDlg.getString("menuEdition"));
			menuCreer0 = new MtgMenu(MtgDlg.getString("menuCreer"));
			menuModifier0 = new MtgMenu(MtgDlg.getString("menuModifier"));
			menuMouvoir0 = new MtgMenu(MtgDlg.getString("menuMouvoir"));
			menuCalculs0 = new MtgMenu(MtgDlg.getString("menuCalculs"));
			menuMacros0 = new MtgMenu(MtgDlg.getString("menuMacros"));
			menuConstructions0 = new MtgMenu(MtgDlg.getString("menuConstructions"));
			frame.menuFichierInitial.addCloneAvecCheck(menuFichier0, null, null);
			frame.menuEditionInitial.addCloneAvecCheck(menuEdition0, null, null);
			frame.menuCreerInitial.addCloneAvecCheck(menuCreer0, null, null);
			frame.menuCalculsInitial.addCloneAvecCheck(menuCalculs0, null, null);
			frame.menuModifierInitial.addCloneAvecCheck(menuModifier0, null, null);
			frame.menuMouvoirInitial.addCloneAvecCheck(menuMouvoir0, null, null);
			frame.menuMacrosInitial.addCloneAvecCheck(menuMacros0, null, null);
			frame.menuConstructionsInitial.addCloneAvecCheck(menuConstructions0, null, null);
			menuFichier0.ajouteTousItemsADocument(doc);
			menuEdition0.ajouteTousItemsADocument(doc);
			menuCreer0.ajouteTousItemsADocument(doc);
			menuCalculs0.ajouteTousItemsADocument(doc);
			menuModifier0.ajouteTousItemsADocument(doc);
			menuMouvoir0.ajouteTousItemsADocument(doc);
			menuMacros0.ajouteTousItemsADocument(doc);
			menuConstructions0.ajouteTousItemsADocument(doc);
		}
		
		menuFichier.adapteDocument(doc);
		menuEdition.adapteDocument(doc);
		menuCreer.adapteDocument(doc);
		menuCalculs.adapteDocument(doc);
		menuModifier.adapteDocument(doc);
		menuMouvoir.adapteDocument(doc);
		menuMacros.adapteDocument(doc);
		menuConstructions.adapteDocument(doc);
		
		return menuFichier.modifie() || menuEdition.modifie() || menuCreer.modifie() || menuCalculs.modifie()
			|| menuModifier.modifie() || menuMouvoir.modifie() || menuMacros.modifie()
			|| menuConstructions.modifie()
			|| (bInitialItemCochesInterdits != radioItemCochesInterdits.isSelected());
	}

	@Override
	public String titreOnglet() {
		// TODO Auto-generated method stub
		return MtgDlg.getString("ongletMenu");
	}
	
	private void creeNoeuds() {
		nodeFichier = new DefaultMutableTreeNode(menuFichier.getText());
		menuFichier.ajouteNoeud(nodeFichier);
		nodeEdition = new DefaultMutableTreeNode(menuEdition.getText());
		menuEdition.ajouteNoeud(nodeEdition);
		nodeCreer = new DefaultMutableTreeNode(menuCreer.getText());
		menuCreer.ajouteNoeud(nodeCreer);
		nodeCalculs = new DefaultMutableTreeNode(menuCalculs.getText());
		menuCalculs.ajouteNoeud(nodeCalculs);
		nodeModifier = new DefaultMutableTreeNode(menuModifier.getText());
		menuModifier.ajouteNoeud(nodeModifier);
		nodeMouvoir = new DefaultMutableTreeNode(menuMouvoir.getText());
		menuMouvoir.ajouteNoeud(nodeMouvoir);
		nodeMacros = new DefaultMutableTreeNode(menuMacros.getText());
		menuMacros.ajouteNoeud(nodeMacros);
		nodeConstructions = new DefaultMutableTreeNode(menuConstructions.getText());
		menuConstructions.ajouteNoeud(nodeConstructions);
		principalNode.add(nodeFichier);
		principalNode.add(nodeEdition);
		principalNode.add(nodeCreer);
		principalNode.add(nodeCalculs);
		principalNode.add(nodeMouvoir);
		principalNode.add(nodeModifier);
		principalNode.add(nodeMacros);
		principalNode.add(nodeConstructions);
	}
	
	private void btnCocherToutPerformed(boolean verbose) {
		boolean b = radioItemCochesInterdits.isSelected();
		menuFichier.selectAll(b);
		menuEdition.selectAll(b);
		menuCreer.selectAll(b);
		menuCalculs.selectAll(b);
		menuModifier.selectAll(b);
		menuMouvoir.selectAll(b);
		menuMacros.selectAll(b);
		menuConstructions.selectAll(b);
		principalNode.removeAllChildren();
		creeNoeuds();
		tree.updateUI();
		if (verbose) cadre.frame.dialogueAvertissement(MtgDlg.getString("outilsReinitialises"));
	}
	

	
	private void btnDecocherToutPerformed(boolean verbose) {
		boolean b = !radioItemCochesInterdits.isSelected();
		menuFichier.deselectAll(b);
		menuEdition.deselectAll(b);
		menuCreer.deselectAll(b);
		menuCalculs.deselectAll(b);
		menuModifier.deselectAll(b);
		menuMouvoir.deselectAll(b);
		menuMacros.deselectAll(b);
		menuConstructions.deselectAll(b);
		principalNode.removeAllChildren();
		creeNoeuds();
		tree.updateUI();
		if (verbose) cadre.frame.dialogueAvertissement(this, MtgDlg.getString("outilsReinitialises"));
	}
}

