/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import types.NatureCarSpe;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class TexteFigureDlg extends MtgDlg {
	private JLabel label1;
	private JTextArea editeurTexte;
	private JScrollPane scrollPaneAffichage;
	private JButton btnOK, btnCancel;
	private boolean modification; // true si on modifie le texte de la figure
	
  public TexteFigureDlg(MtgFrame frameInit, boolean modificationInit) {
    super(getString("texteFigureDlg1"), true, frameInit);
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
    editeurTexte.requestFocus();
  }
  
  private void initComponents() {
  	if (modification) label1 = new JLabel(getString("texteFigureDlg2"));
  	else label1 = new JLabel(getString("texteFigureDlg1"));
 		if (modification) editeurTexte = new TextAreaAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
 		else editeurTexte = new JTextArea();
		// editeurTexte.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1)); // Supprim version 4.7.5.2 pris en charge augmentation polices systme   
 		editeurTexte.setRows(18);
 		editeurTexte.setColumns(55);
 		editeurTexte.setText(frame.cadreActif().document.TexteFigure);
 		if (!modification) editeurTexte.setEditable(false);
    scrollPaneAffichage = new JScrollPane
  		(editeurTexte, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    if (modification) btnCancel = new javax.swing.JButton(getString("btnCancel"));
    else btnCancel = new JButton(getString("btnFermer"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());  	
  	
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.top = 4;
    constraints.insets.bottom = 4;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 90, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(scrollPaneAffichage, constraints);
    
    JPanel paneRight = new JPanel();
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    if (modification) paneRight.add(btnOK);
    paneRight.add(btnCancel);
    setConstraints(constraints, 2, 0, 1, 4, 20, 50, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    pane.add(paneRight, constraints);
    setContentPane(pane);
    pack();
  }
  
  private void btnOKActionPerformed() {
  	frame.cadreActif().document.TexteFigure = editeurTexte.getText();
  	setRetourOK(true);
  	dispose();
  	frame.activeOutilCapture();
  }

  private void btnCancelActionPerformed() {
  	dispose();
  	frame.activeOutilCapture();
  }
	
}
