/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.tree.DefaultMutableTreeNode;
import objets.CMathGrafDoc;
import actions.MtgAction;

@SuppressWarnings("serial")
public class MtgCheckBoxMenuItem extends JCheckBoxMenuItem {
	public MtgAction actionAssociee;
	public boolean etatInitial;
	
	// Constructeur  partir d'un item de menu pour Options figure en cours
  public MtgCheckBoxMenuItem(MtgMenuItem item, CMathGrafDoc docReference) {
    super(item.getText());
    actionAssociee = (MtgAction)item.getAction();
    // Modification version 4.3
    // etatInitial = docReference.itemContenu(actionAssociee.idMenu());
    etatInitial = (docReference == null) ? true : docReference.itemContenu(actionAssociee.idMenu());    
    setSelected(etatInitial);    
  }
  
  // Constructeur pour incorporation dans le menu principal
   public MtgCheckBoxMenuItem(MtgAction a, boolean isSelected) {
  	 super(a);
  	 setSelected(isSelected);
   }
 
    
  public MtgAction mtgActionAssociee() {
  	return actionAssociee;
  }
  
  public boolean modifie() {
  	return (isSelected() != etatInitial);
  }
	
  /**
   * Fonction adaptant listeIdMenus du document  cet item suivant la valeur itemsCochesInterdits de doc
   */
  public void adapteDocument(CMathGrafDoc doc) {
		int id = actionAssociee.idMenu();
		boolean b = doc.itemContenu(id);
  	if (isSelected()) {
  		if (!b) doc.listeIdMenus.add(new Integer(id));
  	}
  	else {
  		if (b) doc.retireItem(id);
  	}
  }
  
  public void ajouteNoeud(DefaultMutableTreeNode node) {
  	node.add(new DefaultMutableTreeNode(new MtgCheckBoxMenuItemForNode(this)));
  }

  /**
   * Fonction adaptant la visibilit de l'item aux outils permis dans doc et docNiveau
   * Attention : docNiveau peut tre nul
   * @param doc
   * @return
   */
  public boolean adapteADocument(CMathGrafDoc doc, CMathGrafDoc docNiveau) {
  	if (getAction() != null) {
  		MtgAction action = (MtgAction)getAction();
  		int id = action.idMenu();
  		boolean b1 = doc.itemPermis(id);
  		boolean b2 = (docNiveau == null) ? true : docNiveau.itemPermis(id);
  		if (b1 & b2) {
  			setVisible(true);
  			return true;
  		}
  		else {
  			setVisible(false);
  			return false;
  		}
  	}
  	else {
  		return false;
  	}
  }  
}
