/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;


import types.*;

import dialogs.AngleArcDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;



/** Version Java : cette classe a t optimise.
 * Dans Positionne, elle stocke dans valeurAngleAuCentre la valeur de l'angle au centre qui
 * est calcule dans Positionne()
 * */
public class CArcDeCercle extends CArcDeCercleAncetre {
  double ang1;// angle polaire de l'origine
  double ang2;// angle polaire de l'extremit
  public CValeurAngle angleAuCentre;
  CPointAncetre O;
  CPointAncetre A;
  /// nouveau membre depuis la version 1.8
  CPointAncetre pointFinArc;
  /** Creates new CArcDeCercle */
  public CArcDeCercle() {
    pointFinArc = null;
  }
  
  // Constructeur pour clonage
  public CArcDeCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, CPointAncetre OInit, CPointAncetre AInit,
    CPointAncetre pointFinArcInit, CValeurAngle angleAuCentreInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    O = OInit;
    A = AInit;
    pointFinArc = pointFinArcInit;
    angleAuCentre = angleAuCentreInit;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    ang1 = ((CArcDeCercle)ptel).ang1;
    ang2 = ((CArcDeCercle)ptel).ang2;
    valeurAngleAuCentre = ((CArcDeCercle)ptel).valeurAngleAuCentre;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(O);
    int ind2 = listeSource.indexOf(A);
    CPointAncetre ptFinArc = null;
    CValeurAngle angleAuCentreClone = null;
    if (pointFinArc == null)
      angleAuCentreClone = (CValeurAngle) angleAuCentre.getClone(listeSource, listeCible);
    else {
      int ind3 = listeSource.indexOf(pointFinArc);
      ptFinArc = (CPointAncetre)(listeCible.get(ind3,CPointAncetre.class));
    }
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CArcDeCercle(listeCible, (CImplementationProto)(listeCible.get(ind4,CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), ptFinArc, angleAuCentreClone);
    return ptelb;
  }
  
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    if (pointFinArc == null) {
    	angleAuCentre.positionne(infoRandom, dimfen);
      existe = O.existe && A.existe && angleAuCentre.existe();
      if (!existe) return;
      valeurAngleAuCentre = angleAuCentre.rendValeurRadian(infoRandom);
      if (Kernel.zeroAngle(valeurAngleAuCentre)) {
        existe = false;
        return;
      };
      VecteurDirecteur u = new VecteurDirecteur(O,A);
      rayon = u.norme();
      centre_x = O.x;
      centre_y = O.y;
      origine_x = A.x;
      origine_y = A.y;
      ang1 = u.angleRad();
      ang2 = ang1 + valeurAngleAuCentre;
      if (ang2 < 0) ang2 = ang2 + 2*Math.PI;
      if (ang2 >= 2*Math.PI) ang2 = ang2 - 2*Math.PI;
    } 
    else {
      existe = O.existe && A.existe && pointFinArc.existe;
      if (!existe) return;
      VecteurDirecteur u = new VecteurDirecteur(O,A);
      rayon = u.norme();
      centre_x = O.x;
      centre_y = O.y;
      origine_x = A.x;
      origine_y = A.y;
      ang1 = u.angleRad();
      VecteurDirecteur v = new VecteurDirecteur(O, pointFinArc);
      ang2 = v.angleRad();
      valeurAngleAuCentre = Kernel.mesurePrincipale(ang2 - ang1);
      if (Kernel.zeroAngle(valeurAngleAuCentre)) {
        existe = false;
        return;
      }
    }
    super.positionne(infoRandom, dimfen);
  }
  
  /** 
   * A t modifi par rapport au C++
   */
  @Override
	public boolean confonduAvec(CElementBase p) {
    CArcDeCercle pt;
    if (p.getClass() == this.getClass()) {
      pt = (CArcDeCercle) p;
  		if (pointFinArc == null)
  		{
  			return (O == pt.O) && (A == pt.A) && angleAuCentre.confonduAvec(pt.angleAuCentre);
  		}
  		else
  		{
  			return ((O == pt.O) && (A == pt.A) && (pointFinArc == pt.pointFinArc));
  		}
   }
    else return false;
	}
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("chinfo53");
    if (pointFinArc != null) ch = ch + "\n" + getString("chinfo54");
    else ch = ch + "\n" + getString("chinfo55") + " " + angleAuCentre.chaineInfo();
  	return ch;
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(O);
    liste.add(A);
    if (pointFinArc != null) liste.add(pointFinArc);
  } 
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    if (pointFinArc == null)
      return memoriseDependance(super.dependDe(p) || O.dependDe(p) || A.dependDe(p) || angleAuCentre.dependDe(p));
    else return memoriseDependance(super.dependDe(p) || O.dependDe(p) || A.dependDe(p) ||pointFinArc.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    if (pointFinArc == null)
      return super.dependDePourReclassement(p) || O.dependDePourReclassement(p)
      || A.dependDePourReclassement(p) || angleAuCentre.dependDePourReclassement(p);
    else return super.dependDePourReclassement(p) || O.dependDePourReclassement(p) 
    	|| A.dependDePourReclassement(p) ||pointFinArc.dependDePourReclassement(p);
  }
  
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    if (pointFinArc == null)
      return (p == this) || O.dependDePourBoucle(p) ||
        A.dependDePourBoucle(p) || angleAuCentre.dependDePourBoucle(p);
    else return (p == this) || O.dependDePourBoucle(p) ||
      A.dependDePourBoucle(p) ||pointFinArc.dependDePourBoucle(p);
  }

  // Modifi pour la version 1.9.5 car trace ne doit plus utiliser de pointeurs
  // sur les objets
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleTrait;
    if (horsFenetre) return;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    // Rajout en java par rapport  la version C++
    if (Kernel.testToile(centre_x, centre_y) && Kernel.testToile(origine_x, origine_y)) {
      pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
      pDC.setColor(couleur);
      Kernel.secteurSaillant(pDC, origine_x, origine_y, centre_x, centre_y, rayon, valeurAngleAuCentre);
    }
  }
  // Modifi pour la version 1.9.5 car surArc ne doit plus utiliser de pointeurs
  // sur les objets sources
  public boolean surArc(double xt, double yt) {
    double ang,ang1aux,ang2aux,dif;
    
    VecteurDirecteur w = new VecteurDirecteur(centre_x, centre_y, xt, yt);
    ang = w.angleRad();
    if (Kernel.zeroAngle(Math.abs(valeurAngleAuCentre)-Math.PI))
    {
      if (valeurAngleAuCentre > 0) // Cas d'un petit arc trac dans le sens direct
      {
        if (ang1 <= Math.PI)
                return (ang >=  ang1) && (ang <= ang1 + Math.PI);
        else return (ang >= ang1) || (ang <= ang1 - Math.PI);
      }
      else
      {
        if (ang1 <= Math.PI) return (ang >= ang1 + Math.PI) || (ang <= ang1);
        else return (ang <= ang1) && (ang >= ang1 - Math.PI);
      }
    }
    else
    {
      ang1aux = ang1;
      ang2aux = ang2;
      // On ordonne ang1Aux et ang2Aux dans l'ordre croissant
      if (ang1aux > ang2aux) {
        double d = ang2aux;
        ang2aux = ang1aux;
        ang1aux = d;
      }
      dif = ang2aux - ang1aux;
      if (dif < Math.PI) {
        return ((ang >= ang1aux) && (ang <= ang2aux));
        } 
      else {
        return ((ang <= ang1aux) || (ang >= ang2aux));
      }
    }
  }
  
  /** distancePoint renvoie -1 dans le cas o le l'objet n'est pas dsignable */
  public double distancePointPourSurface(double xp, double yp, boolean masquage) {
    if (!existe) return -1;
    else {
      boolean appSecteur = surArc(xp,yp);
      if (! appSecteur) return - 1;
      else return Math.abs(Kernel.distancePointPoint(centre_x,centre_y,xp,yp) - rayon);
   }      
  }

  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
 	 if (!(existe) || (masquage && masque)) return -1;
 	 else return distancePointPourSurface(xp,yp,masquage);
 }
  
  // Java : a t modifi par rapport au C++
  public double abscisseCurviligne(double ang) {
    double difang;
    
    if (valeurAngleAuCentre == 0) return 0;
    difang = Math.abs(ang - ang1);
    if (difang > Math.PI) difang = 2*Math.PI - difang;
    return difang / Math.abs(valeurAngleAuCentre);
  }
  
  public double abscisseMaximale() {
    return 1;
  }
  
  public double abscisseMinimale() {
    return 0;
  }
  
  /**
   * Fonction qui renverra true si la classe de l'objet possde un menu pour le modifier
   * Dans le cas o cette fonction renvoie true, la classe devra dfinir une fonction dialogueModification()
   * qui renverra une dialogue modifiant l'objet
   * @return
   */
  @Override
  /**
   *  Seuls les arcs dfinis par une valeur d'angle au centre sont modifiables par le menu de modification
   *  d'objet graphique
   */
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return (pointFinArc == null);
  	return !estElementFinal && (pointFinArc == null);
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new AngleArcDlg(frame, this, true);
  }
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil63");
  }  
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	if (pointFinArc == null)
  	{
  		return
  			(listeOb.contains(O) || O.estDefiniParObjetDans(listeOb))
  			&&
  			(listeOb.contains(A) || A.estDefiniParObjetDans(listeOb))
  			&&
  			(angleAuCentre.estDefiniParObjetDans(listeOb));
  	}
  	else
  	{
  		return
  			(listeOb.contains(O) || O.estDefiniParObjetDans(listeOb))
  			&&
  			(listeOb.contains(A) || A.estDefiniParObjetDans(listeOb))
  			&&
  			(listeOb.contains(pointFinArc) || pointFinArc.estDefiniParObjetDans(listeOb));
  		
  	}	
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    angleAuCentre = null;
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    A = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    pointFinArc = null;
    if (nVersion <= 1) {
      angleAuCentre = new CValeurAngle();
      angleAuCentre.chargeDepuisFlux(inps, list);
    } else {
      int ind3 = inps.readCInt();
      if (ind3 == -1) {
        angleAuCentre = new CValeurAngle();
        angleAuCentre.chargeDepuisFlux(inps, list);
      } else pointFinArc = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
    }
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }
  /* Chargement depuis un flux java */
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    angleAuCentre = null;
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    A = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    pointFinArc = null;
    int ind3 = inps.readInt();
    if (ind3 == -1) {
      angleAuCentre = new CValeurAngle();
      angleAuCentre.read(inps, list);
    } else pointFinArc = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(O);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(A);
    oups.writeInt(ind2);
    int ind3;
    if (pointFinArc == null) ind3 = -1;
    else ind3 = list.indexOf(pointFinArc);
    oups.writeInt(ind3);
    if (pointFinArc == null) angleAuCentre.write(oups, list);
  }  
  // Mthode dfinie pour la version 1.8
  public int natureArc() {
    return NatureArc.NatureArcPetitArc;
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (O == ancienPoint) O = nouveauPoint;
    if (A == ancienPoint) A = nouveauPoint;
    if (pointFinArc == ancienPoint) pointFinArc = nouveauPoint;
  }
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (A == po);
  }

  @Override
  public String chaineDesignation() {
    return getString("designationArc");
  }
  
}
