/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import types.DimensionFenetre;
import types.NatureCalcul;

import entreesSorties.MathGraphDataInputStream;

public class CCalcConst extends CCalcul {

  /** Creates new CCalcConst */
  public CCalcConst() {
  }
  /**
   * Constructeur pour cration d'objet
   * @param liste
   * @param impProtoInit
   * @param estElementFinalInit
   */
  public CCalcConst(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit);
  }
  
  // Constructeur pour clonage
  public CCalcConst(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomCalculInit, String chaineCalculInit, CCalculBase calculInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit, chaineCalculInit, calculInit);
    calculeValeurConstante();
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    int ind1 = listeSource.indexOf(impProto);
    return new CCalcConst(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, chaineCalcul, calculClone);
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NCalculReelConstant;
  }
  
  // Rajout pour la version 2.0
  public void calculeValeurConstante()
  {
    // Les paramtres passs n'ont aucune importance.
    // On calcule la valeur de la cosntante qu ne peut tre que pi dans cette version
    super.positionne(false, new DimensionFenetre(0,0));
  }
  // Positionne ne fait rien car la valeur constante est calcule au chargement de l'objet
  // ou lors du clonage
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = true;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return false; // Un calcul constant ne peut pas tre un objet final
	}
  
  @Override // Les calculs constants ne sont pas modifiables
  public boolean modifiableParMenu() {
  	return false;
  }  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps,list);
    calculeValeurConstante();
  }
  
  /* Lecture depuis dans un flux java */  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    calculeValeurConstante();
  }
}
