/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import mathgraph32.MtgFrame;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CCentreGravite extends CPointAncetre {
  CPointAncetre point1;
  CPointAncetre point2;
  CPointAncetre point3;
  
  /** Creates new CCentreGravite */
  public CCentreGravite() {
  }
  public CCentreGravite(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit,
    String nomInit, int indiceTailleNomInit, int motifInit , CPointAncetre point1Init, CPointAncetre point2Init,
    CPointAncetre point3Init, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit,
      dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    point1 = point1Init;
    point2 = point2Init;
    point3 = point3Init;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);
    int ind3 = listeSource.indexOf(point3);
    int ind4 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CCentreGravite(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x,
      dec_y, masque, nom, indiceTailleNom, motif, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)),
      marquePourTrace);
    return ptelb;
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo161");
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(point1);
    liste.add(point2);
    liste.add(point3);
  }   
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p)
    || point1.dependDe(p)
    || point2.dependDe(p)
    || point3.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p)
    || point1.dependDePourReclassement(p)
    || point2.dependDePourReclassement(p)
    || point3.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this)
    || point1.dependDePourBoucle(p)
    || point2.dependDePourBoucle(p)
    || point3.dependDePourBoucle(p));
  }
    
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = ((point1.existe) && (point2.existe) && (point3.existe));
    if (existe) {
      super.placeEn((point1.x+point2.x+point3.x)/3,(point1.y+point2.y+point3.y)/3);
      super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
    }
  }
  
  @Override
	public boolean confonduAvec(CElementBase p) {
    CCentreGravite pt;
    if (p.getClass() == this.getClass()) {
      pt = (CCentreGravite) p;
  		return (((point1==pt.point1) && (point2==pt.point2) && (point3==pt.point3))
  	      || 
  					((point1==pt.point1) && (point2==pt.point3) && (point3==pt.point2))
  				||
  					((point1==pt.point2) && (point2==pt.point3) && (point3==pt.point1))
  				||
  					((point1==pt.point2) && (point2==pt.point1) && (point3==pt.point3))
  				||
  					((point1==pt.point3) && (point2==pt.point2) && (point3==pt.point1))
  				||
  					((point1==pt.point3) && (point2==pt.point1) && (point3==pt.point2)));
    }
    else return false;
	}
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point1 == ancienPoint) point1 = nouveauPoint;
    if (point2 == ancienPoint) point2 = nouveauPoint;
    if (point3 == ancienPoint) point3 = nouveauPoint;
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(point1) || point1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point2) || point2.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point3) || point3.estDefiniParObjetDans(listeOb));
	}
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    int ind3 = inps.readCInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    point3 = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }

  @Override  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    point3 = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(point3);
    oups.writeInt(ind3);
  }    
}
