/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Java : Important : en C++ CDemiDroite hritait de CDroite mais appelait la mthode de srialization
// de CElement ligne car il est inutile de mmoriser l'origine et le vecteur
// c'est impossible en java. Une demi-droite ne pouvant pas tre nomme, inutile s'avoir
// un champ abscisseNom
// Mais cela pose un problme car dans la mthode de serialization de CDemiDroiteOA qui hrite de
// CdemiDroite, c'est la mthode de srialization de CDroite qui tait appele, ce qui est illogique.
// Dans chargeDepuisFlux de CDemiDroiteOA, il faut donc dpiler les donnes inutiles
// Mais pourtant uen demi-droite doit descendre de l'anctre des droites. J'ai rsolu le problme
// en rajoutant un type CDroiteAncetre qui soit l'anctre de toutes les droites
package objets;

import java.awt.*;
import java.io.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

// Classe pas abstract car est instancie dans CRepere
public class CDroite extends CDroiteAncetre {
  public static double abscisseNomParDefaut = 0.95;
  public double abscisseNom;
  
  /** Creates new CDroite */
  public CDroite() {
    super(); // Pour crer l'objet vecteur directeur
  }
  
  /**
   * Constructeur pour les droites utilises de faon interne
   * @param liste
   */
  public CDroite (CListeObjets liste) {
  	super();
  	listeProprietaire = liste;
  }

  // Constructeur pour clonage
  public CDroite(CListeObjets liste,CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit,
    int styleInit, double abscisseNomInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
      masqueInit, nomInit, indiceTailleNomInit, styleInit);
    // Vecteur a dj t cr par le constructeur de CDroiteAncetre
    // vecteur = new VecteurDirecteur();
    abscisseNom = abscisseNomInit;
  }
  /** Constructeur pour les descendants de Cdroite n'ayant pas besoin d'tre nomms */
  public CDroite(CListeObjets liste,CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    // Vecteur a dj t cr par le constructeur de CDroiteAncetre
    //vecteur = new VecteurDirecteur();
    abscisseNom = abscisseNomParDefaut;
  }
  
  /** Constructeur pour les descendants de Cdroite n'ayant pas besoin d'tre nomms */
  public CDroite(CListeObjets liste,CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, int indiceTailleNomInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    // Vecteur a dj t cr par le constructeur de CDroiteAncetre
    //vecteur = new VecteurDirecteur();
    abscisseNom = abscisseNomParDefaut;
    indiceTailleNom = indiceTailleNomInit;
  }
  
  
  // Pour satisfaire le compilateur, la classe n'tant pas abstract (voir plus haut)
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    return null;
  }
  
  @Override
  public String complementInfo() {
    return getString("chinfo40");
  }
  
  // Revu en java : inutile de redfinir dependDe et dependDePourBoucle

    
  public void positionneNom() {
    placeNom(xext1+abscisseNom*(xext2-xext1),yext1+abscisseNom*(yext2-yext1));
  }
  // Rajout pour la version 2.0
  // Renvoie la chaine que doit tre fugitivement affiche lorsque le pointeur souris
  // se rapproche d'un objet
  // Renvoie null s'il n'y a rien  afficher
  public String chaineDesignation() {
    return getString("designationDroite");
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    abscisseNom = inps.readCDouble();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
		// Version 4.7.7 : Si le nom est vide on n'enregistre pas abscisseNom
		if (listeProprietaire.numeroVersion < 11) abscisseNom = inps.readDouble();
		else {
			if (nom.isEmpty()) abscisseNom = abscisseNomParDefaut;
			else abscisseNom = inps.readDouble();
		}
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
		// Version 4.7.7 : Si le nom est vide on n'enregistre pas abscisseNom
    if (!nom.isEmpty()) oups.writeDouble(abscisseNom);
  }     
}