/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Contrairement  la version C++, il est inutile de dfinir getNature pour chaque objet
// car cela ne sert que pour les objets dsignables, les macros
// La nature NPoint qui est la nature du point construit sera la nature des points dfinie
// dans CPointAncetre. Il ne sera redfini que pour les points mobiles (pojnts libres et points lis).
// Contrairement au C++, je dfinis une nature AutreObjetGraphique rserv aux autres objets
// Modification pour la version 2.5 : la classe n'est plus abstract car il
// faut pouvoir l'instancier dans CListeObjets.get(int indice, Class cla)
package objets;

import java.awt.*;
import java.io.*;
import java.awt.Color;
import java.awt.geom.Rectangle2D;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CElementGraphique extends CElementBase {

	public Color couleur;

	public boolean nomMasque;
	public double x_nom, y_nom; // N'ont pas  tre srialiss sauf pour les
															// descendants de CAffichageLieAPoint
	public double dec_x, dec_y; // Dcalage utilisateur par rapport  (x_nom,
															// y_nom)
	public boolean masque; // sera true quand l'objet sera gomm
	protected String nom;
	public int indiceTailleNom;

	// Constructeur. Les autres ont t supprims dans la version java //
	protected CElementGraphique() {
	};

	protected CElementGraphique(CListeObjets liste, CImplementationProto impProtoInit,
			boolean estElementFinalInit, Color couleurInit, boolean nomMasqueInit, double dec_xInit,
			double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit) {
		super(liste, impProtoInit, estElementFinalInit);
		couleur = couleurInit;
		nomMasque = nomMasqueInit;
		dec_x = dec_xInit;
		dec_y = dec_yInit;
		masque = masqueInit;
		nom = new String(nomInit);
		indiceTailleNom = indiceTailleNomInit;
	}

	// Constructeur pour les lments n'ayant pas besoin d'tre nomms sauf
	// ventuellement
	// lors de la confection du script de la figure
	protected CElementGraphique(CListeObjets liste, CImplementationProto impProtoInit,
			boolean estElementFinalInit, Color couleurInit, double dec_xInit, double dec_yInit, boolean masqueInit) {
		super(liste, impProtoInit, estElementFinalInit);
		couleur = couleurInit;
		nomMasque = false;
		dec_x = dec_xInit;
		dec_y = dec_yInit;
		masque = masqueInit;
		nom = "";
		indiceTailleNom = 2;
	}

	/*
	 * Fonction rajoute pour la version 1.9.5 crant un objet de ce type dont la
	 * fonction sera uniquement de mmoriser la position d'un objet de mme type
	 */
	public CElementGraphique creeClone() {
		return (CElementGraphique) getClone(listeProprietaire, listeProprietaire);
	}

  @Override
	public void setClone(CElementBase ptel) {
		super.setClone(ptel);
		x_nom = ((CElementGraphique) ptel).x_nom;
		y_nom = ((CElementGraphique) ptel).y_nom;
		// Les deux lignes suivantes rajoute version 2.2.3 pour que quand on capture un affichage un ventuel lieu d'objets
		// de cet affichage soit aussi recalcul
		dec_x = ((CElementGraphique) ptel).dec_x;
		dec_y = ((CElementGraphique) ptel).dec_y;
		masque = ((CElementGraphique) ptel).masque;
	}

  /**
   * Fonction renvoyant la position relative de l'objet droite (ou un descendant) avec un autre objet
   * du type point, droite ou cercle.
   * L'entier retourn doit tre dfini par la classe PositionRelative
   * Redfini dans CDroiteAncetre et CCercle
   */
  public int positionRelative(CElementGraphique elg, Bool bperpendiculaires){
  	return PositionRelative.vide; // Pas utilis
  }
  
	/**
	 * Fonction rajoute pour la version 2.5.3 pour grer les lieux de lieux
	 * d'objets. Renvoie 1 par dfaut et ne sera redfinie que pour les lieux
	 * d'objets de faon que les figures ne soient pas trop lourdes
	 * 
	 * @return
	 */
	public int nombreObjetsPourLieuObjet() {
		return 1;
	}

	public void positionneNom() {
	}

	public void cache() {
		masque = true;
	}

	public void montre() {
		masque = false;
	}

	public final void montreNom() {
		nomMasque = false;
	}

	public final void masqueNom() {
		nomMasque = true;
	}

	public final void donneNom(String nomInit) {
		nom = nomInit;
	}

	public String rendNom() {
		return nom;
	}

	// Plus final pour version 4.5.2 car redfini dans CEditeurFormule
	// public final void placeNom(double xn, double yn) {
	public void placeNom(double xn, double yn) {
		x_nom = xn;
		y_nom = yn;
	}

	public final void decaleNom(double dec_xn, double dec_yn) {
		dec_x = dec_xn;
		dec_y = dec_yn;
	}

	/**
	 * Fonction affichant le nom d'un lment;
	 * @param pDC
	 */
	public void afficheNom(Graphics2D pDC) {
		char[] s = new char[1]; // Affichage caractre par caractre
		char carac;
		int xn, yn, yc;

		int len = nom.length();
		// Modification version 3.4.0 : On affiche les noms des objets masqus (outil rideau)
		// if ((len == 0) || nomMasque || masque) return;
		if ((len == 0) || nomMasque) return;
		Font fonteNom = FonteNom.getFonteNom(indiceTailleNom, FonteNom.Normal, listeProprietaire.coefMult);
		FontMetrics fm1 = pDC.getFontMetrics(fonteNom);
		int largeurCaractereNom;
		int hauteurCaractere = fm1.getMaxAscent();

		pDC.setColor(couleur);
		pDC.setFont(fonteNom);
		//  revoir pDC->SetBkMode(TRANSPARENT);
		//  revoir pDC->SetTextAlign(TA_LEFT+TA_TOP+TA_NOUPDATECP);
		xn = (int) (x_nom + dec_x);
		yn = (int) (y_nom + dec_y + hauteurCaractere);
		for (int i = 0; i < len; i++) {
			carac = nom.charAt(i);
			s[0] = carac;
			if (FonteNom.chiffre(carac))
				yc = yn + 4;
			else
				yc = yn;
			pDC.drawChars(s, 0, 1, xn, yc);
			// Si c'est un prime, le seul caractre qui peut
			// suivre est un chiffre et inutile de la dcaler autant
			// que les autres
			largeurCaractereNom = fm1.charWidth(carac) + 1;
			if (FonteNom.prime(carac))
				xn = xn + largeurCaractereNom / 2;
			else
				xn = xn + largeurCaractereNom;
		} // for
	}


	public void donneCouleur(Color coul) {
		couleur = coul;
	}

	public final Color rendCouleur() {
		return couleur;
	}

	/** Pour la version java, distancePoint ne sera dfinie que pour les points et
	 * ses
	 * descendants et pour les macros
	 * distancePointPourSurface sera dfini mais renverra -1
	 * */
	public double distancePoint(double xp, double yp, boolean masquage) {
		return -1;
	}

	/**
	 * La diffrence entre distancePoint et distancePointPourSurface est que
	 * descendants et pour les macros distancePointPourSurface sera dfini mais
	 * renverra -1
	 */
	public double distancePointPourSurface(double xp, double yp, boolean masquage) {
		return -1;
	}

	// Rajout pour la version 3.0 pour pouvoir renommer un objet.
	// L'ergonomi sera diffrente de la version C++
	// On pourra dplacer un nom sans avoir fait apparatre le dialogue
	// d'entre de nom
	public double distanceNom(Graphics2D dc, double xp, double yp, boolean masquage) {
		if (nom.length() == 0)
			return -1;
		Font fonte = FonteNom.getFonteNom(indiceTailleNom, FonteNom.Normal, -1);
		dc.setFont(fonte);
		FontMetrics fm = dc.getFontMetrics(fonte);
		Rectangle2D rect = fm.getStringBounds(nom, dc);
		double left = x_nom + dec_x;
		double top = y_nom + dec_y;
		double right = left + rect.getWidth();
		double bottom = top + rect.getHeight(); // Ecrture  partir de la baseline
		if (((xp > left) && (xp < right) && (yp < bottom) && (yp > top))) {
			return 0;
		} else
			return -1;
	}

	public double abscisseMaximale() {
		return 1;
	}

	public double abscisseMinimale() {
		return 0;
	}

	// Elle n'est plus abstract  partir de la version 1.9.5
	// pour viter de la redfinir dans les lments anctres des lments
	// pouvant crer une trace d'objets
	/**
	 * Fonction traant ou effaant l'objet suivant la valeur de effacement Si
	 * masquage est false les objets non masqus seront affichs d'une faon spciale
	 * Les deux derniers paramtres ont t rajouts pour la version 2.0
	 */
	public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
	}

	// Amlioration java par rapport c++ : si masque, pas d'affichage ni
	// d'effacement
	public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
		if (!(masquage && masque)) trace(pDC, couleurFond, transparence);
	}

	/**
	 * Fonction renvoyant true seulement si la droite est visible dans la fentre
	 * dimfen
	 * 
	 * @param dimfen
	 *          Renvoie true par dfaut
	 */
	public boolean dansFenetre(DimensionFenetre dimfen) {
		return true;
	}
	  
  @Override
  public void adaptePourHauteResolution(double coef) {
 		dec_x *= coef;
 		dec_y *= coef;
   }
	
  @Override
	public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.chargeDepuisFlux(inps, list);
		masque = inps.readBoolean();
		// A partir de la version 2.5 (n de version 4 et plus) on passe en couleurs
		// RVB
		if (listeProprietaire.numeroVersion <= 3) {
			int coul = inps.readByte();
			couleur = Couleur.TraduitCouleur((coul));
		} else {
			int r = inps.readUnsignedByte();
			int g = inps.readUnsignedByte();
			int b = inps.readUnsignedByte();
			@SuppressWarnings("unused")
			int a = inps.readUnsignedByte();
			couleur = new Color(r, g, b, 255);
		}

		nomMasque = inps.readBoolean();
		indiceTailleNom = inps.readByte();
		//  revoir
		nom = inps.readCStringAvecFiltrage();
		dec_x = inps.readCDouble()/10;
		dec_y = inps.readCDouble()/10;
	}

  @Override
	public void read(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.read(inps, list);
		masque = inps.readBoolean();
		int r = inps.readUnsignedByte();
		int g = inps.readUnsignedByte();
		int b = inps.readUnsignedByte();
		couleur = new Color(r, g, b, 255);
		nomMasque = inps.readBoolean();
		indiceTailleNom = inps.readByte();
		nom = inps.readUTF();
		// Version 4.7.7 : Comme la plupart du temps dec_x et dec_y sont nuls, on enregistre un byte 1 s'ils sont tous les deux nuls et sinon
		// un byte 0 suivi de l'enregistrement des deux double correspondants
		if (listeProprietaire.numeroVersion < 11) {
			dec_x = inps.readDouble();
			dec_y = inps.readDouble();
		}
		else {
			byte by = inps.readByte();
			if (by == 1) {
				dec_x = 0;
				dec_y = 0;
			}
			else {
				dec_x = inps.readDouble();
				dec_y = inps.readDouble();				
			}
		}
	}

  @Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list) throws IOException {
		super.write(oups, list);
		oups.writeBoolean(masque);
		oups.writeByte(couleur.getRed());
		oups.writeByte(couleur.getGreen());
		oups.writeByte(couleur.getBlue());
		oups.writeBoolean(nomMasque);
		oups.writeByte(indiceTailleNom);
		oups.writeUTF(nom);
		// Version 4.7.7 : Comme la plupart du temps dec_x et dec_y sont nuls, on enregistre un byte 1 s'ils sont tous les deux nuls et sinon
		// un byte 0 suivi de l'enregistrement des deux double correspondants
		if ((dec_x == 0) && (dec_y == 0)) oups.writeByte(1);
		else {
			oups.writeByte(0);
			oups.writeDouble(dec_x);
			oups.writeDouble(dec_y);
		}
	}

	// Rajout pour la version 2.0
	// Renvoie la chaine que doit tre fugitivement affiche lorsque le pointeur
	// souris
	// se rapproche d'un objet
	// Renvoie null s'il n'y a rien  afficher
	public String chaineDesignation() {
		return null;
	}
}