/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package objets;

import java.io.IOException;

import mathgraph32.MtgFrame;
import dialogs.FonctionReelle2VarDlg;
import dialogs.FonctionReelle3VarDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;
import types.ErreurCalculException;
import types.NatureCalcul;

/**
 * Fonction relle de  plusieurs variables relles
 * @author Yves
 *
 */
public class CFoncNVar extends CCalcul {
	public int nbVar; // Le nombre de variables
	public String[] chaineVariable; // Reprsente les variables formelles

	/** Constructeur */
	public CFoncNVar() {
		super();
	}
	
	/** Constructeur pour outil de cration */
	public CFoncNVar(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
			int nbVarInit) {
	  super(liste, impProtoInit, estElementFinalInit);
	  nbVar = nbVarInit;
    chaineVariable = new String[nbVar];
	}
	
	// Constructeur pour clonage
	public CFoncNVar(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
	    String nomInit,String chaineCalculInit, CCalculBase calculInit, String[] chaineVariableInit) {
	  super(liste, impProtoInit, estElementFinalInit, nomInit, chaineCalculInit, calculInit);
	  nbVar = chaineVariableInit.length;
	  chaineVariable = new String[nbVar];
	  for (int i = 0; i < nbVar; i++) chaineVariable[i] = new String(chaineVariableInit[i]);
	}
	
	/**
	 * Version 4.6 : Constructeur spcial pour appel par le constructeur de CDeriveePartielle
	 * @param liste
	 * @param impProtoInit
	 * @param estElementFinalInit
	 * @param nomInit
	 * @param chaineCalculInit
	 * @param calculInit
	 * @param nbVarInit
	 */
	public CFoncNVar(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
	    String nomInit,String chaineCalculInit, CCalculBase calculInit, int nbVarInit) {
	  super(liste, impProtoInit, estElementFinalInit, nomInit, chaineCalculInit, calculInit);
	  nbVar = nbVarInit;
	  chaineVariable = null;
	}
	
	/**
	 * Fonction renvoyant un pointeur sur le tableau de chanes reprsentant les variables de la fonction
	 * Devra tre redfini pour CDeriveePartielle
	 * @return
	 */
	public String[] chaineVariable() {
		return chaineVariable;
	}
		
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    CElementBase ptelb = new CFoncNVar(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, chaineCalcul, calculClone, chaineVariable);
    return ptelb;    
  }
	
  @Override
  public long getNatureCalcul() {
  	switch (nbVar) {
		case 2 :
			return NatureCalcul.NFonction2Var;
		case 3 :
			return NatureCalcul.NFonction3Var;
		default:
			return NatureCalcul.NFonction2Var;
		}
  }
  
  @Override
  public int nombreVariables() {
  	return nbVar;
  }
  
  public double rendValeurFonction(boolean infoRandom, double[] parametre)
  throws ErreurCalculException {
    return calcul.resultatFonction(infoRandom, parametre);
  }

	@Override
	public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
	  existe = calcul.existe(infoRandom);
	}
  
  // A revoir version 3.0
  /***
   * Fonction reconstruisant la chaine reprsentant le calcul (par exemple lorsqu'un autre calcul a t renomm
   * A redfinir pour les fonctions
   */
 @Override
  public void reconstruitChaineCalcul() {
  	chaineCalcul = calcul.chaineCalculSansPar(chaineVariable);	
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("ch86") + " " + nom() + " " + getString("ch87") + "\n" + nom() 
    	+ "(";
    for (int i = 0; i < nbVar; i++) {
    	ch = ch + chaineVariable[i];
    	if (i != nbVar - 1) ch = ch + ',';
    }
    ch = ch + ")" + " = " + chaineCalcul;
    if (!existe) ch = ch + "\n" + getString("ch18");
    return ch;
  }
  
  // Vrifier l'utilit pour version 3.0
  /**
   * Fonction renvoyant une chaine du type f(t) = .... pour certain champs de botes de dialogues
   * @return
   */
  public String infoFormule() {
  	String ch = nom() + "(";
    // for (int i = 1; i < nbVar; i++) {
    for (int i = 0; i < nbVar; i++) {
    	ch = ch + chaineVariable[i];
    	if (i != nbVar - 1) ch = ch + ',';
    }
  	return ch + ")" + " = " + chaineCalcul;
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
  // A revoir version 3.0
  @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	switch (nbVar) {
	  	case 2 : return new FonctionReelle2VarDlg(frame, this, true);
	  	case 3 : return new FonctionReelle3VarDlg(frame, this, true);
	  	default : return null;
  	}
  }
  
  @Override
  public boolean aPourParametre(String ch) {
  	for (int i = 1; i < nbVar; i++) {
  		if (chaineVariable[i].equals(ch)) return true;
  	}
  	return false;
  }  
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    nbVar = inps.readInt();
    chaineVariable = new String[nbVar];
    for (int i = 0; i < nbVar; i++) chaineVariable[i] = inps.readUTF();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(nbVar);
    for (int i = 0; i < nbVar; i++) oups.writeUTF(chaineVariable[i]);
  }
  
  @Override
  public boolean estFonctionOuSuite() {
    return true;
  }
    
}



