/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.util.*;

import java.awt.Color;

import types.*;

public class CLigneBrisee extends CLignePolygonaleAncetre {

  /** Creates new CLigneBrisee */
  public CLigneBrisee() {
  	super();
  }
  
  /**
   * Constructeur pour construction d'objet graphique
   * @param couleurInit
   * @param styleInit
   */
  public CLigneBrisee(CListeObjets liste, Color couleurInit, int styleInit) {
  	super(liste, couleurInit, styleInit);
  }
  
  /** Construction  partir d'un vecteur de CNoeudPointeurSurPoint */
  public CLigneBrisee(CListeObjets liste, Vector<CNoeudPointeurSurPoint> vecteur, Color couleurInit, int styleInit) {
  	super(liste, vecteur, couleurInit, styleInit);
  }
  
  
  // Constructeur pour clonage
  public CLigneBrisee(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, Vector<CNoeudPointeurSurPoint> collectionPointsInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, collectionPointsInit);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    Vector<CNoeudPointeurSurPoint> colClone = new Vector<CNoeudPointeurSurPoint>();
    ListIterator<CNoeudPointeurSurPoint> li = collectionPoints.listIterator();
    while(li.hasNext()) {
      CNoeudPointeurSurPoint noeud = (CNoeudPointeurSurPoint) (li.next());
      CNoeudPointeurSurPoint noeudClone = noeud.getClone(listeSource, listeCible);
      colClone.add(noeudClone);
    }
    int ind1 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CLigneBrisee(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, colClone);
    return ptelb;    
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo52");
  }
  

  public double abscisseMaximale() {
    return 1;
  }

  public double abscisseMinimale() {
    return 0;
  }
 
  @Override
  public long getNature() {
    return NatureObjet.NLigneBrisee;
  }

  @Override
  public String complementInfo() {
    return getString("chinfo46");
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationLigneBrisee");
  }
  
  public boolean confonduAvec(CElementBase p) {
    CLigneBrisee pt;
  	CPointAncetre ptp1;
  	CPointAncetre ptp2;

    if (p.getClass() != CLigneBrisee.class) return false;
  	
  	pt= (CLigneBrisee) p;

  	int nombrePoints = collectionPoints.size();
  	if (nombrePoints != pt.collectionPoints.size()) return false;
  	// On regarde si une ligne brise forme des mmes points
  	// a dj t cre dans le mme sens
  	boolean confondu = true;
  	for (int i = 0; i < nombrePoints; i++) {
  		ptp1 = collectionPoints.get(i).pointeurSurPoint;
  		ptp2 = pt.collectionPoints.get(i).pointeurSurPoint;
  		confondu = confondu && (ptp1 == ptp2);
  	}
  	// ou dans le sens inverse
  	if (! confondu)
  	{
  		confondu = true;
    	for (int i = 0; i < nombrePoints; i++) {
    		ptp1 = collectionPoints.get(i).pointeurSurPoint;
    		ptp2 = pt.collectionPoints.get(nombrePoints-i-1).pointeurSurPoint;
    		confondu = confondu && (ptp1 == ptp2);
    	}
  	}
  	return confondu;  	
  }
  
  
}
