/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Java :  noter que le fonctionnement de cette macro est amlior par rapport au C++ avec
// l'utilisation d'une liste d'objets sur les lements dpendant de la variable modifie
import java.io.*;
import java.awt.Color;

import dialogs.MacroAffectationValeurVariableDlg;
import dialogs.MtgDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroAffectationValeurVariable extends CMacro {
	public CValeurAssocieeAVariable valeurAssociee;
  public CVariableBornee variableAssociee;
  private CListeObjets listeDep;
  /** Creates new CMacroAffectationValeurVariable */
  public CMacroAffectationValeurVariable() {
  }
  
  // Constructeur pour clonage
  public CMacroAffectationValeurVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      CValeurAssocieeAVariable valeurAssocieeInit, CVariableBornee variableAssocieeInit,
    	int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit,
					alignementHorizontalInit, alignementVerticalInit);
        valeurAssociee = valeurAssocieeInit;
        variableAssociee = variableAssocieeInit;
        // Il faut crer la liste des lments dpendant de la variable
        listeDep = new CListeObjets();    
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CValeurAssocieeAVariable valeurClone = (CValeurAssocieeAVariable) valeurAssociee.getClone(listeSource, listeCible);
    int ind1 = listeSource.indexOf(variableAssociee);
    int ind2 = listeSource.indexOf(pointLie);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMacroAffectationValeurVariable(listeCible,
    	(CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind2, CPointAncetre.class), indiceTaillePolice,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, valeurClone,
      (CVariableBornee)(listeCible.get(ind1, CVariableBornee.class)), alignementHorizontal, alignementVertical);
    listeDep.initialise(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
    // On ne peut pas  ce niveau faire une image de la liste ListeDep dans la liste clone car tous
    // les lments n'ont pas encor t chargs.
    //((CMacroAffectationValeurVariable)ptelb).listeDep.setImage(listeDep, listeProprietaire,
    //  ptelb.listeProprietaire);
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || variableAssociee.dependDe(p) || valeurAssociee.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || variableAssociee.dependDePourReclassement(p) || valeurAssociee.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || variableAssociee.dependDePourBoucle(p)
      || valeurAssociee.dependDePourBoucle(p);
  }

  // Ajout pour version 3.0
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  	valeurAssociee.positionne(infoRandom, dimfen);
  	existe = existe && valeurAssociee.existe();
  }
  
  @Override
  public boolean agitSurVariable(CVariableBornee var) {
  	return (var == variableAssociee);
  }
  
  @Override
  public boolean affecteValeurAVariable(CValeurDynamique val) {
  	return valeurAssociee.calcul.dependDe(val);
  }
  
  @Override
  public boolean peutEtreMacroFinBoucle() {
  	return true;
  }
  

  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
  	// Important : ligne suivante ncessaire pour la version 3.0 (changement de CValeurDynamique)
  	valeurAssociee.positionne(cadre.infRand, cadre.dimf);
    boolean infor = cadre.infRand;
    DimensionFenetre dimfen = cadre.dimf;
    double resultatValeur = valeurAssociee.rendValeur(infor);
    variableAssociee.donneValeur(resultatValeur);
    listeDep.positionne(infor, dimfen);
    cadre.effacegbuffer();
    cadre.afficheToutDansgbuffer();
  	// cadre.metAJourDialogueAffichageVariables(variableAssociee);
    cadre.metAJourPaneAffichageVariables(variableAssociee);
  	// frame.cadreActif().redessine();
    // Modifi version 1.8
    executionEnCours = false;
  }

  @Override
  public void executeSimple(MtgInternalFrame cadre) {
  	// A revoir pour l'applet
  	// Important : ligne suivante ncessaire pour la version 3.0 (changement de CValeurDynamique)
  	valeurAssociee.positionne(cadre.infRand, cadre.dimf);
    executionEnCours = false;
    double resultatValeur = valeurAssociee.rendValeur(cadre.infRand);
    variableAssociee.donneValeur(resultatValeur);
    // Ligne suivante supprime version 2.4.0
    // listeDep.positionne(cadre.infRand, cadre.dimf);
  }
  
  /** Mthode charge de remettre la figure en tat initial
   * et n'existant pas dans l'applet. Charge de mettre  jour
   * le dialogue ventuel d'affichage des variables en bas  droite
   */
  public void termineAction(MtgInternalFrame cadre) {
    super.termineAction(cadre);
    // cadre.metAJourDialogueAffichageVariables(variableAssociee);
    cadre.metAJourPaneAffichageVariables(variableAssociee);
  }
  
  @Override
  public boolean executionPossible() {
    boolean resultat = valeurAssociee.existe() && variableAssociee.existe;
    if (resultat)
    {
      boolean infr = false; // Pas de recalcul alatoire
      double nouvelleValeur = valeurAssociee.rendValeur(infr);
      resultat = (nouvelleValeur >= variableAssociee.valeurMini)
        && (nouvelleValeur <= variableAssociee.valeurMaxi);
    }
    return resultat;
  }
    
  @Override
  public void etablitListesInternes() {
    listeDep = new CListeObjets(listeProprietaire.uniteAngle, listeProprietaire.pointeurLongueurUnite);        
    listeDep.ajouteObjetsDependantsSauf(variableAssociee, listeProprietaire, this);
  }
  
  // Ajout version 4.7.4
  @Override
  public void metAJour() {
  	etablitListesInternes();
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return super.estDefiniParObjetDans(listeOb)
			&& (listeOb.contains(variableAssociee) || variableAssociee.estDefiniParObjetDans(listeOb))
			&& valeurAssociee.estDefiniParObjetDans(listeOb);
	}  
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 4.5.1
  	// return !estElementFinal;
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroAffectationValeurVariableDlg(frame, this, true);
  }
  
   @Override
   public String infoPourHistorique() {
     return getString("chinfo127") + " " + valeurAssociee.chaineInfo() +
     " " + getString("chinfo128")  + " " + variableAssociee.nom();
   }
   
   // Suprim version 2.0 car provoques des stackoverflow
   /*
   @Override
   public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
   	listeDep.remplacePoint(ancienPoint, nouveauPoint);
   }
   */
   
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    valeurAssociee = new CValeurAssocieeAVariable();
    valeurAssociee.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut crer la liste des lments dpendant de la variable modifie
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    listeDep = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    valeurAssociee = new CValeurAssocieeAVariable();
    valeurAssociee.read(inps, list);
    int ind1 = inps.readInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut crer la liste des lments dpendant de la variable modifie
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    listeDep = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    valeurAssociee.write(oups, list);
    int ind1 = list.indexOf(variableAssociee);
    oups.writeInt(ind1);
  }
}
