/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import dialogs.MacroPauseDlg;
import dialogs.MtgDlg;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroPause extends CMacro {
	public int dureePause; // Dure de la pause en secondes

  /** Creates new CMacroPause */
  public CMacroPause() {
  }
  
  // Constructeur pour clonage
  public CMacroPause(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit, int dureePauseInit,
    	int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit,
					alignementHorizontalInit, alignementVerticalInit);
        dureePause = dureePauseInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointLie);  
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMacroPause(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, dureePause,
      alignementHorizontal, alignementVertical);
    return ptelb;
  }
    
  @Override
  public long dureePourAnimation() {
    return dureePause;
  }

  @Override
  public void actionAvantExecution(MtgInternalFrame cadre) {
  	super.actionAvantExecution(cadre);
    timeDebut = System.currentTimeMillis();
    timeFin = timeDebut + dureePause*1000; // La dure est en diximes de secondes
  }

  @Override
  public int typeAnimation() {
    return CMacro.animationParTimer;
  }

  @Override
  public int frequenceAnimation() {
    return 10; // On utilise un timer au centime de seconde
  }

  /*
  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
    // ne fait rien
  }
  */
  
  //  Rajout version 3.0 pour corriger un bug (ne marchait plus)
  public void executePourTimer(MtgInternalFrame cadre, boolean repaint) {
		if (System.currentTimeMillis() >= timeFin) termineAction(cadre);
  }
  
  
  public void termineAction(MtgInternalFrame cadre) {
    super.termineAction(cadre);
    cadre.paneFigure.repaint();
  }
    
  @Override
  public String infoPourHistorique() {
    return getString("chinfo134") + " " + Integer.toString(dureePause);
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 4.5.1
  	// return !estElementFinal;
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroPauseDlg(frame, this, true);
  }  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    dureePause = inps.readCInt();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    dureePause = inps.readInt();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(dureePause);
  }
}
