/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import entreesSorties.MathGraphDataInputStream;
import glob.*;

import java.awt.*;
import java.io.IOException;

import mathgraph32.MtgFrame;


import types.*;

/** Classe reprsentant une surface dlimite par un arc de cercle
 * Modifie version 3.0 pour pouvoir tre un objet gnrant un lieu d'objets
 * @author Yves
 *
 */

public class CSurfaceSecteurCirculaire extends CSurface {
	private CArcDeCercleAncetre arc;
	private int natureArcBord; // Rajout version 3.0 pour que trace ne fasse pas rfrence au pointeur bord
  /** Creates new CSurfaceSecteurCirculaire */
  public CSurfaceSecteurCirculaire() {
  }
  
  // Constructeur pour clonage
   public CSurfaceSecteurCirculaire(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
        Color couleurInit, boolean masqueInit, int styleRemplissageInit, CArcDeCercleAncetre bordInit) {
     super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleRemplissageInit, bordInit);
     arc = new CArcDeCercleAncetre(liste, null, false, Color.black, false, StyleTrait.traitFin);
     }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(bord);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSurfaceSecteurCirculaire(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, masque, styleRemplissage, (CArcDeCercleAncetre)(listeCible.get(ind1, CArcDeCercleAncetre.class)));
    return ptelb;
  }
  
  // Rajout version 3.0 (oubli versions prcdentes pour historique)
  @Override
  public String infoPourHistorique() {
    return getString("chinfo168");
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(bord);
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	if (!existe || (masquage && masque)) return -1;
  	return arc.distancePointPourSurface(xp, yp, masquage);
  }
  
  // Rajout version 3.0 pour pouvoir tre un objet gnrant un lieu d'objets
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  	arc.setClone((CArcDeCercleAncetre) bord);
  	natureArcBord = ((CArcDeCercleAncetre) bord).natureArc();
  }  
    
  // Ajout version 3.0
  @Override
	public void setClone(CElementBase ptel) {
		super.setClone(ptel);
		arc.setClone(((CSurfaceSecteurCirculaire) ptel).arc);
		natureArcBord = ((CSurfaceSecteurCirculaire) ptel).natureArcBord;
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleRemplissageEffectif;
    if (masque) styleRemplissageEffectif = StyleRemplissage.QuadrillageCroise;
    else styleRemplissageEffectif = styleRemplissage;
    if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePleinTransp) {
    	if (transparence != 1) {
    		pDC.setColor(couleur);
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
  	    dessine(pDC);
  	    pDC.setComposite(originalComposite);    		
    	}
    	else {
    		pDC.setColor(couleur);
  	    dessine(pDC);
    	}
    }
    else {
      if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePlein) {
    		pDC.setColor(couleur);
  	    dessine(pDC);
      }
      else {
	    	TexturePaint tp = StyleRemplissage.getTexturePaint(couleur, styleRemplissageEffectif, listeProprietaire, couleurFond);
	    	pDC.setPaint(tp);
		    dessine(pDC);
      }
    }
  }
  
  private void dessine(Graphics2D pDC) {
    switch (natureArcBord) {   
      case NatureArc.NatureArcPetitArc :
        Kernel.secteurSaillantPlein(pDC, arc.origine_x, arc.origine_y, arc.centre_x, arc.centre_y,
          arc.rayon, arc.valeurAngleAuCentre);
        break;
      case NatureArc.NatureArcGrandArc :
        Kernel.secteurRentrantPlein(pDC, arc.origine_x, arc.origine_y, arc.centre_x, arc.centre_y,
        arc.rayon, arc.valeurAngleAuCentre);
        break;
      case NatureArc.NatureArcArcDirect :
        Kernel.secteurDirectPlein(pDC, arc.origine_x, arc.origine_y, arc.centre_x, arc.centre_y,
        arc.rayon, arc.valeurAngleAuCentre);
        break;
      case NatureArc.NatureArcArcIndirect :
        VecteurDirecteur u = new VecteurDirecteur(arc.centre_x, arc.centre_y, arc.origine_x,
          arc.origine_y);
        VecteurDirecteur v = new VecteurDirecteur();
        u.tourne(arc.valeurAngleAuCentre, v);
        double x1 = arc.centre_x + v.x;
        double y1 = arc.centre_y + v.y;
        Kernel.secteurDirectPlein(pDC, x1, y1, arc.centre_x, arc.centre_y, arc.rayon,
          -arc.valeurAngleAuCentre);
    }  	
  }
  
  @Override
  public boolean peutGenererLieuObjet() {
  	return true;
  }
  
  // Ajout version 3.0 pour initialiser arc
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
  	super.chargeDepuisFlux(inps, list);
    arc = new CArcDeCercleAncetre(list, null, false, Color.black, false, StyleTrait.traitFin);   	
  }
  
  // Ajout version 3.0 pour initialiser arc
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    arc = new CArcDeCercleAncetre(list, null, false, Color.black, false, StyleTrait.traitFin);  
  }
}
