/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;
import mathgraph32.MtgFrame;
import dialogs.MtgDlg;
import dialogs.TestDependanceVariableDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;
import types.NatureCalcul;

public class CTestDependanceVariable extends CCalculAncetre {
	public CCalcul fonctionAssociee; // Pointera soit sur un CFonc soit sur un CFoncNVar
	public int indiceVariable;
	private double resultat;
	
  public CTestDependanceVariable() {
  	existe = true;
  }

  // Constructeur pour clonage
  public CTestDependanceVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit, CCalcul fonctionAssocieeInit, int indiceVariableInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit);
    fonctionAssociee = fonctionAssocieeInit;
    indiceVariable = indiceVariableInit;
    existe = true;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CTestDependanceVariable(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CCalcul)(listeCible.get(ind1, CCalcul.class)), indiceVariable);
    return ptelb;
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NTestDependanceVariable;
  }
  
  @Override
  public String infoPourHistorique() {
  	long natcalc = fonctionAssociee.getNatureCalcul();
  	String chvar = (natcalc == NatureCalcul.NFonction) ? ((CFonc) fonctionAssociee).chaineVariable
  			: ((CFoncNVar)fonctionAssociee).chaineVariable()[indiceVariable];
    String ch =  getString("ch86") + " " + nom() + "\n" + getString("ch192") + " " + fonctionAssociee.nom()
    		+ "\n" + getString("ch191") + " " + chvar;
    ch = ch + "\n" + getString("chinfo97");
    if (resultat == 1) ch = ch + "1"; else ch = ch + "0";
    return ch;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || fonctionAssociee.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || fonctionAssociee.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || fonctionAssociee.dependDePourBoucle(p);
  }

  public double rendValeur(boolean infoRandom) {
    return resultat;
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    // Existe toujours gal  true pour cet objet
  	resultat = fonctionAssociee.calcul.dependDeVariable(indiceVariable) ? 1 : 0;
  }

  @Override
  public boolean confonduAvec(CElementBase p) {
    CTestDependanceVariable pt;
    if (p.getClass() == this.getClass()) {
      pt = (CTestDependanceVariable) p;
      return (fonctionAssociee == pt.fonctionAssociee)  && (indiceVariable == pt.indiceVariable);
    }
    else return false;
  }    

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(fonctionAssociee) || fonctionAssociee.estDefiniParObjetDans(listeOb);
	}    
    

  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new TestDependanceVariableDlg(frame, this, true);
  }

   @Override
   public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
     super.read(inps, list);
     int ind1 = inps.readInt();
     fonctionAssociee = (CCalcul) (list.get(ind1,CCalcul.class));
     indiceVariable = inps.readInt();
   }

   @Override
   public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
     super.write(oups, list);
     int ind1 = list.indexOf(fonctionAssociee);
     oups.writeInt(ind1);
     oups.writeInt(indiceVariable);
   }    

}
