/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;

import mathgraph32.MtgFrame;
import dialogs.MtgDlg;
import dialogs.TestEqNatOpDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;
import types.NatureCalcul;
import types.Operateur;

public class CTestEqNatOp extends CCalculAncetre {
	public CCalcul calcul1;
	public CCalcul calcul2;
	private double resultat;
	private boolean dejaPositionne;

	/**
	 * Constructeur
	 */
	public CTestEqNatOp() {
		existe = true;
		dejaPositionne = false;
	}
	
  public CTestEqNatOp(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomCalculInit, CCalcul calcul1Init, CCalcul calcul2Init) {
      super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
      calcul1 = calcul1Init;
      calcul2 = calcul2Init;
      existe = true;
      dejaPositionne = false;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    int ind2 = listeSource.indexOf(calcul1);
    int ind3 = listeSource.indexOf(calcul2);   
    return new CTestEqNatOp(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CCalcul)(listeCible.get(ind2, CCalcul.class)), (CCalcul)(listeCible.get(ind3, CCalcul.class)));
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NTestNatureOperateur;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("ch196") + " " + calcul1.nom() + " " + getString("chinfo101") + " " + calcul2.nom() + "\n";
    ch = ch + getString("chinfo97") + " " + glob.Kernel.chaineNombre(resultat, 16);
    return ch;
  }
  
  public double rendValeur(boolean infoRandom) {
    return resultat;
  }
    
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	if (!dejaPositionne) {
  		metAJour();
  		dejaPositionne = true;
  	}
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || calcul1.dependDe(p) || calcul2.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || calcul1.dependDePourReclassement(p) || calcul2.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || calcul1.dependDePourBoucle(p) || calcul2.dependDePourBoucle(p);
  }

  @Override
  public boolean modifiableParMenu() {
  	return !estElementFinal;
  }
  
  @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	 return new TestEqNatOpDlg(frame,this, true);
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
			(listeOb.contains(calcul1) || calcul1.estDefiniParObjetDans(listeOb))
			&&
			(listeOb.contains(calcul2) || calcul2.estDefiniParObjetDans(listeOb));
	}
  
  @Override
	public void metAJour() {
  	if (verifieEquivalenceNature()) {
  		dejaPositionne = true;
  		resultat = 1;
  	}
  	else resultat = 0;
	}
  
  private boolean verifieEquivalenceNature() {
  	if (calcul1.calcul.getClass() != calcul2.calcul.getClass()) return false;
  	// Les deux calculs sont de mme classe
  	if (calcul1.calcul.getClass() == COperation.class) {
  		COperation op1 = (COperation) calcul1.calcul;
  		COperation op2 = (COperation) calcul2.calcul;
  		if (op1.operateur == Operateur.Egalite) return (op2.operateur == Operateur.Egalite);
  		if ((op1.operateur == Operateur.Sup) || (op1.operateur == Operateur.Inf)) return ((op2.operateur == Operateur.Sup) || (op2.operateur == Operateur.Inf));
  		if ((op1.operateur == Operateur.SupOuEgal) || (op1.operateur == Operateur.InfOuEgal))
  			return ((op2.operateur == Operateur.SupOuEgal) || (op2.operateur == Operateur.InfOuEgal));
  		// Sinon l'oprateur doit tre le mme
  		return (op1.operateur == op2.operateur);
  	}
  	else return true;
  }

	 @Override
	 public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
	   super.read(inps, list);
	   int ind1 = inps.readInt();
	   calcul1 = (CCalcul) (list.get(ind1,CCalcul.class));
	   int ind2 = inps.readInt();
	   calcul2 = (CCalcul) (list.get(ind2,CCalcul.class));
	 }

	@Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
	  super.write(oups, list);
	  int ind1 = list.indexOf(calcul1);
	  oups.writeInt(ind1);
	  int ind2 = list.indexOf(calcul2);
	  oups.writeInt(ind2);
	}          
  
}
