/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;


import glob.*;

import java.awt.*;


import types.*;


// Modification pour la version 2.5 : la classe n'est plus abstract car il
// faut pouvoir l'instancier dans CListeObjets.get(int indice, Class cla)
public class CValeurDynamique extends CElementBase {
  // Attribut rajout pour optimisation lors de la version 2.0
  Complexe z1 = new Complexe();
  
  /** Creates new CValeurDynamique */
  public CValeurDynamique() {
  }

  public CValeurDynamique(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit);
  }
  
  public void donneNom(String  NouveauNom)
  {
  }
  
  // Mthode  redfinir pour les descendants renvoyant la valeur. Ne sert pas ici
  public double rendValeur(boolean infoRandom) {
    return 0;
  }
  public void rendValeurComplexe(boolean infoRandom, Complexe zRes) {
    zRes.x = rendValeur(infoRandom);
    zRes.y = 0;
  }
  String rendChaineValeur(int nbDecimales, boolean infoRandom)
  {
    return Kernel.chaineNombre(rendValeur(infoRandom), nbDecimales);
  }
  // Rajout pour la version 2.0
  String rendChaineValeurComplexe(int nbDecimales, boolean infoRandom) {
    rendValeurComplexe(infoRandom, z1);
    String ch1 = Kernel.chaineNombre(z1.x, nbDecimales);
    String ch2 = Kernel.chaineNombre(z1.y, nbDecimales);
    if (ch2.equals("0")) return ch1;
    else
    {
      if (ch1.equals("0"))
      {
        if (ch2.equals("-1")) ch2 = "-";
        else if (ch2.equals("1")) ch2 = "";
        return ch2 + " i";
      }
      else
      {
        if (ch2.equals("1")) return ch1 + " + i";
        else
        {
          if (ch2.equals("-1")) ch2 = "-";
          if (ch2.charAt(0) == '-') return ch1 + " - " + ch2.substring(1) + " i";
          else return ch1 + " + " + ch2 + " i";
        }
      }
    }
  }
  // La fonction suivante ne sera redfinie que pour CCalculComplexe
  // Sert dans les commentaires dynamiques
  public String rendChaineValeurPourCommentaire(int nbDecimales, boolean infoRandom)
  {
    return rendChaineValeur(nbDecimales, infoRandom);
  }
  
  // Rajout pour la version 1.8 : nom() qui rend le nom du calcul. Appel rendNom en C++
  // Ne sera redfini que pour les calculs et les variables
  @Override
  public String nom() {
    return "";
  }
  
  // Ajout pour la version 2 du CNED
  // Fonction renvoyant true si la chaine pointe par PChaine
  // commence par l'intitul de la valeur dynamique. Si oui, elle
  // renvoie dans LongueurNom.x la longueur de cet intitul
  // Sera redfini pour certains descendants
  public boolean chaineCommenceParNom(String pChaine,Point longueurNom){
    String ChaineNom = nom();
    longueurNom.x = ChaineNom.length();
    int longueurChaine2 = pChaine.length();
    if (longueurChaine2 < longueurNom.x) return false;
    else
    {
      return ChaineNom.equals(pChaine.substring(0, longueurNom.x));
    }
  }
  
  // Ajout spcifique version avec construction permettant de crer des calculs
  /**
   * Renverra true si la chaine ch est gale (sans tenir compte de la casse)
   * soit au nom de la valeur dans le cas d'un calcul ou une fonction soit 
   * la chane reprsentant la mesure dans le cas d'une mesure
   * Modifi version 3.5.0 : on diffrencie majuscules et minuscules
   * @param ch
   * @return
   */
  public boolean chaineEgaleANom(String ch) {
    return (nom().equals(ch));
  }

  /**
   * Pour savoir si le calcul est une fonction relle, une suite relle ou non
   * Redfini seulement pour CFonc, CFoncNVar, CFoncComplexe, CFoncComplexeNVar, CSuiteRec et CSuiteRecComplexe;
   */ 
  public boolean estFonctionOuSuite() {
    return false;
  }
  
  /**
   * Ajout pour la version 3.0 : renvoie true seulement si la valeur dynamique est constant
   * @return
   */
	public boolean estConstant() {
		return false;
	}
	
	// Ajout version 4.6
	/**
	 * Si l'objet associ possde une formule qui est un test renvoie le membre de gauche
	 * sinon renvoie null;
	 * @return
	 */
	public CCalculBase membreGauche() {
		return null;
	}
	
	// Ajout version 4.6
	/**
	 * Si l'objet associ possde une formule qui est un test renvoie le membre de gauche
	 * sinon renvoie null;
	 * @return
	 */
	public CCalculBase membreDroit() {
		return null;
	}
	
}
