/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import objets.CListeObjets;
import objets.CLongueur;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilCreerLongueurUnite extends OutilPar2pts {
  /** Constructor */
  public OutilCreerLongueurUnite(MtgFrame frameInit) {
    super(frameInit);
  }

  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soeint nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  @Override
  boolean nomsPointsIndisp() {
    return true;
  }  

  @Override
  public void creeObjet() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	// On regarde si une longueur unit existe dj dans la figure
  	CListeObjets liste = frame.cadreActif().listePr;
    CLongueur lon = new CLongueur(frame.cadreActif().listePr, null, false , point1, point2);
    frame.ajouteElement(lon);
    liste.pointeurLongueurUnite = lon;
    objetCree = true; // Necessaire  cause du fait que point1 et point2 ont t ventuellement
    // nomms et que si on ne cre pas l'objet on remet alors leur nom  une chaine vide
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil15"));
    annuleClignotement();
    cadre.videDesignation();
    frame.dialogueAvertissement(getString("ch148") + " " + lon.nom() + " " + getString("ch149"));
    cadre.redessineAvecCalcul();
    frame.activeOutilCapture();
  }
  
  @Override
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("creationLongueurUnite1");
      case 2 : return getString("creationLongueurUnite2");
      default: return "";
    }
  } 
  
  @Override
  public boolean activationValide() {
    return (frame.cadreActif().listePr.pointeurLongueurUnite == null);
  }

	@Override
	public String messageActivationNonValide() {
		return getString("messageLongueurUniteDejaDefinie");
	}	
  
}
