/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Color;
import java.awt.Point;

import objets.CAffichageLieAPoint;
import objets.CCalcul;
import objets.CCommentaire;
import objets.CElementGenerique;
import objets.CElementGraphique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointAncetrePointsMobiles;
import objets.CPointBaseEnt;
import objets.CPointLieDroite;
import objets.CPrototype;
import objets.CResultatValeur;
import objets.CSegment;
import objets.CValeurAffichee;
import objets.CValeurAssocieeAVariable;
import objets.CValeurDynamique;
import types.MotifPoint;
import types.NatureObjet;
import types.StyleEncadrement;
import dialogs.CurseurDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilCurseur extends Outil {
	CurseurDlg dlg;
	private CPointAncetre point1;
	public static String nomCurseur = "MathGraph32 Cursor"; // Ajout version 4.5
	
	/** Constructeur */
	public OutilCurseur(MtgFrame frameInit) {
		super(frameInit, true); // True pour gestion clignotement
	}
	
  @Override
	public void select() {
		super.select();
    point1 = null;
    frame.outilPointageActif = frame.outilPointageCreationCurseur;
    frame.outilPointageActif.designationDe(NatureObjet.NToutPoint);
    frame.indication(getString("curseur1"));
	}

	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (point1 == null) {
      point1 = (CPointAncetre) elg;
      ajouteClignotementDe(elg);
      CListeObjets liste = frame.cadreActif().listePr;
      // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
      // pour matrialiser l'emplacement de l'affichage
      int x = pointClique.x;
      int y = pointClique.y;
      CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 7d, -6d, false, null, 2,
      		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch165"),
  				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
      comClignotant.positionne(false, frame.cadreActif().dimf);
      ajouteClignotementDe(comClignotant);
      resetClignotement();
      frame.cadreActif().videDesignation();
    	dlg = new CurseurDlg(frame);
  		dlg.setLocationRelativeTo(frame);
    	dlg.setVisible(true);
    	if (dlg.retourOK()) creeObjet();
    	else {
    		frame.detruitDernierElement();
    		frame.activeOutilCapture();
    	}	
    }
  }
  
  @Override
  public void creeObjet() {
  	CValeurDynamique valeurMini; // La valeur qui sera associ au deuxime objet source
  	CValeurDynamique valeurMaxi; // La valeur qui sera associ au troisime objet source
  	
  	MtgInternalFrame cadre = frame.cadreActif();
  	cadre.videDesignation();
  	CListeObjets liste = cadre.listePr;
  	// On regarde d'abord si pour les valeurs mini et maxi l'utilisateur a fait rfrence  une valeur
  	// existante ou entr un calcul
  	CValeurAssocieeAVariable valMin =dlg.valMin;
  	CValeurAssocieeAVariable valMax =dlg.valMax;
  	if (valMin.calcul.getClass() != CResultatValeur.class) {
  		String nomMin = liste.genereNomPourCalcul("min", false);
  		CCalcul calculMin = new CCalcul(liste, null, false, nomMin, valMin.calcul.chaineCalculSansPar(null), valMin.calcul);
  		frame.ajouteElement(calculMin);
  		valeurMini = calculMin;
  	}
  	else {
  		valeurMini = ((CResultatValeur)valMin.calcul).valeurAssociee;
  	}
  	if (valMax.calcul.getClass() != CResultatValeur.class) {
  		String nomMax = liste.genereNomPourCalcul("max", false);
  		CCalcul calculMax = new CCalcul(liste, null, false, nomMax, valMax.calcul.chaineCalculSansPar(null), valMax.calcul);
  		frame.ajouteElement(calculMax);
  		valeurMaxi = calculMax;
  	}
  	else {
  		valeurMaxi = ((CResultatValeur)valMax.calcul).valeurAssociee;
  	}
  	// On regarde quel est le prototype  implmenter (ils ont t chargs dans le constructeur du MtgFrame)
  	CPrototype proto = null;
  	if (dlg.radioHorizontal.isSelected()) {
  		if (dlg.checkValeursEntieres.isSelected()) proto = frame.constructionCurseurEntierHorizontal;
  		else proto = frame.constructionCurseurHorizontal;
  	}
  	else {
    	if (dlg.radioVertical.isSelected()) {
    		if (dlg.checkValeursEntieres.isSelected()) proto = frame.constructionCurseurEntierVertical;
    		else proto = frame.constructionCurseurVertical;
    	}
  	}
  	//On implmente maintenant ce prototype dans la figure
		((CElementGenerique)proto.get(0)).elementAssocie = valeurMini;
		((CElementGenerique)proto.get(1)).elementAssocie = valeurMaxi;
		((CElementGenerique)proto.get(2)).elementAssocie = point1;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre);
		// On modifie le nom de l'implmentation (pour internationalisation)
		// Modifi version 4.5 : On garde le mme nom pour l'implmentation de prototype pour pouvoir
		// lors du zoom dtecter les droites  direction fixe qui ne doivent pas tre traites.
		/*
  	if (dlg.radioHorizontal.isSelected()) {
  		if (dlg.checkValeursEntieres.isSelected()) impProto.nomProto = getString("curseurEntierHorizontal");
  		else impProto.nomProto = getString("curseurHorizontal");
  	}
  	else {
    	if (dlg.radioVertical.isSelected()) {
    		if (dlg.checkValeursEntieres.isSelected()) impProto.nomProto = getString("curseurEntierVertical");
    		else impProto.nomProto = getString("curseurVertical");
    	}
  	}
  	*/
		// Version 4.5 :
		impProto.nomProto = nomCurseur;
		// On modifie le dernier objet implment qui est un affichage de valeur li  un point
		// suivant les choix de la bote de dialogue
		CValeurAffichee affval = (CValeurAffichee) liste.get(liste.size() - 1);
		affval.enTete = dlg.editeurPreChaine.getText();
		affval.postChaine = dlg.editeurPostChaine.getText();
		affval.couleurFondAffichage = cadre.couleurFond;
		affval.effacementFond = true;
		CCalcul calc = (CCalcul) liste.get(liste.size() - 2);
		calc.donneNom(dlg.editeurNom.getText());
		// On modifie le style de trait et de couleur des objets finaux
		Color col = frame.couleurActive();
		int styleTrait = frame.styleTraitActif();
		int indImpProto = liste.indexOf(impProto);
		if (dlg.checkExtremitesPunaisees.isSelected()) {
			if ((point1.getNature() & NatureObjet.NPointMobile) != 0) 
				((CPointAncetrePointsMobiles) point1).pointLibre = false;
		}
		if (dlg.checkValeursEntieres.isSelected()) {
			CPointLieDroite pt = (CPointLieDroite) (liste.get(indImpProto + 2));
			pt.donneCouleur(col);
			pt.pointLibre = !dlg.checkExtremitesPunaisees.isSelected();
			CSegment seg = (CSegment) (liste.get(indImpProto + 3));
			seg.donneCouleur(col);
			seg.donneStyle(styleTrait);
			CPointBaseEnt ptb = (CPointBaseEnt) (liste.get(indImpProto + 11));
			ptb.donneCouleur(col);
			ptb.donneMotif(MotifPoint.rond);
			CValeurAffichee valaff = (CValeurAffichee) (liste.get(indImpProto + 14));
			valaff.donneCouleur(col);
		}
		else {
			CPointLieDroite pt;
			pt= (CPointLieDroite) (liste.get(indImpProto + 2));
			pt.donneCouleur(col);
			pt.pointLibre = !dlg.checkExtremitesPunaisees.isSelected();
			CSegment seg = (CSegment) (liste.get(indImpProto + 7));
			seg.donneCouleur(col);
			seg.donneStyle(styleTrait);
			CPointLieDroite ptlie = (CPointLieDroite) (liste.get(indImpProto + 8));
			ptlie.donneCouleur(col);
			CValeurAffichee valaff = (CValeurAffichee) (liste.get(indImpProto + 11));
			valaff.donneCouleur(col);			
		}
		// Modification version 4.7.2 : Si on a demand un curseur de grande taille, on multiplie l'abscisse du point li par 2
		if (dlg.radioGdeTaille.isSelected()) {
			CPointLieDroite pt = (CPointLieDroite) (liste.get(indImpProto + 2));
			pt.donneAbscisse(pt.abscisse*2);
		}
		if (!dlg.checkValeursEntieres.isSelected()) affval.nombreDecimales = dlg.paneDecimales.nombreDecimales();
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil123"));
    cadre.repaintFigureAvecCalcul();
    frame.activeOutilCapture();
		
  }
	  
}
