/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;
import dialogs.MacroApparitionDlg;
import objets.CAffichageLieAPoint;
import objets.CCommentaire;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMacroApparition;
import objets.CSousListeObjets;
import types.NatureObjet;
import types.StyleEncadrement;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilMacroApparition extends Outil {
  private boolean dialogueRempli;
  private CSousListeObjets sousListe;
  private CMacroApparition macroapp;
  
  public OutilMacroApparition(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public Cursor curseurUtilise() {
		if (dialogueRempli) return MtgFrame.crossHairCursor;
		else return MtgFrame.cornerCursor;
	}

  @Override
	public void select() {
    super.select();
    frame.outilPointageActif = frame.outilPointageClic;
    frame.indication(getString("macro1"));
    dialogueRempli = false;
  }
	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    int x = pointClique.x;
    int y = pointClique.y;
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    if (!dialogueRempli) {
	    // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
	    // pour matrialiser l'emplacement de l'affichage
	    CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
	    comClignotant.positionne(false, frame.cadreActif().dimf);
	    ajouteClignotementDe(comClignotant);
	    resetClignotement();
	    macroapp = new CMacroApparition(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, 2, StyleEncadrement.Effet3D, false, cadre.couleurFondAffichage, "", "", null, false,
	    		CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
	    frame.indication("");
	    MacroApparitionDlg dlg = new MacroApparitionDlg(frame, macroapp, false);
			dlg.setLocationRelativeTo(frame);
	    dlg.setVisible(true);
	    if (dlg.retourOK()) {
	    	dialogueRempli = true;
	    	sousListe = new CSousListeObjets();
	    	frame.outilPointageActif = frame.outilPointageCre;
	    	frame.outilPointageActif.reset();
	    	frame.outilPointageActif.designationDe(NatureObjet.NToutObjet);
	    	frame.indication(getString("macroApparition1"));
	    }
	    else {
	    	annuleClignotement();
	    	frame.activeOutilCapture();
	    }
    }
    else {
    	frame.listeClignotante.add(elg);
    	frame.listeExclusionDesignation.add(elg);
    	sousListe.add(elg);
    }
  }
  
  @Override
	public void creeObjetApresClicDroit() {
		if (sousListe.size() == 0) {
			frame.dialogueAvertissement(getString("ch88"));
			frame.activeOutilCapture();
		}
		else {
			macroapp.listeAssociee = sousListe;
    	annuleClignotement();
			if (frame.ajouteElementAvecVerification(macroapp)) {
			  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil88"));
			  frame.cadreActif().repaintFigureAvecCalcul();
			}
			frame.activeOutilCapture();	
		}
	}	
}
