/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;
import java.util.ListIterator;

import objets.*;
import types.NatureObjet;
import mathgraph32.MtgFrame;

/**
 * Outil de mesure de l'abscisse d'un point sur une droite
 * @author Yves
 */
public class OutilMesAbs extends OutilPar3pts {

  /** Constructor */
  public OutilMesAbs(MtgFrame frameInit) {
    super(frameInit);
  }

  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soient nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  @Override
  boolean nomsPointsIndisp() {
    return true;
  }
  
  /**
   * Action aprs dsignation du deuxime point
   * Ne fera rien et ne sera redfini qu pour l'outil de mesure d'ascisse
   */
  @Override
  public void actionApresPoint2() {
    // On ajoute  la liste clignotante les points qui sont aligns
    // avec point1 et point2
    CListeObjets list = frame.cadreActif().listePr;
    ListIterator<CElementBase> li = list.listIterator();
    while(li.hasNext()) {
      CElementBase elb = li.next();
      if ((elb.getNature() & NatureObjet.NToutPoint) != 0) {
        CPointAncetre pt = (CPointAncetre) elb;
        if (glob.Kernel.aligne(point1.x, point1.y,point2.x, point2.y, pt.x, pt.y))
                ajouteClignotementDe(pt);
      }
    }
    frame.outilPointageObjetClignotant.designationDe(NatureObjet.NToutPoint); // Ncessaire car on change d'outil de pointage
    frame.outilPointageActif = frame.outilPointageObjetClignotant; 
}
  
  @Override
  public void creeObjet() {
    CMesureAbscisse mes = new CMesureAbscisse
            (frame.cadreActif().listePr, null, false , point1, point2, point3);
    if (frame.ajouteElementAvecVerification(mes)) {
      objetCree = true; // Necessaire  cause du fait que point1 et point2 ont t ventuellement
      // nomms et que si on ne cre pas l'objet on remet alors leur nom  une chaine vide    
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil15"));
      frame.dialogueInformation(getString("ch30") + " " + mes.nom() + " " + getString("ch31") + " "
            + point3.rendNom() + " " + getString("ch32") 
            + " (" + point1.rendNom() + "," + point2.rendNom() + ") " + getString("ch26"));
    }
    annuleClignotement();
    select();
  }
  
/**
   * Fonction renvoyant la chane  afficher  l'tape i (i >= 1) comme indication
   * Doit tre redfinie par les descendants devant crire plusieurs indications
   * @param i : l'indice
   * @return
   */
  @Override
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("mesAbs1");
      case 2 : return getString("mesAbs2");
      case 3 : return getString("mesAbs3");   
      default: return "";
    }
  }
}
