/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Color;
import objets.*;
import types.MotifPoint;
import types.StyleEncadrement;
import types.StyleTrait;
import types.UniteAngle;
import types.VecteurDirecteur;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 *
 * @author Yves
 */
public class OutilMesAngOr extends OutilPar3pts {
  
  /** Constructor */
  public OutilMesAngOr(MtgFrame frameInit) {
    super(frameInit);
  }

  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soient nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  @Override
  boolean nomsPointsIndisp() {
    return true;
  }
  
  @Override
  public void creeObjet() {
  	CMesureAngleOriente mes;
  	MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
  	if (frame.pref_affichageMesuresNiveau[frame.pref_niveau] && (liste.uniteAngle == UniteAngle.Degre)) {
  		int nbObjetsInitiaux = liste.size();
  		CPrototype proto = frame.constructionMesureAngleOriente;
  		((CElementGenerique)proto.get(0)).elementAssocie = point1;
  		((CElementGenerique)proto.get(1)).elementAssocie = point2;
  		((CElementGenerique)proto.get(2)).elementAssocie = point3;
  		CImplementationProto impProto = new CImplementationProto(liste, proto);
  		impProto.implemente(liste, proto, cadre);
  		impProto.nomProto = getString("mesureAngle");  		
	    mes = new CMesureAngleOriente
      	(frame.cadreActif().listePr, null, false , point1, point2, point3);
  		// On regarde si une telle mesure d'angle a dj t cre
  		if (liste.existeDeja(mes, nbObjetsInitiaux - 1)) {	
				frame.dialogueAvertissement(getString("ch7"));
				frame.detruitNDerniersElements(liste.size() - nbObjetsInitiaux);
  		} else {
  			objetCree = true;
	  		CValeurAffichee affval = (CValeurAffichee) liste.get(liste.size() - 2);
	  		// affval.enTete = point1.rendNom() + point2.rendNom() + " = ";
	  		affval.couleurFondAffichage = cadre.couleurFond;
	  		affval.donneCouleur(frame.couleurActive());
	  		affval.indiceTaillePolice = 3; // Rajout version 4.6.3
	  		CMarqueAngleOriente arc = (CMarqueAngleOriente)liste.get(liste.size() - 1);
	  		arc.donneCouleur(frame.couleurActive());
	  		arc.donneStyle(frame.styleTraitActif());
	  		arc.donneStyleMarque(frame.styleMarqueAngleActif());
	  		arc.donneStyleFleche(frame.styleFlecheActif);
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil26"));
	      cadre.repaintFigureAvecCalcul();
  		}  		
  		
  	}
  	else {  	
	    mes = new CMesureAngleOriente
	            (frame.cadreActif().listePr, null, false , point1, point2, point3);
	    if (frame.ajouteElementAvecVerification(mes)) {
	      objetCree = true; // Necessaire  cause du fait que point1 et point2 ont t ventuellement
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil26"));
	    }
  	}
    annuleClignotement();
    frame.dialogueInformation(getString("ch29") + " " + mes.nom() + " " +  getString("ch28"));  	
    select();
  }

  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {    
    CListeObjets list = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    CMathGrafDoc doc = frame.cadreActif().document;
    // On cre deux segments en pointills
  	CSegment pts1 = new CSegment(list, null, false, point1, point2);
    frame.ajouteObjetVisuel(pts1);
  	CSegment pts2 = new CSegment(list, null, false, point2, pane.pointSouris);
    frame.ajouteObjetVisuel(pts2);
    // On cre une marque d'angle orient
    /* Modifi version 4.6.3
    CMarqueAngleGeometrique mar = new CMarqueAngleOriente(list, null, false,
            frame.couleurActive(), false, frame.styleTraitActif(), frame.styleMarqueAngleActif, 30,
            frame.styleFlecheActif(), point1, point2, pane.pointSouris);
    */
    CMarqueAngleGeometrique mar = new CMarqueAngleOriente(list, null, false,
            frame.couleurActive(), false, frame.styleTraitActif(), frame.styleMarqueAngleActif, 35,
            frame.styleFlecheActif(), point1, point2, pane.pointSouris);
    
    frame.ajouteObjetVisuel(mar);
    // On cre un point par abscisse sur le segment joignant point2 et point1 
    // une distance de 20 pixels de point2
    VecteurDirecteur vec = new VecteurDirecteur(point2, point1);
    double d = 17/vec.norme();
    CValeurAssocieeAVariable ab = new CValeurAssocieeAVariable(list, d);
    CPointParAbscisse pt1 = new CPointParAbscisse(list, null, false, Color.BLACK, 
            false, 0d, 0d, true, "", 2, MotifPoint.croix, point2, point1, ab, false);
    frame.ajouteObjetVisuel(pt1);
    // On cre un petit arc de cercle de centre point2, commenant par point 1
    // et dirig vers le pointeur souris
    CArcDeCercle arc = new CArcDeCercle(list, null, false, Color.BLACK,
            true, StyleTrait.traitFin, point2, pt1, pane.pointSouris, null);
    frame.ajouteObjetVisuel(arc);
    // On cre une bissectrice
    CBissectrice bis = new CBissectrice(list, null, false, Color.BLACK, StyleTrait.traitFin,
            point1, point2, pane.pointSouris, 2);
    bis.cache();
    frame.ajouteObjetVisuel(bis);
    // On cre l'intersection de l'arc et de la bissectrice
    CIntDroiteCercle inter = new CIntDroiteCercle(list, null, false, bis, arc);
    frame.ajouteObjetVisuel(inter);
    CPointLieBipoint ptl1 = new CPointLieBipoint(list, null, false, Color.BLACK,
            false, 0d, 0d, true, "", 2, MotifPoint.croix, inter, 1, false);
    frame.ajouteObjetVisuel(ptl1);
    CPointLieBipoint ptl2 = new CPointLieBipoint(list, null, false, Color.BLACK,
            false, 0d, 0d, true, "", 2, MotifPoint.croix, inter, 2, false);
    frame.ajouteObjetVisuel(ptl2);
    // Un des deux points d'intersection n'existera pas mais on ne sait pas lequel
    // On cre un commentaire contenant un point d'intersection li  chacun d'eux
  	if (frame.pref_affichageMesuresNiveau[frame.pref_niveau] && (list.uniteAngle == UniteAngle.Degre)) {
    	CMesureAngleOriente mes = new CMesureAngleOriente(list, null, false, point1, point2, pane.pointSouris);
     	frame.ajouteObjetVisuel(mes);
     	/* Modifi version 4.6.3
    	CValeurAffichee affval1 = new CValeurAffichee(list, null, false, frame.couleurActive(),
    			0d, 0d, 0d, 0d, false, ptl1, 2, StyleEncadrement.Sans, false, doc.couleurFond, mes, "", "", 0,
					CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
			*/
    	CValeurAffichee affval1 = new CValeurAffichee(list, null, false, frame.couleurActive(),
    			0d, 0d, 0d, 0d, false, ptl1, 3, StyleEncadrement.Sans, false, doc.couleurFond, mes, "$", "$", 0,
					CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
    	frame.ajouteObjetVisuel(affval1);
    	/* Modifi version 4.6.3
    	CValeurAffichee affval2 = new CValeurAffichee(list, null, false, frame.couleurActive(),
    			0d, 0d, 0d, 0d, false, ptl2, 2, StyleEncadrement.Sans, false, doc.couleurFond, mes, "", "", 0,
					CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
			*/
    	CValeurAffichee affval2 = new CValeurAffichee(list, null, false, frame.couleurActive(),
    			0d, 0d, 0d, 0d, false, ptl2, 3, StyleEncadrement.Sans, false, doc.couleurFond, mes, "$", "$", 0,
					CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
    	frame.ajouteObjetVisuel(affval2);    	
    }
    else {
	    CCommentaire com1 = new CCommentaire(list, null, false, Color.BLACK, 0d, 0d,
	                0d, -0d, false, ptl1, 4, StyleEncadrement.Sans, false, doc.couleurFond, "?",
	      					CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
	    frame.ajouteObjetVisuel(com1);
	    CCommentaire com2 = new CCommentaire(list, null, false, Color.BLACK, 0d, 0d,
	                0d, 0d, false, ptl2, 4, StyleEncadrement.Sans, false, doc.couleurFond, "?",
	      					CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
	     frame.ajouteObjetVisuel(com2);
    }
  }
  
/**
   * Fonction renvoyant la chane  afficher  l'tape i (i >= 1) comme indication
   * Doit tre redfinie par les descendants devant crire plusieurs indications
   * @param i : l'indice
   * @return
   */          
  @Override
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("mesAngOr1");
      case 2 : return getString("mesAngOr2");
      case 3 : return getString("mesAngOr3");
      
      default: return "";
    }
  }
  
}
