/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.util.ListIterator;
import dialogs.MesureLongueurLigneDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CAffichageLieAPoint;
import objets.CElementGraphique;
import objets.CLignePolygonaleAncetre;
import objets.CListeObjets;
import objets.CMesureLongueurLigne;
import objets.CNoeudPointeurSurPoint;
import objets.CPointAncetre;
import objets.CValeurAffichee;
import types.NatureObjet;
import types.StyleEncadrement;

public class OutilMesureLongueurLigne extends Outil {
	CLignePolygonaleAncetre ligne;

	public OutilMesureLongueurLigne(MtgFrame frameInit) {
		super(frameInit, true);
	}
	
  @Override
	public void select() {
		super.select();
		ligne = null;
		frame.outilPointageActif = frame.outilPointageCre;
		frame.outilPointageActif.designationDe(NatureObjet.NPolygone | NatureObjet.NLigneBrisee);
		frame.indication(getString("mesureLongueurLigne1"));
    frame.outilPointageActif.reset();
	}
	
  @Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (ligne == null) {
      ligne = (CLignePolygonaleAncetre) elg;
      creeObjet();
    }
	}
	
  @Override
	public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		cadre.videDesignation();
		CListeObjets liste = cadre.listePr;
		CMesureLongueurLigne mesure = new CMesureLongueurLigne(liste,null, false, ligne, "");
		MesureLongueurLigneDlg dlg = new MesureLongueurLigneDlg(frame, mesure);
		dlg.setLocationRelativeTo(frame);
		dlg.setVisible(true);
		if (dlg.retourOK()) {
			// On nomme les points de la ligne qui n'ont pas de nom
			ListIterator<CNoeudPointeurSurPoint> li = ligne.collectionPoints.listIterator();
			while (li.hasNext()) {
				CNoeudPointeurSurPoint noeud = li.next();
				CPointAncetre point = noeud.pointeurSurPoint;
				if (point.rendNom().length() == 0) {
					point.donneNom(frame.cadreActif().listePr.genereNomPourPoint());
				}
			}
      if (frame.ajouteElementAvecVerification(mesure))
    		if (frame.pref_affichageMesuresNiveau[frame.pref_niveau]) {
     			CValeurAffichee affval = new CValeurAffichee(liste, null, false, frame.couleurActive(),
    					0d, 0d, 0d, 6d, false, ligne.collectionPoints.get(0).pointeurSurPoint, frame.pref_IndiceTaillePoliceAffichages,
    					StyleEncadrement.Sans, true, cadre.couleurFond, mesure, mesure.nom() + " = ", "", 1,
    					CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalHaut);
     			frame.ajouteElement(affval);
     			cadre.repaintFigureAvecCalcul();
     		}
        cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil112"));
		}
		select();
		
	}
	
  @Override
  public boolean activationValide() {
    return (frame.cadreActif().listePr.pointeurLongueurUnite != null);
  }
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeLongueurUnite");
	}	
	
}
