/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import dialogs.ProduitScalaireDlg;
import mathgraph32.MtgFrame;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CProduitScalaire;
import objets.CVecteur;
import types.NatureObjet;

public class OutilMesureProduitScalaire extends Outil {
	private CVecteur vecteur1;
	private CVecteur vecteur2;
	
	public OutilMesureProduitScalaire(MtgFrame frameInit) {
		super(frameInit, true); // Avec clignotement donc true
	}
	
  @Override
	public void select() {
		super.select();
		vecteur1 = null;
		vecteur2 = null;
		frame.outilPointageActif = frame.outilPointageCre;
		frame.outilPointageActif.designationDe(NatureObjet.NVecteur);
		frame.indication(getString("produitScalaire1"));
    frame.outilPointageObjetClignotant.reset();
	}

  @Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (vecteur1 == null) {
      vecteur1 = (CVecteur) elg;
  		frame.indication(getString("produitScalaire2"));
  		ajouteClignotementDe(elg);
  		resetClignotement();
    }
    else {
    	vecteur2 = (CVecteur) elg;
    	ajouteClignotementDe(vecteur2);
      creeObjet();
    }
	}
	
  @Override
  public void creeObjet() {
  	CListeObjets liste = frame.cadreActif().listePr;
  	frame.indication("");
    // On redessine la figure pour liminer la dsignation ce vecteur affiche pour le deuxime vecteur
    frame.cadreActif().videDesignation();  	
  	// O affiche une bote de dialogue pour demander le nom de la mesure
  	ProduitScalaireDlg dlg = new ProduitScalaireDlg(frame);
		dlg.setLocationRelativeTo(frame);
  	dlg.setVisible(true);
  	if (dlg.retourOK()) {
	    CProduitScalaire prod = new CProduitScalaire(liste, null, false ,
	    		dlg.editeurNom.getText(), vecteur1, vecteur2);
	    if (frame.ajouteElementAvecVerification(prod)) {
	    	// Il faut que les extrmits des vecteurs soient nommes
	    	// Attention : les deux vecteurs peuvent avoir des extrmits en commun
	    	if (vecteur1.point1.rendNom().length() == 0) vecteur1.point1.donneNom(liste.genereNomPourPoint());
	    	if (vecteur1.point2.rendNom().length() == 0) vecteur1.point2.donneNom(liste.genereNomPourPoint());
	    	if (vecteur2.point1.rendNom().length() == 0) vecteur2.point1.donneNom(liste.genereNomPourPoint());
	    	if (vecteur2.point2.rendNom().length() == 0) vecteur2.point2.donneNom(liste.genereNomPourPoint());
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil131"));
	    }
  	}
    annuleClignotement();
    select();
  }
  
  @Override
  public boolean activationValide() {
  	CListeObjets liste = frame.cadreActif().listePr;
    return (liste.pointeurLongueurUnite != null) && liste.nombreObjetsParNatureVisibles(NatureObjet.NVecteur)>0;
  }
  
	@Override
	public String messageActivationNonValide() {
  	CListeObjets liste = frame.cadreActif().listePr;
		if (liste.pointeurLongueurUnite == null) return getString("messagePasDeLongueurUnite");
		else return getString("messagePasDeVecteur");
	}
	
	
  
}
