/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;

import objets.CElementGraphique;
import objets.CElementLigne;
import objets.CGrapheSuiteRecComplexe;
import objets.CLieuDiscretDeBase;
import objets.CMarqueAngleAncetre;
import objets.CMarqueAngleOriente;
import objets.CMarqueSegment;
import objets.CPointAncetre;
import objets.CSurfaceAncetre;
import objets.CVecteur;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilPalette extends Outil {
	CElementGraphique objetDesigne;

	/** Creates a new instance of OutilPtLib */
	public OutilPalette(MtgFrame frameInit) {
		super(frameInit, false);
	}

  @Override
	public void select() {
		super.select();
		frame.outilPointageActif = frame.outilPointageCre;
		frame.outilPointageActif.designationDe(NatureObjet.NToutObjetSaufRepere);
		frame.indication(getString("palette1"));
	}

	@Override
  public Cursor curseurUtilise() {
		return MtgFrame.paletteCursor;
	}
	
	/*
	 * Appel lorsqu'un objet a t cliqu pour tre trait par l'outil par
	 * exemple le premier point pour crer une droite passant par deux points
	 */
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		objetDesigne = elg;
		creeObjet();
	}

  @Override
	public void creeObjet() {
		boolean objetChange = (frame.couleurActive() != objetDesigne.couleur);
		objetDesigne.donneCouleur(frame.couleurActive());
		long nat = objetDesigne.getNature();
		if ((nat & (NatureObjet.NSurface | NatureObjet.NDemiPlan)) != 0) {
			CSurfaceAncetre surf = (CSurfaceAncetre) objetDesigne;
			objetChange = objetChange
					|| (surf.styleRemplissage != frame.styleRemplissageActif());
			if (objetChange) surf.styleRemplissage = frame.styleRemplissageActif();
		} else {
			if ((nat & NatureObjet.NToutPoint) != 0) {
				CPointAncetre pt = (CPointAncetre) objetDesigne;
				objetChange = objetChange || (pt.motif != frame.stylePointActif());
				pt.motif = frame.stylePointActif();
			}
			else {
				if (nat == NatureObjet.NLieuDiscret) {
					CLieuDiscretDeBase pt = (CLieuDiscretDeBase) objetDesigne;
					objetChange = objetChange || (pt.motif != frame.stylePointActif());
					pt.motif = frame.stylePointActif();
				}
				else {
					if (nat == NatureObjet.NMarqueSegment) {
						CMarqueSegment pt = (CMarqueSegment) objetDesigne;
						objetChange = objetChange || (pt.motif != frame.styleMarqueSegmentActif())
							|| (pt.style != frame.styleTraitActif());
						pt.motif = frame.styleMarqueSegmentActif;
						pt.style = frame.styleTraitActif();		
					}
					else {
						if (nat == NatureObjet.NGrapheSuiteRecComplexe) {
							CGrapheSuiteRecComplexe pt = (CGrapheSuiteRecComplexe) objetDesigne;
							objetChange = objetChange || (pt.motif != frame.stylePointActif())
								|| (pt.style != frame.styleTraitActif());
							pt.motif = frame.stylePointActif;
							pt.style = frame.styleTraitActif();
						}
						else {
							if ((nat & NatureObjet.NVecteur) != 0) {
								CVecteur vect = (CVecteur) objetDesigne;
								objetChange = objetChange || (vect.motifFleche != frame.styleFlecheActif());
								vect.donneStyleFleche(frame.styleFlecheActif());
							}
							else {
								if ((nat & NatureObjet.NMarqueAngle) != 0) {
									CMarqueAngleAncetre marque = (CMarqueAngleAncetre) objetDesigne;
									objetChange = objetChange || (marque.styleMarque != frame.styleMarqueAngleActif);
									marque.donneStyleMarque(frame.styleMarqueAngleActif());
									if (marque.estOriente()) {
										CMarqueAngleOriente marqueOr = (CMarqueAngleOriente) marque;
										objetChange = objetChange || (marqueOr.motifFleche != frame.styleFlecheActif());
										marqueOr.donneStyleFleche(frame.styleFlecheActif());
									}
								}
							}
						}
					}							
				}
			}
		}
		if ((nat & NatureObjet.NObjetLigne) != 0) {
			CElementLigne lig = (CElementLigne) objetDesigne;
			objetChange = objetChange || (lig.style != frame.styleTraitActif());
			lig.donneStyle(frame.styleTraitActif());
		}
		MtgInternalFrame cadre = frame.cadreActif();
		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil32"));
		if (objetChange) {
		  cadre.listePr.metAJourObjetsDependantDe(objetDesigne); // Ncessaire pour les lieux d'objets
		  cadre.repaintFigureAvecCalcul();
		}
		select();
	}
}
