/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Color;

import objets.CCercleOA;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointImage;
import objets.CPrototype;
import objets.CSegment;
import objets.CTranslation;
import types.StyleTrait;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilParallelogramme extends OutilPar3pts {
  /** Creates a new instance of OutilCarreDirect */
  public OutilParallelogramme(MtgFrame frameInit) {
    super(frameInit);
  }

  @Override
  public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
		CPrototype proto = frame.constructionParallelogrammeParConst;
		((CElementGenerique)proto.get(0)).elementAssocie = point1;
		((CElementGenerique)proto.get(1)).elementAssocie = point2;
		((CElementGenerique)proto.get(2)).elementAssocie = point3;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre);
		impProto.nomProto = getString("parallelogramme");  				
		CPointAncetre pt1 = (CPointAncetre) liste.get(liste.size() - 1);
		pt1.donneCouleur(frame.couleurActive());
		pt1.donneMotif(frame.stylePointActif);
		pt1.donneNom(frame.genereNomPourPoint()); // Ajout version 4.7.0.1
		CSegment seg1 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), point1, point2);
		if (!liste.existeDeja(seg1)) liste.add(seg1);
		CSegment seg2 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), point2, point3);
		if (!liste.existeDeja(seg2)) liste.add(seg2);
		CSegment seg3 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), point3, pt1);
		if (!liste.existeDeja(seg3)) liste.add(seg3);
		CSegment seg4 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), pt1, point1);
		if (!liste.existeDeja(seg4)) liste.add(seg4);
    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil141"));
    annuleClignotement();
    cadre.repaintFigureAvecCalcul();    
    select();
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
		CListeObjets liste = frame.listeObjetsVisuels;
  	CSegment seg1 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), point1, point2);
  	frame.ajouteObjetVisuel(seg1);
  	CTranslation trans = new CTranslation(liste, null, false, point2, point1);
  	frame.ajouteObjetVisuel(trans);
  	CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(), frame.stylePointActif(),
  			frame.pref_indiceTaillePoliceNom, pane.pointSouris, trans);
  	frame.ajouteObjetVisuel(ptim);
  	CCercleOA cerc = new CCercleOA(liste, null, false, Color.black, true,
  			StyleTrait.traitFin, point1, pane.pointSouris);
  	frame.ajouteObjetVisuel(cerc);
  	CSegment seg2 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), point2, pane.pointSouris);
  	frame.ajouteObjetVisuel(seg2);
  	CSegment seg3 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), pane.pointSouris, ptim);
  	frame.ajouteObjetVisuel(seg3);
  	CSegment seg4 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), ptim, point1);
  	frame.ajouteObjetVisuel(seg4);
  }
  
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("parallelogramme1");
      case 2 : return getString("parallelogramme2");
      case 3 : return getString("parallelogramme3");
      default: return "";
    }
  }
  
}
