/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author Yves
 * Outil servant  dsigner des objets et  les fournir  l'outil de cration
 * d'objet en cours
 */
package outils;

import java.awt.event.MouseEvent;
import objets.*;
import types.*;
import dialogs.*;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilPointageCre extends OutilPointage {
  
  /** Creates a new instance of OutilPointageCre */
  public OutilPointageCre(MtgFrame frameInit) {
    super(frameInit);
  }
  
  /**
   * Renvoie la liste  utiliser.
   * Devra tre redfini pour l'outil de suppresion d'objet
   * qui ne permettra de supprimer que les objets de la liste
   * listeConstruction
   */
  protected CListeObjets listeUtilisee() {
    return frame.cadreActif().listePr;
  }
  
  
  public void mouseMoved(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
    int x = mouseEvent.getX();
    int y = mouseEvent.getY();
    CListeObjets liste = listeUtilisee();
    InfoProx info = frame.infoProx;
    int nbObjetsProches = liste.procheDe(aDesigner, mouseEvent.getPoint(), info, 
      frame.listeExclusionDesignation, !cadre.ctlKeyPressed);
    int nbTypesProches = info.nombreTypesProches;
    boolean designationDejaAffichee = (cadre.commentaireDesignation().chaineCommentaire.length() != 0);
      // true si une dsignation est dj affiche prs du pointeur
    if (nbObjetsProches > 0) {
        String ch = "";
      // Si lors d'un dialogue de choix d'objet proche ona dj choisi un objet
      // et si celui-ci est proche du pointeur souris, c'est celui-l qui sera slectionnn
      if (!cadre.ctlKeyPressed && (info.premierPointVoisin != null)) {
        ch = getString("designationPoint");
      }
      else {
        if (nbTypesProches == 1) {
          for (int i =0; i < NatureObjet.nombreTypesDesignables; i++) {
            if (info.infoParType[i].nombreVoisins > 0) {
              ch = info.infoParType[i].premierVoisin.chaineDesignation();
              break;
            }
          }
        }
        else {
          ch = getString("designationAPreciser");
        }
      }
      cadre.commentaireDesignation().donneChaine(ch);
      cadre.commentaireDesignation().placeEn(x, y);
      cadre.paneFigure.repaint();
    }
    else {
      setActiveCursor(MtgFrame.crossHairCursor);
      frame.cadreActif().videDesignation();
      if ((frame.listeObjetsVisuels.size() > 0) || designationDejaAffichee) {
        cadre.paneFigure.repaint();
      }
    }
  }
  public void mousePressed(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
  	if (mouseEvent.getButton() == MouseEvent.BUTTON1) {
	    InfoProx info = frame.infoProx;
	    if (info.nombreObjetsProches != 0) {
	      CElementGraphique elg;
	      if (info.nombreTypesProches == 1) {
	        int i;
	        for (i =0; i < NatureObjet.nombreTypesDesignables; i++) {
	          if (info.infoParType[i].nombreVoisins > 0) break;
	        }
	        // i contient l'indice du type d'objets proches
	        if (info.infoParType[i].nombreVoisins >= 2) {
	          // On ouvre un dialogue pour demander de choisir entre le premier et le dernier objet
	          PremierDernierDlg dlg = new PremierDernierDlg(frame);
	      		dlg.setLocationRelativeTo(frame);
	          dlg.setVisible(true);
	          if (!dlg.retourOK()) return;
	          // On regarde quelle a t la rponse de l'utilisateur
	          if (dlg.getPremierChoisi()) elg = info.infoParType[i].premierVoisin;
	          else elg = info.infoParType[i].dernierVoisin;
	        }
	        else {
	          elg = info.infoParType[i].premierVoisin;
	        }
	      }
	      else {
	        // Il faut d'abord ouvrir un dialogue demandant le type de l'objet
	        // que l'utilisateur souhaite dsigner
	        ChoixTypeDlg dlgct = new ChoixTypeDlg(frame, info.typesProches);
	    		dlgct.setLocationRelativeTo(frame);
	        dlgct.setVisible(true);
	        if (!dlgct.retourOK()) return;    
	        int indType = dlgct.indiceTypeChoisi;
	        // Il faut regarder si dans le type choisi un seul objet est proche ou plusieurs
	        if (info.infoParType[indType].nombreVoisins == 1) {
	          elg = info.infoParType[indType].premierVoisin;
	        }
	        else { // Plusieurs objets du type choixi sont proches du curseur
	          PremierDernierDlg dlg = new PremierDernierDlg(frame);
	      		dlg.setLocationRelativeTo(frame);
	          dlg.setVisible(true);
	          if (!dlg.getRetourOK()) return;
	          // On regarde quelle a t la rponse de l'utilisateur
	          if (dlg.getPremierChoisi()) elg = info.infoParType[indType].premierVoisin;
	          else elg = info.infoParType[indType].dernierVoisin;        
	        }  
	      }
	      frame.outilActif.traiteObjetDesigne(elg, mouseEvent.getPoint());
	      return;
	    }
	  }
  	else  // Cas d'un appui sur le bouton droit. On finit alors la cration de l'objet
  		frame.outilActif.creeObjetApresClicDroit();
  }
}
