/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;
/*
 * Pinceau.java
 *
 * Created on 27 juin 2004, 18:35
 */

/**
 *
 * @author  Yves
 * @version 
 */
// Classe servant via ses mthodes de classe  crer des pinceaux de style de trait donn
// et de couleur donne
// Si basseDefinition est true on est en rsolution crna et sinon on est en rsolution mulstiplie par 100
// (10 dans chaque direction)
import java.awt.*;

import objets.CListeObjets;
public class Pinceau {
	public static final float[] floatTraitPointille = new float[] {3,3};
	public static final float[] floatTraitTrait = new float[] {10,10};
	public static final float[] floatTraitPointPoint = new float[] {10,3,3,3,3,3};
		
	private static final BasicStroke pinceauTraitFin = new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);;
	private static final BasicStroke pinceauTraitPointille = new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,floatTraitPointille,0f);
	private static final BasicStroke pinceauTraitEpais = new BasicStroke(2.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
	private static final BasicStroke pinceauTraitTrait = new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,floatTraitTrait,0f);
	private static final BasicStroke pinceauTraitPointPoint = new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,floatTraitPointPoint,0f);
	private static final BasicStroke pinceauTraitTresEpais = new BasicStroke(3.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
	private static final BasicStroke pinceauTraitPointilleEpais = new BasicStroke(2.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,floatTraitPointille,0f);

  /** Creates new Pinceau */
  public Pinceau() {
  }
  public static final BasicStroke CreePinceau(int styleTrait, CListeObjets liste) {
  	if ((liste == null) || liste.basseDefinition) {
	    switch (styleTrait) {
	      case StyleTrait.traitFin :
	        return pinceauTraitFin;
	      case StyleTrait.traitPointille :
	        return pinceauTraitPointille;
	      case StyleTrait.traitEpais :
	        return pinceauTraitEpais;
	      case StyleTrait.traitTrait :
	        return pinceauTraitTrait;
	      case StyleTrait.traitPointPoint :
	        return pinceauTraitPointPoint;
	      case StyleTrait.traitTresEpais :
	        return pinceauTraitTresEpais;
	      case StyleTrait.traitPointilleEpais :
	        return pinceauTraitPointilleEpais;
	      default : return pinceauTraitFin;
	    }
  	} 
  	else {
  		float coef = (float) liste.coefMult;
  		// Si demand dans les prfrences on divise l'paisseur des traits par 2 pour un coefficient multiplicateur au moins gal  2
  		if (liste.bAttenuationEpaisseurTrait) {
  			if (coef < 2) coef = 1;
  			else coef /= 2;
  		}
	    switch (styleTrait) {
      case StyleTrait.traitFin :
        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
      case StyleTrait.traitPointille :
        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {3*coef,3*coef},0f);
      case StyleTrait.traitEpais :
        return new BasicStroke(2*coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
      case StyleTrait.traitTrait :
        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {10*coef,10*coef},0f);
      case StyleTrait.traitPointPoint :
        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1 ,new float[] {10*coef,3*coef,3*coef,3*coef,3*coef,3*coef},0f);
      case StyleTrait.traitTresEpais :
        return new BasicStroke(3*coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
      case StyleTrait.traitPointilleEpais :
        return new BasicStroke(2*coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,new float[] {3*coef,3*coef},0f);
      default : return null;
    }
  	}
  }    
}
