/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;
import objets.CPointAncetre;

/**
 *
 * @author  Yves
 * @version 
 */
public class VecteurDirecteur {
  public double x, y;
  /** Creates new VecteurDirecteur */
  public VecteurDirecteur() {
  }
  /** Constructeur  partir d'un point origine et d'un point extrmit **/
  public VecteurDirecteur(double xor, double yor, double xex, double yex) {
    x = xex - xor;
    y = yex - yor;
  }
  // Constructeur  partir de deux points
  public VecteurDirecteur(CPointAncetre origine, CPointAncetre extremite) {
    x = extremite.x - origine.x;
    y= extremite.y - origine.y;
  }
  public final void setCopy(VecteurDirecteur v) {
    v.x = this.x;
    v.y = this.y;
  }
  public final void setVecteur(CPointAncetre point1, CPointAncetre point2) {
    x = point2.x - point1.x;
    y = point2.y - point1.y;
  }
  public final void setVecteur(double x1, double y1, double x2, double y2) {
    x = x2 - x1;
    y = y2 - y1;
  }
  public final double norme() {
  	return Math.sqrt(x*x+y*y);
  }
  public final boolean nonNul() {
    return !Nul();  
  }
  public final boolean Nul() {
  	return (x == 0) && (y == 0);
  }
  public final boolean presqueNul() {
  	return glob.Kernel.zero(x) && glob.Kernel.zero(y);
  }
  public final void getOrthogonal(VecteurDirecteur v) {
    v.y = this.x;
    v.x = -this.y;
  }
  public final void vecteurColineaire(double normeSouhaitee, VecteurDirecteur v)
  {
      double norm = norme();
      v.x = this.x/norm*normeSouhaitee;
      v.y = this.y/norm*normeSouhaitee;
  }
  // Renvoie la mesure en radians de l'angle polaire d'un vecteur non nul
  public final double angleRad()
  {
    double angle;

    if ((x == 0) && (y == 0)) return 0;
    else
    {
      if (x == 0) 
      {
        if (y > 0) 
        angle = 3*Math.PI/2;
        else angle = Math.PI/2;
      }
      else
      {
       if (x >= 0)
          angle = Math.atan(-y/x);
       else
       {
          angle = Math.PI-Math.atan(y/x);
       };
       if (angle < 0) angle = angle + 2*Math.PI;
      }
    }
    return angle;
  }
  // Fonction renvoyant la mesure en radians de l'angle orient
  // form par les vecteurs u et v, cette mesure tant valable
  // dans le repre observateur,  et tant l'oppose de celle
  // valable dans le systme de coordonnes utilis par Windows
  public final double mesureAngleVecteurs(VecteurDirecteur v)
  {
    double prosca,deter,tangente,resul;
    // L'orientation sur l'cran est l'inverse de celle du repre
    // choisi.Il faut donc permuter les rles de U et V
    prosca = v.x*x + v.y*y;
    deter = v.x*y - v.y*x;
    if (prosca == 0)
    {
      if (deter > 0)  resul = Math.PI/2;
      else resul = - Math.PI/2;
    }
    else
    {
      tangente = deter/prosca;
      if (prosca > 0) resul = Math.atan(tangente);
      else
      {
        if (deter >= 0) resul = Math.atan(tangente) + Math.PI;
        else resul = Math.atan(tangente) - Math.PI;
      }
    }
    return resul;
  }
  public final void tourne(double angle,VecteurDirecteur v)
  {
    double kc,ks;

    kc = Math.cos(angle);
    ks = Math.sin(angle);
    v.x = kc*x + ks*y;
    v.y = kc*y - ks*x;
  }
}
